@@ -1,7 +1,6 @@
-# -*- mode: python -*-
-
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -112,22 +111,102 @@ def confirm_db_thread():
         traceback.print_stack()
         raise DatabaseThreadError, error_string
 
-class View(object):
-    def __init__(self, klass, where, values, order_by, joins, limit):
+class ViewObjectFetcher(object):
+    """Interface for classes that handle retrieving objects for Views.
+
+    Views and ViewTrackers usually return DDBObject subclasses for their
+    methods, but sometimes we want to do things like get ItemInfo objects from
+    app.item_info_cache.  To handle this, View/ViewTrackers just work with
+    object ids and ViewObjectFetcher work to fetch objects for those object
+    ids.
+    """
+    def fetch_obj(self, id_):
+        """Fetch an object for a given id."""
+        raise NotImplementedError
+
+    def fetch_obj_for_ddb_object(self, id_):
+        """Fetch an object for a DDBObject."""
+        raise NotImplementedError
+
+    def table_name(self):
+        """Return the DB table to query from."""
+
+    def prepare_objects(self, id_list):
+        """Given a list of ids that we want to fetch objects for, do any prep
+        work needed to make sure that they are fetchable.
+
+        prepare_objects may modify id_list in place to remove some of the ids
+        """
+        pass
+
+class DDBObjectFetcher(ViewObjectFetcher):
+    def __init__(self, klass):
         self.klass = klass
+
+    def fetch_obj(self, id_):
+        return app.db.get_obj_by_id(id_)
+
+    def fetch_obj_for_ddb_object(self, ddb_object):
+        return ddb_object
+
+    def table_name(self):
+        return app.db.table_name(self.klass)
+
+    def prepare_objects(self, id_list):
+        if app.db.ensure_objects_loaded(self.klass, id_list):
+            # sometimes objects will call remove() in setup_restored().
+            # We need to filter those out.
+            new_id_list = [id_ for id_ in id_list if app.db.id_alive(id_)]
+            if len(new_id_list) < id_list:
+                id_list[:] = new_id_list # update id_list in-place
+
+class IDOnlyFetcher(ViewObjectFetcher):
+    """Fetcher that just emits the IDs of objects
+
+    This can be more efficient in some cases
+    """
+
+    def table_name(self):
+        return 'item'
+
+    def fetch_obj(self, id_):
+        return id_
+
+    def fetch_obj_for_ddb_object(self, item):
+        return item.id
+
+class View(object):
+    def __init__(self, fetcher, where, values, order_by, joins, limit):
+        self.fetcher = fetcher
+        self.table_name = fetcher.table_name()
         self.where = where
         self.values = values
         self.order_by = order_by
         self.joins = joins
         self.limit = limit
 
+    def _query(self):
+        id_list = self._query_ids()
+        self.fetcher.prepare_objects(id_list)
+        for id_ in id_list:
+            yield self.fetcher.fetch_obj(id_)
+
+    def _query_ids(self):
+        return list(app.db.query_ids(self.table_name, self.where, self.values,
+            self.order_by, self.joins, self.limit))
+
+    def _query_count(self):
+        return app.db.query_count(self.table_name, self.where, self.values,
+                self.joins, self.limit)
+
     def __iter__(self):
-        return app.db.query(self.klass, self.where, self.values,
-                            self.order_by, self.joins, self.limit)
+        return self._query()
+
+    def id_list(self):
+        return self._query_ids()
 
     def count(self):
-        return app.db.query_count(self.klass, self.where, self.values,
-                                  self.joins, self.limit)
+        return self._query_count()
 
     def get_singleton(self):
         results = list(self)
@@ -141,7 +220,7 @@ class View(object):
     def make_tracker(self):
         if self.limit is not None:
             raise ValueError("tracking views with limits not supported")
-        return ViewTracker(self.klass, self.where, self.values, self.joins)
+        return ViewTracker(self.fetcher, self.where, self.values, self.joins)
 
 class ViewTrackerManager(object):
     def __init__(self):
@@ -181,17 +260,18 @@ class ViewTrackerManager(object):
             tracker.remove_object(obj)
 
 class ViewTracker(signals.SignalEmitter):
-    def __init__(self, klass, where, values, joins):
-        signals.SignalEmitter.__init__(self, 'added', 'removed', 'changed')
-        self.klass = klass
+    def __init__(self, fetcher, where, values, joins):
+        signals.SignalEmitter.__init__(self, 'added', 'removed', 'changed',
+                'bulk-added', 'bulk-removed', 'bulk-changed')
+        self.fetcher = fetcher
+        self.table_name = fetcher.table_name()
         self.where = where
         if isinstance(values, list):
             raise TypeError("values must be a tuple")
         self.values = values
         self.joins = joins
-        self.current_ids = set(app.db.query_ids(klass, where, values,
-            joins=joins))
-        self.table_name = app.db.table_name(klass)
+        self.bulk_mode = False
+        self.current_ids = self._view_object_ids()
         vt_manager = app.view_tracker_manager
         vt_manager.trackers_for_table(self.table_name).add(self)
 
@@ -199,13 +279,29 @@ class ViewTracker(signals.SignalEmitter):
         vt_manager = app.view_tracker_manager
         vt_manager.trackers_for_table(self.table_name).discard(self)
 
+    def set_bulk_mode(self, bulk_mode):
+        """Set/Unset bulk mode.
+
+        Normally, ViewTrackers emit one added/removed/changed, signal per
+        object.  In bulk mode, they will sometimes also emit the
+        bulk-added/bulk-removed/bulk-changed when a list of objects changes.
+        """
+        self.bulk_mode = bulk_mode
+
     def _obj_in_view(self, obj):
+        """Check if a single object is in our view."""
         where = '%s.id = ?' % (self.table_name,)
         if self.where:
             where += ' AND (%s)' % (self.where,)
 
         values = (obj.id,) + self.values
-        return app.db.query_count(self.klass, where, values, self.joins) > 0
+        return app.db.query_count(self.table_name, where, values,
+                self.joins) > 0
+
+    def _view_object_ids(self):
+        """Get all object ids in our view."""
+        return set(app.db.query_ids(self.table_name,
+                                    self.where, self.values, joins=self.joins))
 
     def object_changed(self, obj):
         self.check_object(obj)
@@ -213,62 +309,95 @@ class ViewTracker(signals.SignalEmitter):
     def remove_object(self, obj):
         if obj.id in self.current_ids:
             self.current_ids.remove(obj.id)
-            self.emit('removed', obj)
+            self.emit('removed', self.fetcher.fetch_obj_for_ddb_object(obj))
 
     def remove_objects(self, objects):
-        object_map = dict((o.id, o) for o in objects)
-        object_ids = set(object_map.keys())
-        for removed_id in self.current_ids.intersection(object_ids):
-            self.current_ids.remove(removed_id)
-            self.emit('removed', object_map[removed_id])
+        for obj in [o for o in objects if o.id in self.current_ids]:
+            self.current_ids.remove(obj.id)
+            self.emit('removed', self.fetcher.fetch_obj_for_ddb_object(obj))
 
     def check_object(self, obj):
         before = (obj.id in self.current_ids)
         now = self._obj_in_view(obj)
         if before and not now:
             self.current_ids.remove(obj.id)
-            self.emit('removed', obj)
+            self.emit('removed', self.fetcher.fetch_obj_for_ddb_object(obj))
         elif now and not before:
             self.current_ids.add(obj.id)
-            self.emit('added', obj)
+            self.emit('added', self.fetcher.fetch_obj_for_ddb_object(obj))
         elif before and now:
-            self.emit('changed', obj)
+            self.emit('changed', self.fetcher.fetch_obj_for_ddb_object(obj))
+
+    def _emit_for_objects(self, signal, objects):
+        if self.bulk_mode:
+            self.emit('bulk-' + signal, objects)
+        else:
+            for obj in objects:
+                self.emit(signal, obj)
 
     def check_all_objects(self):
-        new_ids = set(app.db.query_ids(self.klass, self.where,
-                                       self.values, joins=self.joins))
         old_ids = self.current_ids
-        self.current_ids = new_ids
-        for id_ in new_ids.difference(old_ids):
-            self.emit('added', app.db.get_obj_by_id(id_))
-        for id_ in old_ids.difference(new_ids):
-            self.emit('removed', app.db.get_obj_by_id(id_))
-        for id_ in old_ids.intersection(new_ids):
-            # XXX this hits all the IDs, but there doesn't seem to be
-            # a way to check if the objects have actually been
-            # changed.  luckily, this isn't called very often.
-            self.emit('changed', app.db.get_obj_by_id(id_))
+        self._update_current_ids(self._view_object_ids())
+        # XXX this hits all the IDs, but there doesn't seem to be
+        # a way to check if the objects have actually been
+        # changed.  luckily, this isn't called very often.
+        changed_ids = list(old_ids.intersection(self.current_ids))
+        self.fetcher.prepare_objects(changed_ids)
+        self._emit_for_objects('changed',
+            [self.fetcher.fetch_obj(id_) for id_ in changed_ids])
+
+    def _update_current_ids(self, new_ids):
+        if new_ids == self.current_ids:
+            return
+        added_ids = list(new_ids.difference(self.current_ids))
+        removed_ids = list(self.current_ids.difference(new_ids))
+        self.fetcher.prepare_objects(added_ids)
+        self.fetcher.prepare_objects(removed_ids)
+        self.current_ids = new_ids # set current_ids here so that len() returns
+                                   # the correct value
+        self._emit_for_objects('added',
+                [self.fetcher.fetch_obj(id_) for id_ in added_ids])
+        self._emit_for_objects('removed',
+                [self.fetcher.fetch_obj(id_) for id_ in removed_ids])
 
     def __len__(self):
         return len(self.current_ids)
 
+
 class BulkSQLManager(object):
     def __init__(self):
         self.active = False
         self.to_insert = {}
         self.to_remove = {}
         self.pending_inserts = set()
+        self.pending_removes = set()
+
+        self.last_call = None
 
     def start(self):
         if self.active:
-            raise ValueError("BulkSQLManager.start() called twice")
+            raise ValueError(
+                "BulkSQLManager.start() called twice (previous: %s)",
+                self.last_call)
         self.active = True
+        self.last_call = "".join(traceback.format_stack())
 
     def finish(self):
         if not self.active:
             raise ValueError("BulkSQLManager.finish() called twice")
-        self.commit()
-        self.active = False
+        try:
+            self.commit()
+        finally:
+            # Ensure that this flag always get set back to False even in the
+            # face of any exception thrown from commit() method.
+            self.active = False
+
+        # Force a commit of our current transaction.
+        #
+        # Normally if we throw an exception, we want to rollback.  However, if
+        # we are in the middle of bulk inserting/removing, then we should try
+        # to commit the objects that didn't throw anything see (#16341)
+        app.db.finish_transaction()
 
     def commit(self):
         for x in range(100):
@@ -289,6 +418,7 @@ class BulkSQLManager(object):
         self.to_insert = {}
         self.to_remove = {}
         self.pending_inserts = set()
+        self.pending_removes = set()
 
     def _commit_sql(self, to_insert, to_remove):
         for table_name, objects in to_insert.items():
@@ -322,22 +452,27 @@ class BulkSQLManager(object):
             inserts_for_table = []
             self.to_insert[table_name] = inserts_for_table
         inserts_for_table.append(obj)
-        self.pending_inserts.add(obj)
+        self.pending_inserts.add(obj.id)
 
-    def will_insert(self, obj):
-        return obj in self.pending_inserts
+    def will_insert(self, id_):
+        return id_ in self.pending_inserts
+
+    def will_remove(self, id_):
+        return id_ in self.pending_removes
 
     def add_remove(self, obj):
         table_name = app.db.table_name(obj.__class__)
-        if self.will_insert(obj):
+        if self.will_insert(obj.id):
             self.to_insert[table_name].remove(obj)
-            self.pending_inserts.remove(obj)
+            self.pending_inserts.remove(obj.id)
+            app.db.forget_object(obj)
             return
         try:
             removes_for_table = self.to_remove[table_name]
         except KeyError:
             removes_for_table = []
             self.to_remove[table_name] = removes_for_table
+        self.pending_removes.add(obj.id)
         removes_for_table.append(obj)
 
 class AttributeUpdateTracker(object):
@@ -395,8 +530,11 @@ class DDBObject(signals.SignalEmitter):
             # when setup_new() is being run
             app.db.remember_object(self)
             self.setup_new(*args, **kwargs)
-            # handle setup_new() calling remove()
+            self.after_setup_new()
             if not self.id_exists():
+                # DeprecationWarning
+                logging.warn("deprecated special case: "
+                        "object removed in setup_new")
                 return
 
         self.in_db_init = False
@@ -418,10 +556,11 @@ class DDBObject(signals.SignalEmitter):
 
     @classmethod
     def make_view(cls, where=None, values=None, order_by=None, joins=None,
-                  limit=None):
+            limit=None):
         if values is None:
             values = ()
-        return View(cls, where, values, order_by, joins, limit)
+        fetcher = DDBObjectFetcher(cls)
+        return View(fetcher, where, values, order_by, joins, limit)
 
     @classmethod
     def get_by_id(cls, id_):
@@ -441,12 +580,16 @@ class DDBObject(signals.SignalEmitter):
 
     @classmethod
     def select(cls, columns, where=None, values=None, convert=True):
-        return app.db.select(cls, columns, where, values, convert)
+        return app.db.select(cls, columns, where, values, convert=convert)
 
     def setup_new(self):
         """Initialize a newly created object."""
         pass
 
+    def after_setup_new(self):
+        """Called immediately after setup_new returns."""
+        pass
+
     def setup_restored(self):
         """Initialize an object restored from disk."""
         pass
@@ -539,7 +682,7 @@ class DDBObject(signals.SignalEmitter):
             raise DatabaseConstraintError, msg
         self.on_signal_change()
         self.check_constraints()
-        if app.bulk_sql_manager.will_insert(self):
+        if app.bulk_sql_manager.will_insert(self.id):
             # Don't need to send an UPDATE SQL command, or check the
             # view trackers in this case.  Both will be done when the
             # BulkSQLManager.finish() is called.
