@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,12 +43,16 @@ import heapq
 import Queue
 import logging
 import traceback
+from miro import app
+from miro import config
 from miro import trapcall
 from miro import signals
 from miro import util
 
 from miro.clock import clock
 
+from miro.plat.utils import thread_body
+
 cumulative = {}
 
 class DelayedCall(object):
@@ -162,7 +167,8 @@ class ThreadPool(object):
     def init_threads(self):
         while len(self.threads) < ThreadPool.THREADS:
             t = threading.Thread(name='ThreadPool - %d' % len(self.threads),
-                                 target=self.thread_loop)
+                                 target=thread_body,
+                                 args=[self.thread_loop])
             t.setDaemon(True)
             t.start()
             self.threads.append(t)
@@ -199,13 +205,18 @@ class ThreadPool(object):
     def close_threads(self):
         for x in xrange(len(self.threads)):
             self.queue.put("QUIT")
+        # Why is there a timeout on the join() here, what's wrong?  On
+        # shutdown, the system waits for the eventloop to finish using 
+        # eventloop.join() but eventloop calls close_threads() which wait
+        # for the threadpool threads to shutdown.  But these could be blocked
+        # in a blocking operation which is exactly the point of having them
+        # so eventloop.join() in turn blocks.  So if it doesn't clean up
+        # in time let the daemon flag in the Thread() do its job.  See #16584.
         while len(self.threads) > 0:
             x = self.threads.pop()
             try:
-                x.join()
-            except (SystemExit, KeyboardInterrupt):
-                raise
-            except:
+                x.join(0.5)
+            except StandardError:
                 pass
 
 class SimpleEventLoop(signals.SignalEmitter):
@@ -237,8 +248,10 @@ class SimpleEventLoop(signals.SignalEmitter):
                 if err == errno.EINTR:
                     logging.warning ("eventloop: %s", detail)
                 else:
+                    self.emit('end-loop')
                     raise
             if self.quit_flag:
+                self.emit('end-loop')
                 break
             if self.wake_receiver.fileno() in read_fds_ready:
                 self._slurp_waker_data()
@@ -265,6 +278,7 @@ class EventLoop(SimpleEventLoop):
         self.read_callbacks = {}
         self.write_callbacks = {}
         self.clear_removed_callbacks()
+        self.idles_for_next_loop = []
 
     def clear_removed_callbacks(self):
         self.removed_read_callbacks = set()
@@ -289,6 +303,10 @@ class EventLoop(SimpleEventLoop):
         self.threadpool.queue_call(callback, errback, function, name,
                                   *args, **kwargs)
 
+    def run_idle_next_loop(self, function, name, args=None, kwargs=None):
+        """Add an idle callback to be called on the next event loop."""
+        self.idles_for_next_loop.append((function, name, args, kwargs))
+
     def process_events(self, read_fds_ready, write_fds_ready, exc_fds_ready):
         self._process_urgent_events()
         if self.quit_flag:
@@ -310,6 +328,17 @@ class EventLoop(SimpleEventLoop):
 
     def do_begin_loop(self):
         self.clear_removed_callbacks()
+        self._add_idles_for_next_loop()
+
+    def _add_idles_for_next_loop(self):
+        if not self.idles_for_next_loop:
+            return
+        for func, name, args, kwargs in self.idles_for_next_loop:
+            self.idle_queue.add_idle(func, name, args, kwargs)
+        self.idles_for_next_loop = []
+        # call wakeup() to make sure we process the idles we just
+        # added
+        self.wakeup()
 
     def _process_urgent_events(self):
         queue = self.urgent_queue
@@ -409,6 +438,7 @@ def add_timeout(delay, function, name, args=None, kwargs=None):
     call.
     """
     dc = _eventloop.scheduler.add_timeout(delay, function, name, args, kwargs)
+    _eventloop.wakeup()
     return dc
 
 def add_idle(function, name, args=None, kwargs=None):
@@ -460,6 +490,10 @@ def startup():
     lt.start()
     _eventloop.loop_ready.wait()
 
+def setup_config_watcher():
+    app.backend_config_watcher = config.ConfigWatcher(
+            lambda func, *args: add_idle(func, "config callback", args=args))
+
 def join():
     if lt is not None:
         lt.join()
@@ -548,7 +582,8 @@ def _idle_iterate_step(iterator, name):
         if retval is not None:
             logging.warn("idle_iterate yield value ignored: %s (%s)",
                          retval, name)
-        add_idle(_idle_iterate_step, name, args=(iterator, name))
+        _eventloop.run_idle_next_loop(_idle_iterate_step, name,
+                args=(iterator, name))
 
 def idle_iterator(func):
     """Decorator to wrap a generator function in a ``idle_iterate()``
