@@ -1,6 +1,7 @@
 /*
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -164,6 +165,11 @@ static PyObject *LinkedListIter_copy(LinkedListIterObject* self, PyObject *obj)
     return (PyObject*)LinkedListIterObject_new(self->list, self->node);
 }
 
+static PyObject *LinkedListIter_valid(LinkedListIterObject* self, PyObject *obj)
+{
+    return PyBool_FromLong(self->node->deleted == 0);
+}
+
 PyObject* LinkedListIter_richcmp(LinkedListIterObject *o1,
         LinkedListIterObject *o2, int opid)
 {
@@ -196,6 +202,9 @@ static PyMethodDef LinkedListIter_methods[] = {
     {"copy", (PyCFunction)LinkedListIter_copy, METH_NOARGS,
              "Duplicate iter",
     },
+    {"valid", (PyCFunction)LinkedListIter_valid, METH_NOARGS,
+             "Test if the iter is valid",
+    },
     {NULL},
 };
 
@@ -334,7 +343,7 @@ static int LinkedList_init(LinkedListObject *self)
     return 0;
 }
 
-static int LinkedList_len(LinkedListObject *self)
+static Py_ssize_t LinkedList_len(LinkedListObject *self)
 {
     return self->count;
 }
@@ -421,7 +430,7 @@ static PyObject *LinkedList_lastIter(LinkedListObject* self, PyObject *obj)
 }
 
 static PyMappingMethods LinkedListMappingMethods = {
-    LinkedList_len,
+    (lenfunc)LinkedList_len,
     (binaryfunc)LinkedList_get,
     (objobjargproc)LinkedList_set,
 };
