@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,21 +31,20 @@
 
 import StringIO
 
-import pango
 import gobject
 import gtk
+import os
 
 from miro import app
-from miro import config
 from miro import prefs
 from miro import signals
 from miro import dialogs
+from miro.fileobject import FilenameType
 from miro.gtcache import gettext as _
 from miro.frontends.widgets.gtk import wrappermap, widgets
-from miro.frontends.widgets.gtk import keymap
+from miro.frontends.widgets.gtk import keymap, layout
 from miro.frontends.widgets import menus
 from miro.plat import resources
-from miro.plat import utils
 
 # keeps the objects alive until destroy() is called
 alive_windows = set()
@@ -78,7 +78,8 @@ for i in range(1, 13):
     keymap.menubar_key_map[getattr(menus, name)] = name
 
 def get_accel_string(shortcut):
-    mod_str = ''.join(keymap.menubar_mod_map[mod] for mod in shortcut.modifiers)
+    mod_str = ''.join(
+        keymap.menubar_mod_map[mod] for mod in shortcut.modifiers)
     key_str = keymap.menubar_key_map.get(shortcut.shortcut, shortcut.shortcut)
     return mod_str + key_str
 
@@ -101,13 +102,51 @@ class WrappedWindow(gtk.Window):
         wrappermap.wrapper(self).emit('active-change')
 
     def do_key_press_event(self, event):
+        if self.activate_key(event): # event activated a menu item
+            return
+
+        if self.propagate_key_event(event): # event handled by widget
+            return
+
         ret = keymap.translate_gtk_event(event)
-        if ret == None:
-            return gtk.Window.do_key_press_event(self, event)
-        key, modifiers = ret
-        if wrappermap.wrapper(self).emit('key-press', key, modifiers):
-            return # handler returned True, don't process the key more
-        return gtk.Window.do_key_press_event(self, event)
+        if ret is not None:
+            key, modifiers = ret
+            rv = wrappermap.wrapper(self).emit('key-press', key, modifiers)
+            if not rv:
+                gtk.Window.do_key_press_event(self, event)
+
+    def _get_focused_wrapper(self):
+        """Get the wrapper of the widget with keyboard focus"""
+        focused = self.get_focus()
+        # some of our widgets created children for their use
+        # (GtkSearchTextEntry).  If we don't find a wrapper for
+        # focused, try it's parents
+        while focused is not None:
+            try:
+                wrapper = wrappermap.wrapper(focused)
+            except KeyError:
+                focused = focused.get_parent()
+            else:
+                return wrapper
+        return None
+
+    def change_focus_using_wrapper(self, direction):
+        my_wrapper = wrappermap.wrapper(self)
+        focused_wrapper = self._get_focused_wrapper()
+        if direction == gtk.DIR_TAB_FORWARD:
+            to_focus = my_wrapper.get_next_tab_focus(focused_wrapper, True)
+        elif direction == gtk.DIR_TAB_BACKWARD:
+            to_focus = my_wrapper.get_next_tab_focus(focused_wrapper, False)
+        else:
+            return False
+        if to_focus is not None:
+            to_focus.focus()
+            return True
+        return False
+
+    def do_focus(self, direction):
+        if not self.change_focus_using_wrapper(direction):
+            gtk.Window.do_focus(self, direction)
 
 gobject.type_register(WrappedWindow)
 
@@ -117,7 +156,7 @@ class WrappedMainWindow(WrappedWindow):
                 event.state & gtk.gdk.MOD1_MASK and
                 app.playback_manager.is_playing):
             # Hack for having 2 shortcuts for fullscreen
-            app.widgetapp.on_fullscreen_clicked()
+            app.playback_manager.enter_fullscreen()
             return
         return WrappedWindow.do_key_press_event(self, event)
 
@@ -187,14 +226,15 @@ class WindowBase(signals.SignalEmitter):
 
         # on linux, we don't have a CheckVersion option because
         # we update with the package system.
-        this_platform = config.get(prefs.APP_PLATFORM)
+        this_platform = app.config.get(prefs.APP_PLATFORM)
         if this_platform == 'linux':
             file_menu = self.menu_structure.get("FileMenu")
             file_menu.remove("CheckVersion")
 
         # If the renderer supports it, create a the subtitle encodings menu
         try:
-            app.video_renderer.setup_subtitle_encoding_menu(self.menu_structure)
+            app.video_renderer.setup_subtitle_encoding_menu(
+                self.menu_structure)
         except AttributeError:
             pass
 
@@ -207,7 +247,6 @@ class WindowBase(signals.SignalEmitter):
 
         outstream = StringIO.StringIO()
         outstream.write('<ui><menubar name="MiroMenu">')
-        extra_accelerators = []
         for mem in self.menu_structure.menuitems:
             self._add_menu(mem, outstream)
         outstream.write('</menubar>')
@@ -241,7 +280,9 @@ class WindowBase(signals.SignalEmitter):
             gtk_action.set_group(root_action)
 
     def setup_action(self, gtk_action, groups, shortcuts):
-        callback = menus.lookup_handler(gtk_action.get_name())
+        action_name = gtk_action.get_name()
+        self.actions[action_name] = gtk_action
+        callback = menus.lookup_handler(action_name)
         if callback is not None:
             gtk_action.connect("activate", self.on_activate, callback)
         action_group_name = groups[0]
@@ -261,7 +302,7 @@ class WindowBase(signals.SignalEmitter):
                 action_group.add_action_with_accel(extra_action,
                                                    get_accel_string(shortcut))
 
-    def make_check_action(self, action, label, groups, callback, index,
+    def _raw_check_action(self, action, label, groups, callback, index,
                           group=None):
         gtk_action = gtk.RadioAction(action, label, None, None, index)
         if group is not None:
@@ -269,9 +310,22 @@ class WindowBase(signals.SignalEmitter):
         gtk_action.connect("activate", callback, index)
         self.action_groups[groups[0]].add_action(gtk_action)
 
+    def make_check_action(self, action, check_group, label, groups, shortcuts):
+        gtk_action = gtk.ToggleAction(action, label, None, None)
+        self.actions[action] = gtk_action
+        callback = menus.lookup_handler(gtk_action.get_name())
+        if callback is not None:
+            gtk_action.connect("toggled", self.on_activate, callback)
+        if check_group not in self.check_groups:
+            self.check_groups[check_group] = list()
+        self.check_groups[check_group].append(gtk_action)
+        self.action_groups[groups[0]].add_action(gtk_action)
+
     def make_actions(self):
         self.action_groups = {}
+        self.actions = {}
         self.radio_group_actions = {}
+        self.check_groups = {}
 
         for name in self.menu_action_groups.keys():
             self.action_groups[name] = gtk.ActionGroup(name)
@@ -286,17 +340,21 @@ class WindowBase(signals.SignalEmitter):
             elif isinstance(mem, menus.RadioMenuItem):
                 self.make_radio_action(mem.action, mem.radio_group, mem.label,
                         mem.groups, mem.shortcuts)
+            elif isinstance(mem, menus.CheckMenuItem):
+                self.make_check_action(mem.action, mem.check_group, mem.label,
+                        mem.groups, mem.shortcuts)
             elif isinstance(mem, menus.MenuItem):
                 self.make_action(mem.action, mem.label, mem.groups,
                         mem.shortcuts)
 
 
         # make a bunch of SubtitleTrack# actions
-        self.make_check_action("SubtitlesDisabled", _("Disable Subtitles"),
+        self._raw_check_action("SubtitlesDisabled", _("Disable Subtitles"),
                                ["AlwaysOn"], self.on_subtitles_change, -1)
-        radio_group = self.action_groups["AlwaysOn"].get_action("SubtitlesDisabled")
+        radio_group = self.action_groups["AlwaysOn"].get_action(
+            "SubtitlesDisabled")
         for i in range(199):
-            self.make_check_action("SubtitleTrack%d" % i, "", ["AlwaysOn"],
+            self._raw_check_action("SubtitleTrack%d" % i, "", ["AlwaysOn"],
                                    self.on_subtitles_change, i, radio_group)
 
         for action_group in self.action_groups.values():
@@ -310,10 +368,12 @@ class WindowBase(signals.SignalEmitter):
     def on_subtitles_change(self, action, track_index):
         if hasattr(self, "_ignore_on_subtitles_change"):
             return
-        if action.get_property("current-value") != action.get_property("value"):
+        if action.get_property("current-value") != action.get_property(
+            "value"):
             return
         action_group = self.action_groups["AlwaysOn"]
-        action_group.get_action("SubtitlesDisabled").current_value = track_index
+        action_group.get_action(
+            "SubtitlesDisabled").current_value = track_index
         if track_index == -1:
             app.video_renderer.disable_subtitles()
         else:
@@ -350,27 +410,42 @@ class Window(WindowBase):
         self.create_signal('did-move')
         alive_windows.add(self)
 
-        self._window.connect('delete-event', self.on_delete)
+        self._window.connect('delete-event', self.on_delete_window)
+
+    def on_delete_window(self, widget, event):
+        # when the user clicks on the X in the corner of the window we
+        # want that to close the window, but also trigger our
+        # will-close signal and all that machinery unless the window
+        # is currently hidden--then we don't do anything.
+        if not self._window.window.is_visible():
+            return
+        self.close()
+        return True
 
     def _make_gtk_window(self):
         return WrappedWindow()
 
-    def on_delete(self, widget, event):
-        self.emit('will-close')
-        return True
-
     def set_title(self, title):
         self._window.set_title(title)
 
+    def get_title(self):
+        self._window.get_title()
+
     def show(self):
         if self not in alive_windows:
             raise ValueError("Window destroyed")
         self._window.show()
 
     def close(self):
+        if hasattr(self, "_closing"):
+            return
+        self._closing = True
+        self.emit('will-close')
         self._window.hide()
+        del self._closing
 
     def destroy(self):
+        self.close()
         self._window.destroy()
         alive_windows.discard(self)
 
@@ -380,6 +455,9 @@ class Window(WindowBase):
     def is_visible(self):
         return self._window.props.visible
 
+    def get_next_tab_focus(self, current, is_forward):
+        return None
+
     def set_content_widget(self, widget):
         """Set the widget that will be drawn in the content area for this
         window.
@@ -404,6 +482,32 @@ class Window(WindowBase):
         size = self._window.get_size()
         return widgets.Rect(pos[0], pos[1], size[0], size[1])
 
+    def set_frame(self, x=None, y=None, width=None, height=None):
+        if x is not None or y is not None:
+            pos = self._window.get_position()
+            x = x if x is not None else pos[0]
+            y = y if y is not None else pos[1]
+            self._window.move(x, y)
+
+        if width is not None or height is not None:
+            size = self._window.get_size()
+            width = width if width is not None else size[0]
+            height = height if height is not None else size[1]
+            self._window.resize(width, height)
+
+    def get_monitor_geometry(self):
+        """Returns a Rect of the geometry of the monitor that this
+        window is currently on.
+
+        :returns: Rect
+        """
+        gtkwindow = self._window
+        gdkwindow = gtkwindow.window
+        screen = gtkwindow.get_screen()
+
+        monitor = screen.get_monitor_at_window(gdkwindow)
+        return screen.get_monitor_geometry(monitor)
+
     def check_position_and_fix(self):
         """This pulls the geometry of the monitor of the screen this
         window is on as well as the position of the window.
@@ -414,10 +518,8 @@ class Window(WindowBase):
         """
         gtkwindow = self._window
         gdkwindow = gtkwindow.window
-        screen = gtkwindow.get_screen()
+        monitor_geom = self.get_monitor_geometry()
 
-        monitor = screen.get_monitor_at_window(gdkwindow)
-        monitor_geom = screen.get_monitor_geometry(monitor)
         frame_extents = gdkwindow.get_frame_extents()
         position = gtkwindow.get_position()
 
@@ -427,6 +529,11 @@ class Window(WindowBase):
             gtkwindow.move(position[0],
                            monitor_geom.y + (position[1] - frame_extents.y))
 
+class DialogWindow(Window):
+    def __init__(self, title, rect=None):
+        Window.__init__(self, title, rect)
+        self._window.set_resizable(False)
+
 class MainWindow(Window):
     def __init__(self, title, rect):
         Window.__init__(self, title, rect)
@@ -437,23 +544,27 @@ class MainWindow(Window):
         self.connect_menu_keyboard_shortcuts()
         self.create_signal('save-dimensions')
         self.create_signal('save-maximized')
+        self.create_signal('on-shown')
         app.menu_manager.connect('enabled-changed', self.on_menu_change)
         app.menu_manager.connect('radio-group-changed', self.on_radio_change)
-        app.playback_manager.connect('did-start-playing', self.on_playback_change)
+        app.menu_manager.connect('checked-changed', self.on_checked_change)
+        app.playback_manager.connect('did-start-playing',
+                                     self.on_playback_change)
         app.playback_manager.connect('will-play', self.on_playback_change)
         app.playback_manager.connect('did-stop', self.on_playback_change)
 
         self._window.connect('key-release-event', self.on_key_release)
         self._window.connect('window-state-event', self.on_window_state_event)
         self._window.connect('configure-event', self.on_configure_event)
-        self.connect('will-close', self.on_close)
+        self._window.connect('map-event', lambda w, a: self.emit('on-shown'))
         self._clear_subtitles_menu()
 
     def _make_gtk_window(self):
         return WrappedMainWindow()
 
-    def on_close(self, window):
+    def on_delete_window(self, widget, event):
         app.widgetapp.on_close()
+        return True
 
     def on_configure_event(self, widget, event):
         (x, y) = self._window.get_position()
@@ -461,7 +572,8 @@ class MainWindow(Window):
         self.emit('save-dimensions', x, y, width, height)
 
     def on_window_state_event(self, widget, event):
-        maximized = (event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED) != 0
+        maximized = bool(
+            event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
         self.emit('save-maximized', maximized)
 
     def on_key_release(self, widget, event):
@@ -478,41 +590,33 @@ class MainWindow(Window):
     def on_menu_change(self, menu_manager):
         for name, action_group in self.action_groups.items():
             if name in menu_manager.enabled_groups:
+                action_group.set_visible(True)
                 action_group.set_sensitive(True)
             else:
-                action_group.set_sensitive(False)
-
-        removeSomething = self.menu_structure.get("RemoveSomething").label
-        updateFeeds = self.menu_structure.get("UpdateFeeds").label
-        removePlaylists = self.menu_structure.get("RemovePlaylists").label
-        removeItems = self.menu_structure.get("RemoveItems").label
+                # TODO: don't hard-code this here; probably put a "hide when
+                # not checked" property on check menu class --Kaz
+                if name.startswith('column-'):
+                    action_group.set_visible(False)
+                else:
+                    action_group.set_sensitive(False)
 
         def get_state_label(action, state):
             menu = self.menu_structure.get(action)
             return menu.state_labels.get(state, menu.label)
 
-        for state, actions in menu_manager.states.items():
-            if "RemoveSomething" in actions:
-                removeSomething = get_state_label("RemoveSomething", state)
-            if "UpdateFeeds" in actions:
-                updateFeeds = get_state_label("UpdateFeeds", state)
-            if "RemovePlaylists" in actions:
-                removePlaylists = get_state_label("RemovePlaylists", state)
-            if "RemoveItems" in actions:
-                removeItems = get_state_label("RemoveItems", state)
+        action_labels = {}
+        for state, actions in menu_manager.states.iteritems():
+            for action in actions:
+                action_labels[action] = get_state_label(action, state)
 
-        action_groups = self.action_groups
+        for name, action in self.actions.iteritems():
+            default = self.menu_structure.get(name).label
+            new_label = action_labels.get(name, default)
+            action.set_property('label', new_label)
 
-        def change_label(group, action, newlabel):
-            action_groups[group].get_action(action).set_property("label", newlabel)
-
-        change_label("RemoveAllowed", "RemoveSomething", removeSomething)
-        change_label("FeedsSelected", "UpdateFeeds", updateFeeds)
-        change_label("PlaylistsSelected", "RemovePlaylists", removePlaylists)
-        change_label("PlayablesSelected", "RemoveItems", removeItems)
-
-        play_pause = self.menu_structure.get("PlayPauseItem").state_labels[menu_manager.play_pause_state]
-        change_label("PlayPause", "PlayPauseItem", play_pause)
+        play_pause = self.menu_structure.get('PlayPauseItem').state_labels[
+            menu_manager.play_pause_state]
+        self.actions['PlayPauseItem'].set_property('label', play_pause)
 
     def on_radio_change(self, menu_manager, radio_group, value):
         root_action = self.radio_group_actions[radio_group]
@@ -521,12 +625,23 @@ class MainWindow(Window):
                 action.set_active(True)
                 return
 
+    def on_checked_change(self, menu_manager, check_group, values):
+        group = self.check_groups[check_group]
+        for action in group:
+            name = action.get_name()
+            if name in values:
+                checked = values[name]
+                action.handler_block_by_func(self.on_activate)
+                action.set_active(checked)
+                action.handler_unblock_by_func(self.on_activate)
+
     def on_playback_change(self, playback_manager, *extra_args):
         self._ignore_on_subtitles_change = True
         if app.playback_manager.is_playing_audio:
             self._clear_subtitles_menu()
         else:
             tracks = app.video_renderer.get_subtitle_tracks()
+#             if tracks is None or len(tracks) == 0:
             if len(tracks) == 0:
                 self._clear_subtitles_menu()
             else:
@@ -551,7 +666,8 @@ class MainWindow(Window):
       <menu action="SubtitlesMenu">
 ''')
         for i, lang in tracks:
-            outstream.write('         <menuitem action="SubtitleTrack%d"/>\n' % i)
+            outstream.write(
+                '         <menuitem action="SubtitleTrack%d"/>\n' % i)
         outstream.write('''         <separator/>
          <menuitem action="SubtitlesDisabled"/>
          <menuitem action="SubtitlesSelect"/>
@@ -560,13 +676,16 @@ class MainWindow(Window):
 </menubar>
 </ui>''')
 
-        self._merge_id = self.ui_manager.add_ui_from_string(outstream.getvalue())
-        
+        self._merge_id = self.ui_manager.add_ui_from_string(
+            outstream.getvalue())
+
         action_group = self.action_groups["AlwaysOn"]
         for i, lang in tracks:
-            action_group.get_action("SubtitleTrack%d" % i).set_property("label", lang)
+            action_group.get_action("SubtitleTrack%d" % i).set_property(
+                "label", lang)
 
-        action_group.get_action("SubtitlesDisabled").set_property("current-value", enabled_track)
+        action_group.get_action("SubtitlesDisabled").set_property(
+            "current-value", enabled_track)
 
     def _clear_subtitles_menu(self):
         if self._merge_id is not None:
@@ -624,18 +743,23 @@ class Dialog(DialogBase):
         self.create_signal('close')
         self.set_window(gtk.Dialog(title))
         self._window.set_default_size(425, -1)
-        self.packing_vbox = gtk.VBox(spacing=20)
-        self.packing_vbox.set_border_width(6)
-        self._window.vbox.pack_start(self.packing_vbox, True, True)
-        if description:
-            label = gtk.Label(description)
-            label.set_line_wrap(True)
-            label.set_size_request(390, -1)
-            label.set_selectable(True)
-            self.packing_vbox.pack_start(label)
         self.extra_widget = None
         self.buttons_to_add = []
         wrappermap.add(self._window, self)
+        self.description = description
+
+    def build_content(self):
+        packing_vbox = layout.VBox(spacing=20)
+        packing_vbox._widget.set_border_width(6)
+        if self.description:
+            label = gtk.Label(self.description)
+            label.set_line_wrap(True)
+            label.set_size_request(390, -1)
+            label.set_selectable(True)
+            packing_vbox._widget.pack_start(label)
+        if self.extra_widget:
+            packing_vbox._widget.pack_start(self.extra_widget._widget)
+        return packing_vbox
 
     def add_button(self, text):
         self.buttons_to_add.append(_stock.get(text, text))
@@ -656,6 +780,8 @@ class Dialog(DialogBase):
 
     def _run(self):
         self.pack_buttons()
+        packing_vbox = self.build_content()
+        self._window.vbox.pack_start(packing_vbox._widget, True, True)
         self._window.show_all()
         response = self._window.run()
         self._window.hide()
@@ -666,14 +792,9 @@ class Dialog(DialogBase):
 
     def destroy(self):
         DialogBase.destroy(self)
-        if hasattr(self, 'packing_vbox'):
-            del self.packing_vbox
 
     def set_extra_widget(self, widget):
-        if self.extra_widget:
-            self.packing_vbox.remove(widget._widget)
         self.extra_widget = widget
-        self.packing_vbox.pack_start(widget._widget)
 
     def get_extra_widget(self):
         return self.extra_widget
@@ -718,10 +839,21 @@ class FileOpenDialog(FileDialogBase):
         self._window.add_filter(f)
 
     def get_filenames(self):
-        return [utils.FilenameType(f) for f in self._files]
+        return [FilenameType(f) for f in self._files]
 
     def get_filename(self):
-        return utils.FilenameType(self._files[0])
+        if self._files is None:
+            # clicked Cancel
+            return None
+        else:
+            return FilenameType(self._files[0])
+
+    # provide a common interface for file chooser dialogs
+    get_path = get_filename
+    def set_path(self, path):
+        # set_filename puts the whole path in the filename field
+        self._window.set_current_folder(os.path.dirname(path))
+        self._window.set_current_name(os.path.basename(path))
 
 class FileSaveDialog(FileDialogBase):
     def __init__(self, title):
@@ -739,97 +871,124 @@ class FileSaveDialog(FileDialogBase):
         self._window.set_current_name(text)
 
     def get_filename(self):
-        return utils.FilenameType(self._files[0])
+        if self._files is None:
+            # clicked Cancel
+            return None
+        else:
+            return FilenameType(self._files[0])
+
+    # provide a common interface for file chooser dialogs
+    get_path = get_filename
+    def set_path(self, path):
+        # set_filename puts the whole path in the filename field
+        self._window.set_current_folder(os.path.dirname(path))
+        self._window.set_current_name(os.path.basename(path))
 
 class DirectorySelectDialog(FileDialogBase):
     def __init__(self, title):
         FileDialogBase.__init__(self)
         self._files = None
         choose_str =_('Choose').encode('utf-8')
-        fcd = gtk.FileChooserDialog(title,
-                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                    buttons=(gtk.STOCK_CANCEL,
-                                             gtk.RESPONSE_CANCEL,
-                                             choose_str, gtk.RESPONSE_OK))
+        fcd = gtk.FileChooserDialog(
+            title,
+            action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
+            buttons=(gtk.STOCK_CANCEL,
+                     gtk.RESPONSE_CANCEL,
+                     choose_str, gtk.RESPONSE_OK))
         self.set_window(fcd)
 
     def set_directory(self, text):
         self._window.set_filename(text)
 
     def get_directory(self):
-        return utils.FilenameType(self._files[0])
+        if self._files is None:
+            # clicked Cancel
+            return None
+        else:
+            return FilenameType(self._files[0])
+
+    # provide a common interface for file chooser dialogs
+    get_path = get_directory
+    set_path = set_directory
 
 class AboutDialog(Dialog):
     def __init__(self):
         Dialog.__init__(self,
-                        _("About %(appname)s") % {
-                'appname': config.get(prefs.SHORT_APP_NAME)})
-        icon_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
-                resources.share_path('icons/hicolor/128x128/apps/miro.png'),
-                48, 48)
-        self.packing_vbox.pack_start(gtk.image_new_from_pixbuf(icon_pixbuf))
-        if config.get(prefs.APP_REVISION_NUM):
-            version = "%s (%s)" % (
-                config.get(prefs.APP_VERSION),
-                config.get(prefs.APP_REVISION_NUM))
-        else:
-            version = "%s" % config.get(prefs.APP_VERSION)
-        name_label = gtk.Label('<span size="xx-large" weight="bold">%s %s</span>' % (
-                config.get(prefs.SHORT_APP_NAME), version))
-        name_label.set_use_markup(True)
-        self.packing_vbox.pack_start(name_label)
-        copyright_text = _('%(copyright)s.  See license.txt file for details.\n'
-                           '%(trademark)s') % (
-            {"copyright": config.get(prefs.COPYRIGHT),
-             "trademark": config.get(prefs.TRADEMARK)})
-        copyright_label = gtk.Label('<small>%s</small>' % copyright_text)
-        copyright_label.set_use_markup(True)
-        copyright_label.set_justify(gtk.JUSTIFY_CENTER)
-        self.packing_vbox.pack_start(copyright_label)
-        self.packing_vbox.pack_start(gtk.Label(config.get(prefs.PROJECT_URL)))
-
-        # get adopters, remove newlines, and wrap it
-        adopters_data = open(resources.path('ADOPTERS'), 'r').read()
-        first_double_newline = adopters_data.find('\n\n')
-        adopters_data = adopters_data[first_double_newline+2:-1].replace('\n', ', ')
-
-        # get contributors, remove newlines and wrap it
-        contributors = open(resources.path('CREDITS'), 'r').readlines()
-        contributors = [c[2:].strip() for c in contributors if c.startswith("* ")]
-        contributors = ", ".join(contributors)
-
-        contributors = "\n\nThank you to all the people who contributed to Miro 3.5:\n\n" + contributors
-
-        adopters_data = adopters_data + contributors
-
-        # show the adopters
-        adopters_buffer = gtk.TextBuffer()
-        adopters_buffer.set_text(adopters_data)
-        iter = adopters_buffer.get_start_iter()
-        adopters_buffer.insert(iter,
-            'Thank you to the following people who have adopted '
-            'lines of Miro code to help support development! '
-            'To help fund continued Miro development, visit the ')
-        adopt_link_tag = adopters_buffer.create_tag(
-            foreground='blue', underline=pango.UNDERLINE_SINGLE)
-        adopt_link_tag.connect('event', self.on_adopt_link_event)
-        adopters_buffer.insert_with_tags(iter, 'donation page',
-                                         adopt_link_tag)
-        adopters_buffer.insert(iter, '.\n\n')
-        adopters_view = gtk.TextView(adopters_buffer)
-        adopters_view.set_editable(False)
-        adopters_view.set_cursor_visible(False)
-        adopters_view.set_wrap_mode(gtk.WRAP_WORD)
-        adopters_window = gtk.ScrolledWindow()
-        adopters_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
-        adopters_window.add(adopters_view)
-        adopters_window.set_size_request(-1, 100)
-        self.packing_vbox.pack_start(adopters_window)
-
+            _("About %(appname)s",
+              {'appname': app.config.get(prefs.SHORT_APP_NAME)}))
         self.add_button(_("Close"))
         self._window.set_has_separator(False)
 
-    def on_adopt_link_event(self, texttag, widget, event, iter):
+    def build_content(self):
+        packing_vbox = layout.VBox(spacing=20)
+        icon_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
+                resources.share_path('icons/hicolor/128x128/apps/miro.png'),
+                48, 48)
+        packing_vbox._widget.pack_start(gtk.image_new_from_pixbuf(icon_pixbuf))
+        if app.config.get(prefs.APP_REVISION_NUM):
+            version = "%s (%s)" % (
+                app.config.get(prefs.APP_VERSION),
+                app.config.get(prefs.APP_REVISION_NUM))
+        else:
+            version = "%s" % app.config.get(prefs.APP_VERSION)
+        name_label = gtk.Label(
+            '<span size="xx-large" weight="bold">%s %s</span>' % (
+                app.config.get(prefs.SHORT_APP_NAME), version))
+        name_label.set_use_markup(True)
+        packing_vbox._widget.pack_start(name_label)
+        copyright_text = _(
+            '%(copyright)s.  See license.txt file for details.\n'
+            '%(trademark)s',
+            {"copyright": app.config.get(prefs.COPYRIGHT),
+             "trademark": app.config.get(prefs.TRADEMARK)})
+        copyright_label = gtk.Label('<small>%s</small>' % copyright_text)
+        copyright_label.set_use_markup(True)
+        copyright_label.set_justify(gtk.JUSTIFY_CENTER)
+        packing_vbox._widget.pack_start(copyright_label)
+
+        # FIXME - make the project url clickable
+        packing_vbox._widget.pack_start(
+            gtk.Label(app.config.get(prefs.PROJECT_URL)))
+
+        contributor_label = gtk.Label(
+            _("Thank you to all the people who contributed to Miro "
+              "%(version)s:",
+              {"version": app.config.get(prefs.APP_VERSION)}))
+        contributor_label.set_justify(gtk.JUSTIFY_CENTER)
+        packing_vbox._widget.pack_start(contributor_label)
+
+        # get contributors, remove newlines and wrap it
+        contributors = open(resources.path('CREDITS'), 'r').readlines()
+        contributors = [c[2:].strip()
+                        for c in contributors if c.startswith("* ")]
+        contributors = ", ".join(contributors)
+
+        # show contributors
+        contrib_buffer = gtk.TextBuffer()
+        contrib_buffer.set_text(contributors)
+
+        contrib_view = gtk.TextView(contrib_buffer)
+        contrib_view.set_editable(False)
+        contrib_view.set_cursor_visible(False)
+        contrib_view.set_wrap_mode(gtk.WRAP_WORD)
+        contrib_window = gtk.ScrolledWindow()
+        contrib_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
+        contrib_window.add(contrib_view)
+        contrib_window.set_size_request(-1, 100)
+        packing_vbox._widget.pack_start(contrib_window)
+
+        # FIXME - make the project url clickable
+        donate_label = gtk.Label(
+            _("To help fund continued Miro development, visit the "
+              "donation page at:"))
+        donate_label.set_justify(gtk.JUSTIFY_CENTER)
+        packing_vbox._widget.pack_start(donate_label)
+
+        packing_vbox._widget.pack_start(
+            gtk.Label(app.config.get(prefs.DONATE_URL)))
+        return packing_vbox
+
+    def on_contrib_link_event(self, texttag, widget, event, iter_):
         if event.type == gtk.gdk.BUTTON_PRESS:
             resources.open_url('http://getmiro.com/donate/')
 
@@ -846,6 +1005,7 @@ class AlertDialog(DialogBase):
         self.set_window(gtk.MessageDialog(type=message_type,
                                           message_format=description))
         self._window.set_title(title)
+        self.description = description
 
     def add_button(self, text):
         self._window.add_button(_stock.get(text, text), 1)
