@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,14 +30,14 @@
 """Menu handling code."""
 
 from miro import app
+from miro import errors
 from miro import prefs
 from miro import signals
-from miro import config
-from miro import videoconversion
+from miro import conversions
+from miro.frontends.widgets.widgetconst import COLUMN_LABELS
+from miro.frontends.widgets.widgetstatestore import WidgetStateStore
 
 from miro.gtcache import gettext as _
-from miro import config
-from miro import prefs
 
 (CTRL, ALT, SHIFT, CMD, RIGHT_ARROW, LEFT_ARROW, UP_ARROW,
  DOWN_ARROW, SPACE, ENTER, DELETE, BKSPACE, ESCAPE,
@@ -90,7 +91,7 @@ class MenuItem:
 
     >>> MenuItem(_("Preferences"), "EditPreferences")
     >>> MenuItem(_("Cu_t"), "ClipboardCut", Shortcut("x", MOD))
-    >>> MenuItem(_("_Update Feed"), "UpdateFeeds",
+    >>> MenuItem(_("_Update Podcasts and Library"), "UpdatePodcasts",
     ...          (Shortcut("r", MOD), Shortcut(F5)))
     >>> MenuItem(_("_Play"), "PlayPauseItem",
     ...          play=_("_Play"), pause=_("_Pause"))
@@ -120,6 +121,17 @@ class RadioMenuItem(MenuItem):
                 **state_labels)
         self.radio_group = radio_group
 
+class CheckMenuItem(MenuItem):
+    """MenuItem that has a check mark.
+
+    :param check_group: the group this menu item is in
+    """
+    def __init__(self, label, action, check_group, shortcuts=None,
+            groups=None, **state_labels):
+        MenuItem.__init__(self, label, action, shortcuts, groups,
+                **state_labels)
+        self.check_group = check_group
+
 class Separator:
     """This denotes a separator in the menu.
     """
@@ -203,114 +215,129 @@ def get_menu():
             Menu(_("_File"), "FileMenu", [
                     MenuItem(_("_Open"), "Open", Shortcut("o", MOD),
                              groups=["NonPlaying"]),
-                    MenuItem(_("_Download Item"), "NewDownload",
-                             groups=["NonPlaying"]),
-                    MenuItem(_("Check _Version"), "CheckVersion"),
+                    Menu(_("Import"), "Import", [
+                            MenuItem(_("Search all my Files..."),
+                                     "SearchAllMyFiles",
+                                     groups=["NonPlaying"]),
+                            MenuItem(_("Search in a Folder..."),
+                                     "SearchInAFolder",
+                                     groups=["NonPlaying"]),
+                            MenuItem(_("Watch a Folder..."),
+                                     "WatchAFolder",
+                                     groups=["NonPlaying"]),
+                            MenuItem(_("Choose Files...."),
+                                     "ChooseFiles",
+                                     groups=["NonPlaying"]),
+                            ]),
                     Separator(),
-                    MenuItem(_("_Remove Item"), "RemoveItems",
-                             Shortcut(BKSPACE, MOD),
-                             groups=["PlayablesSelected"],
-                             plural=_("_Remove Items")),
-                    MenuItem(_("_Edit Item"), "EditItem",
-                             groups=["PlayableSelected"]),
-                    MenuItem(_("Save Item _As"), "SaveItem",
+                    MenuItem(_("Download from a URL"), "NewDownload",
+                             groups=["NonPlaying"]),
+                    Separator(),
+                    MenuItem(_("Remove Item"), "RemoveItems",
+                             groups=["LocalItemsSelected"],
+                             plural=_("Remove Items")),
+                    MenuItem(_("Edit _Item"), "EditItems", Shortcut("i", MOD),
+                             groups=["LocalItemsSelected"],
+                             plural=_("Edit _Items")),
+                    MenuItem(_("_Save Item As"), "SaveItem",
                              Shortcut("s", MOD),
-                             groups=["PlayableSelected"],
-                             plural=_("Save Items _As")),
+                             groups=["LocalPlayableSelected"],
+                             plural=_("_Save Items As")),
                     MenuItem(_("Copy Item _URL"), "CopyItemURL",
                              Shortcut("u", MOD),
-                             groups=["PlayableSelected"]),
+                             groups=["LocalItemSelected"]),
                     Separator(),
-                    MenuItem(_("_Preferences"), "EditPreferences"),
+                    MenuItem(_("Check Version"), "CheckVersion"),
+                    MenuItem(_("Preferences"), "EditPreferences"),
                     MenuItem(_("_Quit"), "Quit", Shortcut("q", MOD)),
                     ]),
 
             Menu(_("_Sidebar"), "SidebarMenu", [
-                    MenuItem(_("Add _Feed"), "NewFeed", Shortcut("n", MOD),
+                    MenuItem(_("Add Podcast"), "NewPodcast",
+                             Shortcut("n", MOD),
                              groups=["NonPlaying"]),
-                    MenuItem(_("Add Website"), "NewGuide",
+                    MenuItem(_("Add Source"), "NewGuide",
                              groups=["NonPlaying"]),
-                    MenuItem(_("New Searc_h Feed"), "NewSearchFeed",
+                    MenuItem(_("New Search Podcast"), "NewSearchPodcast",
                              groups=["NonPlaying"]),
-                    MenuItem(_("New _Folder"), "NewFeedFolder",
+                    MenuItem(_("_New Folder"), "NewPodcastFolder",
                              Shortcut("n", MOD, SHIFT),
                              groups=["NonPlaying"]),
                     Separator(),
-                    MenuItem(_("Re_name"), "RenameSomething",
+                    MenuItem(_("Rename"), "RenameSomething",
                              groups=["RenameAllowed"],
-                             # groups=["FeedOrFolderSelected", "SiteSelected"],
-                             feed=_("Re_name Feed"),
-                             site=_("Re_name Website")),
-                    MenuItem(_("_Remove"), "RemoveSomething",
-                             Shortcut(BKSPACE, MOD),
+                             # groups=["PodcastOrFolderSelected", "SiteSelected"],
+                             feed=_("Rename Podcast"),
+                             site=_("Rename Source")),
+                    MenuItem(_("Remove"), "RemoveSomething",
                              groups=["RemoveAllowed"],
-                             # groups=["FeedsSelected", "SitesSelected"],
-                             feed=_("_Remove Feed"),
-                             feeds=_("_Remove Feeds"),
-                             folder=_("_Remove Folder"),
-                             folders=_("_Remove Folders"),
-                             site=_("_Remove Website"),
-                             sites=_("_Remove Websites")),
-                    MenuItem(_("_Update Feed"), "UpdateFeeds",
+                             # groups=["PodcastsSelected", "SitesSelected"],
+                             feed=_("Remove Podcast"),
+                             feeds=_("Remove Podcasts"),
+                             folder=_("Remove Folder"),
+                             folders=_("Remove Folders"),
+                             site=_("Remove Source"),
+                             sites=_("Remove Sources")),
+                    MenuItem(_("Update Podcast"), "UpdatePodcasts",
                              (Shortcut("r", MOD), Shortcut(F5)),
-                             groups=["FeedsSelected"],
-                             plural=_("_Update Feeds")),
-                    MenuItem(_("Update _All Feeds"), "UpdateAllFeeds",
+                             groups=["PodcastsSelected"],
+                             plural=_("Update Podcasts")),
+                    MenuItem(_("Update All Podcasts and Library"),
+                              "UpdateAllPodcasts",
                              Shortcut("r", MOD, SHIFT),
                              groups=["NonPlaying"]),
                     Separator(),
-                    MenuItem(_("_Import Feeds (OPML)"), "ImportFeeds",
+                    MenuItem(_("Import Podcasts (OPML)"), "ImportPodcasts",
                              groups=["NonPlaying"]),
-                    MenuItem(_("E_xport Feeds (OPML)"), "ExportFeeds",
+                    MenuItem(_("Export Podcasts (OPML)"), "ExportPodcasts",
                              groups=["NonPlaying"]),
                     Separator(),
-                    MenuItem(_("_Share with a Friend"), "ShareFeed",
-                             groups=["FeedSelected"]),
-                    MenuItem(_("Copy URL"), "CopyFeedURL",
-                             groups=["FeedSelected"]),
+                    MenuItem(_("Share with a Friend"), "SharePodcast",
+                             groups=["PodcastSelected"]),
+                    MenuItem(_("Copy URL"), "CopyPodcastURL",
+                             groups=["PodcastSelected"]),
                     ]),
 
             Menu(_("_Playlists"), "PlaylistsMenu", [
                     MenuItem(_("New _Playlist"), "NewPlaylist",
                              Shortcut("p", MOD),
                              groups=["NonPlaying"]),
-                    MenuItem(_("New Playlist Fol_der"), "NewPlaylistFolder",
+                    MenuItem(_("New Playlist Folder"), "NewPlaylistFolder",
                              Shortcut("p", MOD, SHIFT),
                              groups=["NonPlaying"]),
                     Separator(),
-                    MenuItem(_("Re_name Playlist"),"RenamePlaylist",
+                    MenuItem(_("Rename Playlist"),"RenamePlaylist",
                              groups=["PlaylistSelected"]),
-                    MenuItem(_("_Remove Playlist"),"RemovePlaylists",
-                             Shortcut(BKSPACE, MOD),
+                    MenuItem(_("Remove Playlist"),"RemovePlaylists",
                              groups=["PlaylistsSelected"],
-                             plural=_("_Remove Playlists"),
-                             folders=_("_Remove Playlist Folders"),
-                             folder=_("_Remove Playlist Folder")),
+                             plural=_("Remove Playlists"),
+                             folders=_("Remove Playlist Folders"),
+                             folder=_("Remove Playlist Folder")),
                     ]),
 
             Menu(_("P_layback"), "PlaybackMenu", [
-                    MenuItem(_("_Play"), "PlayPauseItem",
+                    MenuItem(_("Play"), "PlayPauseItem",
                              groups=["PlayPause"],
-                             play=_("_Play"),
-                             pause=_("_Pause")),
-                    MenuItem(_("_Stop"), "StopItem", Shortcut("d", MOD),
+                             play=_("Play"),
+                             pause=_("Pause")),
+                    MenuItem(_("Stop"), "StopItem", Shortcut("d", MOD),
                              groups=["Playing"]),
                     Separator(),
-                    MenuItem(_("_Next Item"), "NextItem",
+                    MenuItem(_("Next Item"), "NextItem",
                              Shortcut(RIGHT_ARROW, MOD),
                              groups=["Playing"]),
-                    MenuItem(_("_Previous Item"), "PreviousItem",
+                    MenuItem(_("Previous Item"), "PreviousItem",
                              Shortcut(LEFT_ARROW, MOD),
                              groups=["Playing"]),
                     Separator(),
-                    MenuItem(_("Skip _Forward"), "FastForward",
+                    MenuItem(_("Skip Forward"), "FastForward",
                              groups=["Playing"]),
-                    MenuItem(_("Skip _Back"), "Rewind",
+                    MenuItem(_("Skip Back"), "Rewind",
                              groups=["Playing"]),
                     Separator(),
-                    MenuItem(_("Volume _Up"), "UpVolume",
+                    MenuItem(_("Volume Up"), "UpVolume",
                              Shortcut(UP_ARROW, MOD)),
-                    MenuItem(_("Volume _Down"), "DownVolume",
+                    MenuItem(_("Volume Down"), "DownVolume",
                              Shortcut(DOWN_ARROW,MOD)),
                     Separator(),
                     MenuItem(_("_Fullscreen"), "Fullscreen",
@@ -319,49 +346,63 @@ def get_menu():
                     MenuItem(_("_Toggle Detached/Attached"), "ToggleDetach",
                              Shortcut("t", MOD),
                              groups=["PlayingVideo"]),
-                    Menu(_("S_ubtitles"), "SubtitlesMenu", [
+                    Menu(_("Subtitles"), "SubtitlesMenu", [
                             MenuItem(_("None Available"), "NoneAvailable",
                                      groups=["NeverEnabled"]),
                             Separator(),
                             MenuItem(_("Select a Subtitles File..."),
                                      "SubtitlesSelect",
-                                     groups=["PlayingVideo"])
+                                     groups=["PlayingLocalVideo"])
                             ]),
                     ]),
 
-            Menu(_("Convert"), "ConvertMenu", _get_convert_menu()),
+            Menu(_("Sorts"), "ViewMenu", _get_view_menu()),
+
+            Menu(_("_Convert"), "ConvertMenu", _get_convert_menu()),
 
             Menu(_("_Help"), "HelpMenu", [
-                    MenuItem(_("_About %(name)s",
-                               {'name': config.get(prefs.SHORT_APP_NAME)}),
+                    MenuItem(_("About %(name)s",
+                               {'name': app.config.get(prefs.SHORT_APP_NAME)}),
                              "About")
                     ])
             ])
 
     help_menu = mbar.get("HelpMenu")
-    if config.get(prefs.DONATE_URL):
-        help_menu.append(MenuItem(_("_Donate"), "Donate"))
+    if app.config.get(prefs.DONATE_URL):
+        help_menu.append(MenuItem(_("Donate"), "Donate"))
 
-    if config.get(prefs.HELP_URL):
-        help_menu.append(MenuItem(_("_Help"), "Help", Shortcut(F1)))
+    if app.config.get(prefs.HELP_URL):
+        help_menu.append(MenuItem(_("Help"), "Help", Shortcut(F1)))
     help_menu.append(Separator())
     help_menu.append(MenuItem(_("Diagnostics"), "Diagnostics"))
-    if config.get(prefs.BUG_REPORT_URL):
-        help_menu.append(MenuItem(_("Report a _Bug"), "ReportBug"))
-    if config.get(prefs.TRANSLATE_URL):
-        help_menu.append(MenuItem(_("_Translate"), "Translate"))
-    if config.get(prefs.PLANET_URL):
-        help_menu.append(MenuItem(_("_Planet Miro"), "Planet"))
+    if app.config.get(prefs.BUG_REPORT_URL):
+        help_menu.append(MenuItem(_("Report a Bug"), "ReportBug"))
+    if app.config.get(prefs.TRANSLATE_URL):
+        help_menu.append(MenuItem(_("Translate"), "Translate"))
+    if app.config.get(prefs.PLANET_URL):
+        help_menu.append(MenuItem(_("Planet Miro"), "Planet"))
+
+    if app.debugmode:
+        dev_menu = Menu(_("Dev"), "DevMenu", [
+                MenuItem(_("Profile Message"), "ProfileMessage"),
+                MenuItem(_("Profile Redraw"), "ProfileRedraw"),
+                MenuItem(_("Test Crash Reporter"), "TestCrashReporter"),
+                MenuItem(_("Test Soft Crash Reporter"),
+                    "TestSoftCrashReporter"),
+                MenuItem(_("Memory Stats"), "MemoryStats")
+                ])
+
+        mbar.menuitems.append(dev_menu)
     return mbar
 
 def _get_convert_menu():
     menu = list()
-    sections = videoconversion.conversion_manager.get_converters()
+    sections = conversions.conversion_manager.get_converters()
     for index, section in enumerate(sections):
         for converter in section[1]:
             handler_name = make_convert_handler(converter)
             item = MenuItem(converter.displayname, handler_name,
-                            groups=["PlayablesSelected"])
+                            groups=["LocalPlayablesSelected"])
             menu.append(item)
         if index+1 < len(sections):
             menu.append(Separator())
@@ -369,7 +410,6 @@ def _get_convert_menu():
     menu.append(MenuItem(_("Show Conversion Folder"), "RevealConversionFolder"))
     return menu
 
-
 def add_subtitle_encoding_menu(menubar, category_label, *encodings):
     """Helper method to set up the subtitles encoding menu.
 
@@ -447,6 +487,9 @@ def group_action_handler(action_prefix):
 def make_convert_handler(converter):
     return "ConvertItemTo-" + converter.identifier
 
+def make_column_toggle_handler(name):
+    return "ToggleColumn-" + name
+
 # File menu
 @action_handler("Open")
 def on_open():
@@ -456,6 +499,10 @@ def on_open():
 def on_new_download():
     app.widgetapp.new_download()
 
+@action_handler("AddFiles")
+def on_add_files():
+    app.widgetapp.add_files()
+
 @action_handler("CheckVersion")
 def on_check_version():
     app.widgetapp.check_version()
@@ -464,9 +511,9 @@ def on_check_version():
 def on_remove_items():
     app.widgetapp.remove_items()
 
-@action_handler("EditItem")
-def on_rename_item():
-    app.widgetapp.edit_item()
+@action_handler("EditItems")
+def on_edit_items():
+    app.widgetapp.edit_items()
 
 @action_handler("SaveItem")
 def on_save_item():
@@ -492,53 +539,69 @@ def on_edit_preferences():
 def on_quit():
     app.widgetapp.quit()
 
-# Feeds menu
-@action_handler("NewFeed")
-def on_new_feed():
+# Podcasts menu
+@action_handler("NewPodcast")
+def on_new_podcast():
     app.widgetapp.add_new_feed()
 
 @action_handler("NewGuide")
-def on_new_guidel():
+def on_new_guide():
     app.widgetapp.add_new_guide()
 
-@action_handler("NewSearchFeed")
-def on_new_search_feed():
+@action_handler("NewSearchPodcast")
+def on_new_search_podcast():
     app.widgetapp.add_new_search_feed()
 
-@action_handler("NewFeedFolder")
-def on_new_feed_folder():
+@action_handler("NewPodcastFolder")
+def on_new_podcast_folder():
     app.widgetapp.add_new_feed_folder()
 
+@action_handler("WatchAFolder")
+def on_watch_a_folder():
+    app.widgetapp.add_new_watched_folder()
+
+@action_handler("SearchAllMyFiles")
+def on_search_all_my_files():
+    app.widgetapp.import_search_all_my_files()
+
+@action_handler("SearchInAFolder")
+def on_search_in_a_folder():
+    app.widgetapp.import_search_in_folder()
+
+@action_handler("ChooseFiles")
+def on_choose_files():
+    app.widgetapp.import_choose_files()
+
 @action_handler("RenameSomething")
-def on_rename_feed():
+def on_rename_podcast():
     app.widgetapp.rename_something()
 
 @action_handler("RemoveSomething")
-def on_remove_feeds():
+def on_remove_podcasts():
     app.widgetapp.remove_something()
 
-@action_handler("UpdateFeeds")
-def on_update_feeds():
+@action_handler("UpdatePodcasts")
+def on_update_podcasts():
     app.widgetapp.update_selected_feeds()
 
-@action_handler("UpdateAllFeeds")
-def on_update_all_feeds():
+@action_handler("UpdateAllPodcasts")
+def on_update_all_podcasts():
     app.widgetapp.update_all_feeds()
 
-@action_handler("ImportFeeds")
-def on_import_feeds():
+@action_handler("ImportPodcasts")
+def on_import_podcasts():
     app.widgetapp.import_feeds()
 
-@action_handler("ExportFeeds")
-def on_export_feeds():
+@action_handler("ExportPodcasts")
+def on_export_podcasts():
     app.widgetapp.export_feeds()
 
-@action_handler("ShareFeed")
-def on_share_feed():
+@action_handler("SharePodcast")
+def on_share_podcast():
     app.widgetapp.share_feed()
 
-@action_handler("CopyFeedURL")
-def on_copy_feed_url():
+@action_handler("CopyPodcastURL")
+def on_copy_podcast_url():
     app.widgetapp.copy_feed_url()
 
 # Playlists menu
@@ -556,7 +619,7 @@ def on_rename_playlist():
 
 @action_handler("RemovePlaylists")
 def on_remove_playlists():
-    app.widgetapp.remove_current_playlist()
+    app.widgetapp.remove_something()
 
 # Playback menu
 @action_handler("PlayPauseItem")
@@ -593,7 +656,7 @@ def on_down_volume():
 
 @action_handler("Fullscreen")
 def on_fullscreen():
-    app.widgetapp.on_fullscreen_clicked()
+    app.playback_manager.toggle_fullscreen()
 
 @action_handler("ToggleDetach")
 def on_toggle_detach():
@@ -610,6 +673,11 @@ def on_subtitle_encoding(converter):
     else:
         app.playback_manager.select_subtitle_encoding(converter)
 
+# Sorts menu
+@group_action_handler("ToggleColumn")
+def on_toggle_column(name):
+    app.widgetapp.toggle_column(name)
+
 # Help menu
 @action_handler("About")
 def on_about():
@@ -617,11 +685,11 @@ def on_about():
 
 @action_handler("Donate")
 def on_donate():
-    app.widgetapp.open_url(config.get(prefs.DONATE_URL))
+    app.widgetapp.open_url(app.config.get(prefs.DONATE_URL))
 
 @action_handler("Help")
 def on_help():
-    app.widgetapp.open_url(config.get(prefs.HELP_URL))
+    app.widgetapp.open_url(app.config.get(prefs.HELP_URL))
 
 @action_handler("Diagnostics")
 def on_diagnostics():
@@ -629,15 +697,39 @@ def on_diagnostics():
 
 @action_handler("ReportBug")
 def on_report_bug():
-    app.widgetapp.open_url(config.get(prefs.BUG_REPORT_URL))
+    app.widgetapp.open_url(app.config.get(prefs.BUG_REPORT_URL))
 
 @action_handler("Translate")
 def on_translate():
-    app.widgetapp.open_url(config.get(prefs.TRANSLATE_URL))
+    app.widgetapp.open_url(app.config.get(prefs.TRANSLATE_URL))
 
 @action_handler("Planet")
 def on_planet():
-    app.widgetapp.open_url(config.get(prefs.PLANET_URL))
+    app.widgetapp.open_url(app.config.get(prefs.PLANET_URL))
+
+@action_handler("ProfileMessage")
+def on_profile_message():
+    app.widgetapp.setup_profile_message()
+
+@action_handler("ProfileRedraw")
+def on_profile_redraw():
+    app.widgetapp.profile_redraw()
+
+class TestIntentionalCrash(Exception):
+    pass
+
+@action_handler("TestCrashReporter")
+def on_test_crash_reporter():
+    raise TestIntentionalCrash("intentional error here")
+
+@action_handler("TestSoftCrashReporter")
+def on_test_soft_crash_reporter():
+    app.widgetapp.handle_soft_failure("testing soft crash reporter",
+            'intentional error', with_exception=False)
+
+@action_handler("MemoryStats")
+def on_memory_stats():
+    app.widgetapp.memory_stats()
 
 def generate_action_groups(menu_structure):
     """Takes a menu structure and returns a map of action group name to
@@ -666,25 +758,38 @@ class MenuStateManager(signals.SignalEmitter):
         signals.SignalEmitter.__init__(self)
         self.create_signal('enabled-changed')
         self.create_signal('radio-group-changed')
+        self.create_signal('checked-changed')
         self.enabled_groups = set(['AlwaysOn'])
         self.states = {}
         self.play_pause_state = "play"
         self.subtitle_encoding_enabled = False
 
     def reset(self):
-        self.states = { "feed": [], "feeds": [],
-                        "folder": [], "folders": [], 
-                        "site": [], "sites": [],
-                        "plural": [] }
+        self.states = {"feed": [],
+                       "feeds": [],
+                       "folder": [],
+                       "folders": [],
+                       "site": [],
+                       "sites": [],
+                       "plural": []}
+        self.plural = False
         self.enabled_groups = set(['AlwaysOn'])
         if app.playback_manager.is_playing:
             self.enabled_groups.add('PlayPause')
             self.enabled_groups.add('Playing')
+            item = app.playback_manager.get_playing_item()
+            if item and item.remote:
+                self.enabled_groups.add('LocalPlayableSelected_PlayPause')
+                self.enabled_groups.add('LocalPlayableSelected_PlayPause')
+            self.enabled_groups.add('PlayablesSelected_PlayPause')
+            self.enabled_groups.add('PlayablesSelected_PlayPause')
             if app.playback_manager.is_playing_audio:
                 # if it's playing audio, then we allow the user to do other
                 # things just as if the window was detached
                 self.enabled_groups.add('NonPlaying')
             else:
+                if not item.remote:
+                    self.enabled_groups.add('PlayingLocalVideo')
                 self.enabled_groups.add('PlayingVideo')
             if app.playback_manager.detached_window is not None:
                 self.enabled_groups.add('NonPlaying')
@@ -703,16 +808,17 @@ class MenuStateManager(signals.SignalEmitter):
 
         ``selected_feeds`` is a list of ChannelInfo objects.
         """
-        self.enabled_groups.add('FeedsSelected')
+        self.enabled_groups.add('PodcastsSelected')
         self.enabled_groups.add("RemoveAllowed")
         if len(selected_feeds) == 1:
-            if selected_feeds[0].is_folder:
+            if (hasattr(selected_feeds[0], 'is_folder') and
+                    selected_feeds[0].is_folder):
                 self.states["folder"].append("RemoveSomething")
             else:
                 self.states["feed"].append("RemoveSomething")
                 self.states["feed"].append("RenameSomething")
-                self.enabled_groups.add('FeedSelected')
-            self.enabled_groups.add('FeedOrFolderSelected')
+                self.enabled_groups.add('PodcastSelected')
+            self.enabled_groups.add('PodcastOrFolderSelected')
             self.enabled_groups.add("RenameAllowed")
         else:
             selected_folders = [s for s in selected_feeds if s.is_folder]
@@ -722,23 +828,31 @@ class MenuStateManager(signals.SignalEmitter):
                 self.states["plural"].append("RemoveSomething")
                 self.states["feeds"].append("RemoveSomething")
                 self.states["feeds"].append("RenameSomething")
-            self.states["plural"].append("UpdateFeeds")
+            self.states["plural"].append("UpdatePodcasts")
 
     def _handle_site_selection(self, selected_sites):
         """Handle the user selecting things in the site list.
         selected_sites is a list of GuideInfo objects
         """
+        has_stores = bool([True for info in selected_sites if info.store])
         self.enabled_groups.add('SitesSelected')
-        self.enabled_groups.add("RemoveAllowed")
+        if not has_stores:
+            self.enabled_groups.add("RemoveAllowed")
         if len(selected_sites) == 1:
             self.enabled_groups.add('SiteSelected')
-            self.enabled_groups.add("RenameAllowed")
-            self.states["site"].append("RemoveSomething")
-            self.states["site"].append("RenameSomething")
-        else:
+            if not has_stores:
+                self.enabled_groups.add("RenameAllowed")
+                self.states["site"].append("RemoveSomething")
+                self.states["site"].append("RenameSomething")
+        elif not has_stores:
             self.states["sites"].append("RemoveSomething")
             self.states["sites"].append("RenameSomething")
 
+    def _handle_connect_selection(self, selected_devices):
+        selected_info = selected_devices[0]
+        if selected_info.type == u'device':
+            self.enabled_groups.add('DeviceSelected')
+
     def _handle_playlist_selection(self, selected_playlists):
         self.enabled_groups.add('PlaylistsSelected')
         if len(selected_playlists) == 1:
@@ -760,17 +874,24 @@ class MenuStateManager(signals.SignalEmitter):
         pass
 
     def _update_menus_for_selected_tabs(self):
-        selection_type, selected_tabs = app.tab_list_manager.get_selection()
-        if selection_type is None:
+        try:
+            selection_type, selected_tabs = app.tabs.selection
+        except errors.WidgetActionError:
+            return
+        if len(selected_tabs) == 1:
+            app.menu_manager._update_view_menu()
+        if selection_type is None or selected_tabs[0].type == u'tab':
             pass
-        elif selection_type in ('feed', 'audio-feed'):
+        elif selection_type == 'feed':
             app.menu_manager._handle_feed_selection(selected_tabs)
         elif selection_type == 'playlist':
             app.menu_manager._handle_playlist_selection(selected_tabs)
         elif selection_type in ('static', 'library'):
             app.menu_manager._handle_static_tab_selection(selected_tabs)
-        elif selection_type == 'site':
+        elif selection_type in ('site', 'store'):
             app.menu_manager._handle_site_selection(selected_tabs)
+        elif selection_type == 'connect':
+            app.menu_manager._handle_connect_selection(selected_tabs)
         else:
             raise ValueError("Unknown tab list type: %s" % selection_type)
 
@@ -789,25 +910,48 @@ class MenuStateManager(signals.SignalEmitter):
         selected_items = app.item_list_controller_manager.get_selection()
         downloaded = False
         has_audio = False
+        is_remote = False
         for item in selected_items:
             if item.downloaded:
                 downloaded = True
             if item.file_type == 'audio':
                 has_audio = True
+            if item.remote:
+                is_remote = True
+
+        if selected_items and not is_remote:
+            if len(selected_items) == 1:
+                self.enabled_groups.add('LocalItemSelected')
+            else:
+                self.states['plural'].append('EditItems')
+            self.enabled_groups.add('LocalItemsSelected')
+
         if downloaded:
+            if not is_remote:
+                self.enabled_groups.add('LocalPlayablesSelected')
+                self.enabled_groups.add('LocalPlayablesSelected_PlayPause')
             self.enabled_groups.add('PlayablesSelected')
+            self.enabled_groups.add('PlayablesSelected_PlayPause')
             if not has_audio:
                 self.enabled_groups.add('PlayableVideosSelected')
             if len(selected_items) == 1:
+                if not is_remote:
+                    self.enabled_groups.add('LocalPlayableSelected')
+                    self.enabled_groups.add('LocalPlayableSelected_PlayPause')
                 self.enabled_groups.add('PlayableSelected')
+                self.enabled_groups.add('PlayableSelected_PlayPause')
             else:
                 self.states["plural"].append("RemoveItems")
 
-        if len(app.item_list_controller_manager.get_current_playlist()) > 0:
+        can_play = app.item_list_controller_manager.can_play_items()
+        if can_play:
             self.enabled_groups.add('PlayPause')
-            app.widgetapp.window.videobox.handle_new_selection(True)
-        else:
-            app.widgetapp.window.videobox.handle_new_selection(False)
+            if not is_remote:
+                self.enabled_groups.add('LocalPlayableSelected_PlayPause')
+                self.enabled_groups.add('LocalPlayablesSelected_PlayPause')
+            self.enabled_groups.add('PlayableSelected_PlayPause')
+            self.enabled_groups.add('PlayablesSelected_PlayPause')
+        app.widgetapp.window.videobox.handle_new_selection(can_play)
 
     def update_menus(self):
         self.reset()
@@ -815,3 +959,27 @@ class MenuStateManager(signals.SignalEmitter):
         self._update_menus_for_selected_items()
         self._set_play_pause()
         self.emit('enabled-changed')
+
+    def _update_view_menu(self):
+        display = app.display_manager.get_current_display()
+        # using hasattr because not all displays have ids and types
+        # TODO: refactor the display type / id system
+        if not (hasattr(display, 'type') and hasattr(display, 'id')):
+            return
+
+        enabled = set(app.widget_state.get_sorts_enabled(display.type, display.id))
+        checks = dict(('ToggleColumn-' + column, column in enabled)
+            for column in WidgetStateStore.get_columns())
+        self.emit('checked-changed', 'ListView', checks)
+
+        for column in WidgetStateStore.get_columns_available(display.type):
+            self.enabled_groups.add('column-%s' % column)
+
+def _get_view_menu():
+    menu = list()
+    for name in sorted(WidgetStateStore.get_columns(), key=COLUMN_LABELS.get):
+        groups = ['column-%s' % name]
+        label = COLUMN_LABELS[name]
+        handler_name = make_column_toggle_handler(name)
+        menu.append(CheckMenuItem(label, handler_name, 'ListView', groups=groups))
+    return menu
