@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,7 +46,6 @@ from cStringIO import StringIO
 import pycurl
 
 from miro import app
-from miro import config
 from miro import download_utils
 from miro import eventloop
 from miro import fileutil
@@ -56,6 +56,7 @@ from miro import signals
 from miro import util
 from miro.gtcache import gettext as _
 from miro.xhtmltools import url_encode_dict, multipart_encode
+from miro.plat import utils
 from miro.plat.resources import get_osname
 from miro.net import NetworkError, ConnectionError, ConnectionTimeout
 
@@ -65,9 +66,9 @@ MAX_AUTH_ATTEMPTS = 5
 _logged_noproxy_error = False
 
 def user_agent():
-    return "%s/%s (%s; %s)" % (config.get(prefs.SHORT_APP_NAME),
-            config.get(prefs.APP_VERSION),
-            config.get(prefs.PROJECT_URL),
+    return "%s/%s (%s; %s)" % (app.config.get(prefs.SHORT_APP_NAME),
+            app.config.get(prefs.APP_VERSION),
+            app.config.get(prefs.PROJECT_URL),
             get_osname())
 
 def _proxy_auth_url():
@@ -75,8 +76,8 @@ def _proxy_auth_url():
 
     # we hack things by using a "proxy" scheme.  This keeps it separate from
     # regular HTTP passwords
-    return 'proxy://%s:%s/' % (config.get(prefs.HTTP_PROXY_HOST),
-            config.get(prefs.HTTP_PROXY_PORT))
+    return 'proxy://%s:%s/' % (app.config.get(prefs.HTTP_PROXY_HOST),
+            app.config.get(prefs.HTTP_PROXY_PORT))
 
 def trap_call(when, function, *args, **kwargs):
     """Version of trap_call for the libcurl thread.
@@ -105,6 +106,11 @@ class ServerClosedConnection(HTTPError):
         HTTPError.__init__(self, _('%(host)s closed connection',
                                    {"host": host}))
 
+class EmptyResponse(HTTPError):
+    def __init__(self, host):
+        HTTPError.__init__(self, _('%(host)s gave us an empty response',
+                                   {"host": host}))
+
 class PossiblyTemporaryError(HTTPError):
     def __init__(self, status):
         self.friendlyDescription = _("Host returned %(status)s",
@@ -125,8 +131,8 @@ class ProxyAuthorizationFailed(HTTPError):
     def __init__(self):
         HTTPError.__init__(self,
                 _('Authorization failed for proxy: %(host)s:%(port)s',
-                               {"host": config.get(prefs.HTTP_PROXY_HOST),
-                               "port": config.get(prefs.HTTP_PROXY_PORT)}))
+                               {"host": app.config.get(prefs.HTTP_PROXY_HOST),
+                               "port": app.config.get(prefs.HTTP_PROXY_PORT)}))
 
 class UnexpectedStatusCode(HTTPError):
     def __init__(self, code):
@@ -156,7 +162,7 @@ class MalformedURL(NetworkError):
 class UnknownHostError(NetworkError):
     """A file: URL doesn't exist"""
     def __init__(self, host):
-        NetworkError.__init__(self, _('Unknow Host'),
+        NetworkError.__init__(self, _('Unknown Host'),
             _('The domainname "%(host)s" couldn\'t be found', {"host": host}))
 
 class FileURLNotFoundError(NetworkError):
@@ -176,8 +182,8 @@ class WriteError(NetworkError):
     """
 
     def __init__(self, path):
-        msg = _("Could not write to %(filename)s") % \
-            {"filename": util.stringify(self.filename)}
+        msg = _("Could not write to %(filename)s",
+            {"filename": util.stringify(path)})
         NetworkError.__init__(self, _('Write error'), msg)
 
 class TransferOptions(object):
@@ -200,6 +206,7 @@ class TransferOptions(object):
         self.post_vars = post_vars
         self.post_files = post_files
         self.write_file = write_file
+        self.requires_cookies = False
         self.head_request = False
         self.invalid_url = False
         # _cancel_on_body_data is an internal attribute used for grab_headers.
@@ -247,6 +254,11 @@ class TransferOptions(object):
         handle.setopt(pycurl.NOPROGRESS, 1)
         handle.setopt(pycurl.NOSIGNAL, 1)
         handle.setopt(pycurl.CONNECTTIMEOUT, net.SOCKET_CONNECT_TIMEOUT)
+        if self.requires_cookies:
+            # we don't do this for every request, since on OS X this generates
+            # a new cookies.txt file
+            handle.setopt(pycurl.COOKIEFILE, utils.get_cookie_path())
+
         # The following 2 settings makes it so that if we don't receive any
         # data after SOCKET_READ_TIMEOUT, the transfer ends in an error
         handle.setopt(pycurl.LOW_SPEED_LIMIT, 1)
@@ -258,15 +270,15 @@ class TransferOptions(object):
         return handle
 
     def _setup_proxy(self, handle):
-        if not config.get(prefs.HTTP_PROXY_ACTIVE):
+        if not app.config.get(prefs.HTTP_PROXY_ACTIVE):
             return
 
         # FIXME honor prefs.HTTP_PROXY_SCHEME
-        handle.setopt(pycurl.PROXY, str(config.get(prefs.HTTP_PROXY_HOST)))
+        handle.setopt(pycurl.PROXY, str(app.config.get(prefs.HTTP_PROXY_HOST)))
         handle.setopt(pycurl.PROXYPORT,
-                int(config.get(prefs.HTTP_PROXY_PORT)))
+                int(app.config.get(prefs.HTTP_PROXY_PORT)))
 
-        ignore_hosts = config.get(prefs.HTTP_PROXY_IGNORE_HOSTS)
+        ignore_hosts = app.config.get(prefs.HTTP_PROXY_IGNORE_HOSTS)
         if ignore_hosts:
             try:
                 handle.setopt(pycurl.NOPROXY, ','.join(
@@ -326,6 +338,7 @@ class CurlTransfer(object):
         self.errback = errback
         self.auth_attempts = {'http': 0, 'proxy': 0}
         self.canceled = False
+        self.last_url = None
 
         self.stats = TransferStats()
         self._lookup_auth()
@@ -342,6 +355,12 @@ class CurlTransfer(object):
         self.resume_from = 0
         self.out_headers = {}
         self.status_code = None
+        self.trying_head_request = False
+        self.saw_head_success = False
+
+    def _send_new_request(self):
+        self._reset_transfer_data()
+        curl_manager.add_transfer(self)
 
     def start(self):
         if self.options.invalid_url:
@@ -406,14 +425,13 @@ class CurlTransfer(object):
             existing_auth = httpauth.find_http_auth(url, auth_header)
             if existing_auth is not None:
                 self.http_auth = existing_auth
-                self._reset_transfer_data()
-                curl_manager.add_transfer(self)
+                self._send_new_request()
                 return
         try:
             httpauth.ask_for_http_auth(self._ask_for_http_auth_callback,
                     url, auth_header, location)
-        except ValueError, e:
-            logging.warn("ValueError when parsing auth header: %s", e)
+        except (AssertionError, ValueError), e:
+            logging.warn("Error when parsing auth header: %s", e)
             self.handle_auth_failure()
 
     def _ask_for_http_auth_callback(self, auth):
@@ -426,26 +444,41 @@ class CurlTransfer(object):
                 self.http_auth = auth
             else:
                 self.proxy_auth = auth
-            self._reset_transfer_data()
-            curl_manager.add_transfer(self)
+            self._send_new_request()
 
     def build_handle(self):
         """Build a libCURL handle.  This should only be called inside the
         LibCURLManager thread.
         """
         self.handle = self.options.build_handle(self.out_headers)
+        # don't authenticate SSL certificates see #15180
+        self.handle.setopt(pycurl.SSL_VERIFYPEER, 0)
+
         self._setup_http_auth()
         self._setup_proxy_auth()
         if self.options._cancel_on_body_data:
             self.handle.setopt(pycurl.WRITEFUNCTION, self._write_func_abort)
         elif self.options.write_file is not None:
-            self._open_file()
-            self.handle.setopt(pycurl.WRITEDATA, self._filehandle)
+            if not self.saw_head_success:
+                # try a HEAD request first to see if the request will work.
+                # This avoids writing error responses to our file.  It also
+                # avoids the issue of RESUME_FROM being applied to the error
+                # response.
+                self.handle.setopt(pycurl.NOBODY, 1)
+                self.trying_head_request = True
+            else:
+                self.handle.setopt(pycurl.URL, self.last_url)
+                self._open_file()
+                self.handle.setopt(pycurl.WRITEDATA, self._filehandle)
         elif self.content_check_callback is not None:
             self.handle.setopt(pycurl.WRITEFUNCTION, self._call_content_check)
         else:
             self.handle.setopt(pycurl.WRITEFUNCTION, self.buffer.write)
         self.handle.setopt(pycurl.HEADERFUNCTION, self.header_func)
+        if self.should_debug_request():
+            logging.warn("debugging request: %s", self.options.url)
+            self.handle.setopt(pycurl.VERBOSE, 1)
+            self.handle.setopt(pycurl.DEBUGFUNCTION, self.debug_func)
 
     def _lookup_auth(self):
         """Lookup existing HTTP passwords to use.
@@ -479,10 +512,10 @@ class CurlTransfer(object):
             self.handle.setopt(pycurl.PROXYUSERPWD, user_pwd)
         else:
             # fallback on system auth info
-            if config.get(prefs.HTTP_PROXY_AUTHORIZATION_ACTIVE):
+            if app.config.get(prefs.HTTP_PROXY_AUTHORIZATION_ACTIVE):
                 self.handle.setopt(pycurl.PROXYUSERPWD, '%s:%s' % (
-                    str(config.get(prefs.HTTP_PROXY_AUTHORIZATION_USERNAME)),
-                    str(config.get(prefs.HTTP_PROXY_AUTHORIZATION_PASSWORD))))
+                    str(app.config.get(prefs.HTTP_PROXY_AUTHORIZATION_USERNAME)),
+                    str(app.config.get(prefs.HTTP_PROXY_AUTHORIZATION_PASSWORD))))
 
     def _call_content_check(self, data):
         self.buffer.write(data)
@@ -507,15 +540,49 @@ class CurlTransfer(object):
         try:
             self._filehandle = fileutil.open_file(self.options.write_file, mode)
         except IOError:
-            raise WriteError(self.filename)
+            raise WriteError(self.options.write_file)
+
+    def should_debug_request(self):
+        # return True here to debug HTTP requests in the log file
+        return False
+
+
+    def debug_func(self, type, msg):
+        type_map = {
+                pycurl.INFOTYPE_HEADER_IN: 'header-in',
+                pycurl.INFOTYPE_HEADER_OUT: 'header-out',
+                pycurl.INFOTYPE_DATA_IN: 'data-in',
+                pycurl.INFOTYPE_DATA_OUT: 'data-out',
+                pycurl.INFOTYPE_TEXT: 'text',
+        }
+        type_str = type_map.get(type, type)
+        logging.warn("libcurl debug (%s) %r", type_str, msg)
+
 
     def header_func(self, line):
         line = line.strip()
         if line.startswith("HTTP"):
-            curl_manager.call_after_perform(self.check_status_line)
+            # we can't use self.header.getinfo() because we're inside the
+            # perform call.  However, we also can't use call_after_perform()
+            # because that call might happen after a redirect, which might
+            # make us miss the temporary redirect.  So parse the status
+            # ourselves.
+            try:
+                code = self.status_code = int(line.split()[1])
+            except Exception, e:
+                logging.warn("Error parsing status line (%r): %s", line, e)
+                self.status_code = 200
+            else:
+                if 300 <= code <= 399 and code != 301:
+                    self.saw_temporary_redirect = True
             return
         if line == '':
-            if 'location' in self.headers:
+            if self.status_code == 100:
+                # server sent a HTTP 100 continue.  we ditch the data
+                # we got so far and ignore this "empty header" because
+                # it's not the response we're looking for.
+                self.status_code = None
+            elif 'location' in self.headers:
                 # doing a redirect, clear out the headers
                 self.headers = {}
             elif not self.headers_finished:
@@ -550,12 +617,6 @@ class CurlTransfer(object):
                     'httpclient header callback',
                     args=(self._make_callback_info(),))
 
-    def check_status_line(self):
-        code = self.handle.getinfo(pycurl.RESPONSE_CODE)
-        if 300 <= code <= 399 and code != 301:
-            self.saw_temporary_redirect = True
-        self.status_code = code
-
     def check_response_code(self, code):
         expected_codes = set([200])
         if self.options.resume:
@@ -587,16 +648,31 @@ class CurlTransfer(object):
 
     def on_finished(self):
         info = self._make_callback_info()
+        self.last_url = self.handle.getinfo(pycurl.EFFECTIVE_URL)
         if self.options.write_file is None:
             info['body'] = self.buffer.getvalue()
         if self.check_response_code(info['status']):
-            self.call_callback(info)
+            if not self.trying_head_request:
+                self.call_callback(info)
+            else:
+                # we tried a HEAD request and it worked, now we can do the
+                # transfer for real
+                self._send_new_request()
+                self.saw_head_success = True
         elif info['status'] == 401:
             self.handle_http_auth()
+        elif info['status'] in (405, 501):
+            if self.trying_head_request:
+                # server didn't like the HEAD request, so just try the GET
+                self._send_new_request()
+                self.saw_head_success = True
+            else:
+                self.call_errback(UnexpectedStatusCode(info['status']))
         elif info['status'] == 407:
             self.handle_proxy_auth()
         elif info['status'] >= 500 and info['status'] < 600:
-            logging.info("possibly temporary error")
+            logging.info("httpclient: possibly temporary http error: HTTP %s",
+                         info['status'])
             self.call_errback(PossiblyTemporaryError(info['status']))
         else:
             self.call_errback(UnexpectedStatusCode(info['status']))
@@ -654,6 +730,8 @@ class CurlTransfer(object):
             error = ConnectionError(self.options.host)
         elif code == pycurl.E_PARTIAL_FILE:
             error = ServerClosedConnection(self.options.host)
+        elif code == pycurl.E_GOT_NOTHING:
+            error = EmptyResponse(self.options.host)
         elif code == pycurl.E_HTTP_RANGE_ERROR:
             error = ResumeFailed(self.options.host)
         elif code == pycurl.E_TOO_MANY_REDIRECTS:
@@ -758,8 +836,9 @@ class LibCURLManager(eventloop.SimpleEventLoop):
         self.after_perform_callbacks = []
 
     def start(self):
-        self.thread = threading.Thread(target=self.loop,
-                name="LibCURL Event Loop")
+        self.thread = threading.Thread(target=utils.thread_body,
+                                       args=[self.loop],
+                                       name="LibCURL Event Loop")
         self.thread.start()
 
     def stop(self):
@@ -944,9 +1023,7 @@ def _grab_file_url(url, callback, errback, default_mime_type):
     else:
         try:
             data = f.read()
-        except (SystemExit, KeyboardInterrupt):
-            raise
-        except:
+        except IOError:
             eventloop.add_idle(errback, 'grab file url errback',
                     args=(FileURLReadError(path),))
         else:
