@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -66,13 +67,14 @@ class NetworkError(Exception):
     """Base class for all errors that will be passed to errbacks from get_url
     and friends.  NetworkErrors can be display in 2 ways:
 
-    getFriendlyDescription() -- short, newbie friendly description 
+    getFriendlyDescription() -- short, newbie friendly description
     getLongDescription() -- detailed description
     """
     def __init__(self, shortDescription, longDescription=None):
         if longDescription is None:
             longDescription = shortDescription
-        self.friendlyDescription = _("Error: %(msg)s", {"msg": shortDescription})
+        self.friendlyDescription = _("Error: %(msg)s",
+                                     {"msg": shortDescription})
         self.longDescription = longDescription
 
     def getFriendlyDescription(self):
@@ -83,7 +85,7 @@ class NetworkError(Exception):
 
     def __str__(self):
         return "%s: %s -- %s" % (self.__class__,
-                util.stringify(self.getFriendlyDescription()), 
+                util.stringify(self.getFriendlyDescription()),
                 util.stringify(self.getLongDescription()))
 
 class ConnectionError(NetworkError):
@@ -106,7 +108,8 @@ def trap_call(object, function, *args, **kwargs):
     """Convenience function do a trapcall.trap_call, where when is
     'While talking to the network'
     """
-    return trapcall.time_trap_call("Calling %s on %s" % (function, object), function, *args, **kwargs)
+    return trapcall.time_trap_call("Calling %s on %s" % (function, object),
+                                   function, *args, **kwargs)
 
 class NetworkBuffer(object):
     """Responsible for storing incomming network data and doing some basic
@@ -224,14 +227,31 @@ class AsyncSocket(object):
         self.lastClock = clock()
         if self.readTimeout is not None:
             return
-        self.readTimeout = eventloop.add_timeout(SOCKET_INITIAL_READ_TIMEOUT, self.onReadTimeout,
-                "AsyncSocket.onReadTimeout")
+        self.readTimeout = eventloop.add_timeout(SOCKET_INITIAL_READ_TIMEOUT,
+                                                 self.onReadTimeout,
+                                                 "AsyncSocket.onReadTimeout")
 
     def stopReadTimeout(self):
         if self.readTimeout is not None:
             self.readTimeout.cancel()
             self.readTimeout = None
 
+    def _pick_address(self, addresses):
+        """Pick the best entry to use from a list of addresses
+        
+        :param addresses: list of address tuples returned by getaddrinfo()
+        :returns: one of the tuples, or None if no address could be found
+        """
+        if util.use_ipv6(): # prefer ipv6 if possible
+            for entry in addresses:
+                if entry[0] == socket.AF_INET6:
+                    return entry
+        # fall back on ipv4
+        for entry in addresses:
+            if entry[0] == socket.AF_INET:
+                return entry
+        return None
+
     def open_connection(self, host, port, callback, errback,
                         disable_read_timeout=None):
         """Open a connection.  On success, callback will be called with this
@@ -241,35 +261,56 @@ class AsyncSocket(object):
             self.disable_read_timeout = disable_read_timeout
         self.name = "Outgoing %s:%s" % (host, port)
 
-        try:
-            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        except socket.error, e:
-            trap_call(self, errback, ConnectionError(e[1]))
-            return
-        self.socket.setblocking(0)
         self.connectionErrback = errback
-        def handleGetHostByNameException(e):
-            trap_call(self, errback, ConnectionError(e[1] + " (host: %s)" % host))
-        def onAddressLookup(address):
-            if self.socket is None:
-                # the connection was closed while we were calling gethostbyname
+        def handleGetAddrInfoException(e):
+            if self.connectionErrback is None:
+                # called connectionErrback while we were waiting for
+                # getaddrinfo
+                return
+            trap_call(self, errback,
+                      ConnectionError(e[1] + " (host: %s)" % host))
+        def createSocketHandle(family):
+            try:
+                self.socket = socket.socket(family, socket.SOCK_STREAM)
+            except socket.error, e:
+                trap_call(self, errback, ConnectionError(e[1]))
+                return
+
+            self.socket.setblocking(0)
+            return self.socket
+
+        def onAddressLookup(addresses):
+            if self.connectionErrback is None:
+                # called connectionErrback while we were waiting for
+                # getaddrinfo
+                return
+            entry = self._pick_address(addresses)
+            if entry is None:
+                # FIXME - wtf kind of user message is this?  it's too
+                # technical and there's no way a user would know what
+                # to do about it.
+                msg = _("Couldn't find address family to use")
+                trap_call(self, errback, ConnectionError(msg))
                 return
             try:
-                rv = self.socket.connect_ex((address, port))
+                self.socket = socket.socket(entry[0], socket.SOCK_STREAM)
+            except socket.error, e:
+                trap_call(self, errback, ConnectionError(e[1]))
+                return
+            self.socket.setblocking(0)
+            try:
+                rv = self.socket.connect_ex(entry[4])
             except socket.gaierror:
                 trap_call(self, errback, ConnectionError('gaierror'))
                 return
             if rv in (0, errno.EINPROGRESS, errno.EWOULDBLOCK):
                 eventloop.add_write_callback(self.socket, onWriteReady)
                 self.socketConnectTimeout = eventloop.add_timeout(
-                        SOCKET_CONNECT_TIMEOUT, onWriteTimeout,
+                         SOCKET_CONNECT_TIMEOUT, onWriteTimeout,
                         "socket connect timeout")
             else:
-                try:
-                    msg = errno.errorcode[rv]
-                except KeyError:
-                    msg = "Unknown connection error: %s" % rv
-                trap_call(self, errback, ConnectionError(msg))
+                fullmsg = "Connection failed"
+                trap_call(self, errback, ConnectionError(fullmsg))
         def onWriteReady():
             eventloop.remove_write_callback(self.socket)
             self.socketConnectTimeout.cancel()
@@ -284,10 +325,12 @@ class AsyncSocket(object):
             eventloop.remove_write_callback(self.socket)
             trap_call(self, errback, ConnectionTimeout(host))
             self.connectionErrback = None
-        eventloop.call_in_thread(onAddressLookup, handleGetHostByNameException,
-                socket.gethostbyname, "getHostByName - %s" % host, host)
+        eventloop.call_in_thread(onAddressLookup, handleGetAddrInfoException,
+                                 socket.getaddrinfo,
+                                 "getAddrInfo - %s:%s" % (host, port),
+                                 host, port)
 
-    def acceptConnection(self, host, port, callback, errback):
+    def accept_connection(self, family, host, port, callback, errback):
         def finishAccept():
             eventloop.remove_read_callback(self.socket)
             (self.socket, addr) = self.socket.accept()
@@ -296,9 +339,21 @@ class AsyncSocket(object):
 
         self.name = "Incoming %s:%s" % (host, port)
         self.connectionErrback = errback
-        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.socket.bind((host, port))
-        (self.addr, self.port) = self.socket.getsockname()
+
+        try:
+            self.socket = socket.socket(family, socket.SOCK_STREAM)
+            self.socket.bind((host, port))
+        except socket.error, e:
+            trap_call(self, errback, ConnectionError(e[1]))
+            return
+
+        if self.socket.family == socket.AF_INET:
+            (self.addr, self.port) = self.socket.getsockname()
+        elif self.socket.family == socket.AF_INET6:
+            (self.addr, self.port, self.flowinfo,
+             self.scopeid) = self.socket.getsockname()
+        else:
+            raise ValueError("Unknown socket family: %s", self.socket.family)
         self.socket.listen(63)
         eventloop.add_read_callback(self.socket, finishAccept)
 
@@ -308,10 +363,10 @@ class AsyncSocket(object):
             self.stopReadTimeout()
             self.socket.close()
             self.socket = None
-            if self.connectionErrback is not None:
-                error = NetworkError(_("Connection closed"))
-                trap_call(self, self.connectionErrback, error)
-                self.connectionErrback = None
+        if self.connectionErrback is not None:
+            error = NetworkError(_("Connection closed"))
+            trap_call(self, self.connectionErrback, error)
+            self.connectionErrback = None
 
     def isOpen(self):
         return self.socket is not None
@@ -357,7 +412,8 @@ class AsyncSocket(object):
             timeout = SOCKET_INITIAL_READ_TIMEOUT
 
         if clock() < self.lastClock + timeout:
-            self.readTimeout = eventloop.add_timeout(self.lastClock + timeout - clock(), self.onReadTimeout,
+            self.readTimeout = eventloop.add_timeout(
+                self.lastClock + timeout - clock(), self.onReadTimeout,
                 "AsyncSocket.onReadTimeout")
         else:
             self.readTimeout = None
@@ -373,7 +429,8 @@ class AsyncSocket(object):
         else:
             expectedErrors = (errno.ECONNREFUSED, errno.ECONNRESET)
         if code not in expectedErrors:
-            logging.warning("WARNING, got unexpected error during %s", operation)
+            logging.warning("WARNING, got unexpected error during %s",
+                            operation)
             logging.warning("%s: %s", errno.errorcode.get(code), msg)
         self.handleEarlyClose(operation)
 
@@ -413,7 +470,8 @@ class AsyncSocket(object):
                 logging.error("ERROR: Too many MemoryErrors on %s", self)
                 self.handleEarlyClose('read')
             else:
-                logging.warning("WARNING: Memory error while reading from %s", self)
+                logging.warning(
+                    "WARNING: Memory error while reading from %s", self)
         else:
             self.memoryErrors = 0
             self.handleReadData(data)
@@ -431,10 +489,10 @@ class AsyncSocket(object):
         self.close_connection()
         if self.closeCallback:
             if operation == 'read':
-                type = socket.SHUT_RD
+                typ = socket.SHUT_RD
             else:
-                type = socket.SHUT_WR
-            trap_call(self, self.closeCallback, self, type)
+                typ = socket.SHUT_WR
+            trap_call(self, self.closeCallback, self, typ)
 
 class AsyncSSLStream(AsyncSocket):
     def __init__(self, closeCallback=None):
@@ -590,10 +648,10 @@ class ConnectionHandler(object):
             lastState = self.state
             self.readHandler()
 
-    def closeCallback(self, stream, type):
-        self.handle_close(type)
+    def closeCallback(self, stream, typ):
+        self.handle_close(typ)
 
-    def handle_close(self, type):
+    def handle_close(self, typ):
         """Handle our stream becoming closed.  Type is either socket.SHUT_RD,
         or socket.SHUT_WR.
         """
