@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,8 +50,7 @@ Go to the bottom of this file for the current database schema.
 import datetime
 import time
 from types import NoneType
-from miro.plat.utils import FilenameType
-from miro.frontendstate import WidgetsFrontendState
+from miro.plat.utils import PlatformFilenameType
 
 class ValidationError(Exception):
     """Error thrown when we try to save invalid data."""
@@ -97,14 +97,14 @@ class SchemaItem(object):
             raise ValidationError("%r (type: %s) is not a %s" %
                                   (data, type(data), correctType))
 
-    def validateTypes(self, data, possibleTypes):
+    def validateTypes(self, data, possibleTypes, msg=""):
         if data is None:
             return
         for t in possibleTypes:
             if isinstance(data, t):
                 return
-        raise ValidationError("%r (type: %s) is not any of: %s" %
-                              (data, type(data), possibleTypes))
+        raise ValidationError("%r (type: %s) is not any of: %s (%s)" %
+                              (data, type(data), possibleTypes, msg))
 
 class SchemaSimpleItem(SchemaItem):
     """Base class for SchemaItems for simple python types."""
@@ -137,7 +137,7 @@ class SchemaFilename(SchemaSimpleItem):
     """Defines the SchemaFilename type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
-        self.validateType(data, FilenameType)
+        self.validateType(data, PlatformFilenameType)
 
 class SchemaURL(SchemaSimpleItem):
     """Defines the SchemaURL type."""
@@ -168,6 +168,12 @@ class SchemaTimeDelta(SchemaSimpleItem):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, datetime.timedelta)
 
+class SchemaMultiValue(SchemaSimpleItem):
+    """Stores integer, boolean, string, or unicode data"""
+    def validate(self, data):
+        super(SchemaSimpleItem, self).validate(data)
+        self.validateTypes(data, [int, long, bool, str, unicode])
+
 class SchemaReprContainer(SchemaItem):
     """SchemaItem saved using repr() to save nested lists, dicts and
     tuples that store simple types.  The look is similar to JSON, but
@@ -207,6 +213,29 @@ class SchemaReprContainer(SchemaItem):
             else:
                 self.validateTypes(obj, self.VALID_TYPES)
 
+class SchemaTuple(SchemaReprContainer):
+    """Special case of SchemaReprContainer that stores a simple tuple
+    """
+    def __init__(self, *childSchemas, **kargs):
+        if 'noneOk' in kargs:
+            noneOk = kargs['noneOk']
+        else:
+            noneOk = False
+        super(SchemaTuple, self).__init__(noneOk)
+        self.childSchemas = childSchemas
+
+    def validate(self, data):
+        if data is None:
+            super(SchemaTuple, self).validate(data)
+            return
+        self.validateType(data, tuple)
+        for i, value in enumerate(data):
+            try:
+                self.childSchemas[i].validate(value)
+            except ValidationError:
+                raise ValidationError("%r (index: %s) has the wrong type" %
+                                      (value, i))
+
 class SchemaList(SchemaReprContainer):
     """Special case of SchemaReprContainer that stores a simple list
 
@@ -248,12 +277,12 @@ class SchemaDict(SchemaReprContainer):
             try:
                 self.keySchema.validate(key)
             except ValidationError:
-                raise ValidationError("key %r has the wrong type" % key)
+                raise ValidationError("key %r has the wrong type: %s" % (key, type(key)))
             try:
                 self.valueSchema.validate(value)
             except ValidationError:
-                raise ValidationError("value %r (key: %r) has the wrong type"
-                        % (value, key))
+                raise ValidationError("value %r (key: %r) has the wrong type: %s"
+                        % (value, key, type(value)))
 
 class SchemaStatusContainer(SchemaReprContainer):
     """Version of SchemaReprContainer that stores the status dict for
@@ -269,17 +298,17 @@ class SchemaStatusContainer(SchemaReprContainer):
         for key, value in data.items():
             self.validateTypes(key, [bool, int, long, float, unicode,
                                      str, NoneType, datetime.datetime,
-                                     time.struct_time])
+                                     time.struct_time], "%s: %s" % (key, value))
             if key not in binary_fields:
                 self.validateTypes(value, [bool, int, long, float, unicode,
                                            NoneType, datetime.datetime,
-                                           time.struct_time])
+                                           time.struct_time], "%s: %s" % (key, value))
             else:
                 self.validateType(value, str)
 
     def _binary_fields(self):
-        rv = ('metainfo', 'fastResumeData')
-        if FilenameType != unicode:
+        rv = ('metainfo',)
+        if PlatformFilenameType != unicode:
             rv += self.filename_fields
         return rv
 
@@ -333,14 +362,16 @@ from miro.feed import (Feed, FeedImpl, RSSFeedImpl, SavedSearchFeedImpl,
                        ScraperFeedImpl)
 from miro.feed import (SearchFeedImpl, DirectoryWatchFeedImpl,
                        DirectoryFeedImpl, SearchDownloadsFeedImpl)
-from miro.feed import ManualFeedImpl, SingleFeedImpl
-from miro.folder import ChannelFolder, PlaylistFolder, PlaylistFolderItemMap
+from miro.feed import ManualFeedImpl
+from miro.folder import (HideableTab, ChannelFolder, PlaylistFolder,
+                         PlaylistFolderItemMap)
 from miro.guide import ChannelGuide
 from miro.item import Item, FileItem
 from miro.iconcache import IconCache
 from miro.playlist import SavedPlaylist, PlaylistItemMap
 from miro.tabs import TabOrder
 from miro.theme import ThemeHistory
+from miro.widgetstate import DisplayState, ViewState, GlobalState
 
 class DDBObjectSchema(ObjectSchema):
     klass = DDBObject
@@ -382,8 +413,6 @@ class ItemSchema(MultiClassObjectSchema):
         ('autoDownloaded', SchemaBool()),
         ('pendingManualDL', SchemaBool()),
         ('pendingReason', SchemaString()),
-        ('title', SchemaString(noneOk=True)),
-        ('description', SchemaString(noneOk=True)),
         ('expired', SchemaBool()),
         ('keep', SchemaBool()),
         ('creationTime', SchemaDateTime()),
@@ -391,13 +420,13 @@ class ItemSchema(MultiClassObjectSchema):
         ('icon_cache_id', SchemaInt(noneOk=True)),
         ('downloadedTime', SchemaDateTime(noneOk=True)),
         ('watchedTime', SchemaDateTime(noneOk=True)),
+        ('lastWatched', SchemaDateTime(noneOk=True)),
         ('subtitle_encoding', SchemaString(noneOk=True)),
         ('isContainerItem', SchemaBool(noneOk=True)),
         ('releaseDateObj', SchemaDateTime()),
         ('eligibleForAutoDownload', SchemaBool()),
         ('duration', SchemaInt(noneOk=True)),
         ('screenshot', SchemaFilename(noneOk=True)),
-        ('media_type_checked', SchemaBool()),
         ('resumeTime', SchemaInt()),
         ('channelTitle', SchemaString(noneOk=True)),
         ('license', SchemaString(noneOk=True)),
@@ -417,7 +446,30 @@ class ItemSchema(MultiClassObjectSchema):
         ('deleted', SchemaBool(noneOk=True)),
         ('shortFilename', SchemaFilename(noneOk=True)),
         ('offsetPath', SchemaFilename(noneOk=True)),
+        ('play_count', SchemaInt()),
+        ('skip_count', SchemaInt()),
+        ('cover_art', SchemaFilename(noneOk=True)),
+        ('mdp_state', SchemaInt(noneOk=True)),
+        # metadata:
+        ('metadata_version', SchemaInt()),
+        ('title', SchemaString(noneOk=True)),
+        ('title_tag', SchemaString(noneOk=True)),
+        ('description', SchemaString(noneOk=True)),
+        ('album', SchemaString(noneOk=True)),
+        ('album_artist', SchemaString(noneOk=True)),
+        ('artist', SchemaString(noneOk=True)),
+        ('track', SchemaInt(noneOk=True)),
+        ('album_tracks', SchemaInt(noneOk=True)),
+        ('year', SchemaInt(noneOk=True)),
+        ('genre', SchemaString(noneOk=True)),
+        ('rating', SchemaInt(noneOk=True)),
         ('file_type', SchemaString(noneOk=True)),
+        ('has_drm', SchemaBool(noneOk=True)),
+        ('show', SchemaString(noneOk=True)),
+        ('episode_id', SchemaString(noneOk=True)),
+        ('episode_number', SchemaInt(noneOk=True)),
+        ('season_number', SchemaInt(noneOk=True)),
+        ('kind', SchemaString(noneOk=True)),
     ]
 
     indexes = (
@@ -449,7 +501,7 @@ class FeedSchema(DDBObjectSchema):
         ('fallBehind', SchemaInt()),
         ('expire', SchemaString()),
         ('expireTime', SchemaTimeDelta(noneOk=True)),
-        ('section', SchemaString()),
+        ('section', SchemaString()), # not used anymore
         ('visible', SchemaBool()),
         ('last_viewed', SchemaDateTime()),
     ]
@@ -537,11 +589,6 @@ class ManualFeedImplSchema(FeedImplSchema):
     table_name = 'manual_feed_impl'
     # no addition fields over FeedImplSchema
 
-class SingleFeedImplSchema(FeedImplSchema):
-    klass = SingleFeedImpl
-    table_name = 'single_feed_impl'
-    # no addition fields over FeedImplSchema
-
 class RemoteDownloaderSchema(DDBObjectSchema):
     klass = RemoteDownloader
     table_name = 'remote_downloader'
@@ -553,7 +600,6 @@ class RemoteDownloaderSchema(DDBObjectSchema):
         ('channelName', SchemaFilename(noneOk=True)),
         ('status', SchemaStatusContainer()),
         ('metainfo', SchemaBinary(noneOk=True)),
-        ('fast_resume_data', SchemaBinary(noneOk=True)),
         ('manualUpload', SchemaBool()),
         ('state', SchemaString()),
         ('main_item_id', SchemaInt(noneOk=True)),
@@ -568,13 +614,25 @@ class RemoteDownloaderSchema(DDBObjectSchema):
     def handle_malformed_status(row):
         return {}
 
+class HideableTabSchema(DDBObjectSchema):
+    klass = HideableTab
+    table_name = 'hideable_tab'
+    fields = DDBObjectSchema.fields + [
+        ('type', SchemaString()),
+        ('expanded', SchemaBool()),
+    ]
+
+    indexes = (
+        ('hideable_tab_type', ('type',)),
+    )
+
 class ChannelFolderSchema(DDBObjectSchema):
     klass = ChannelFolder
     table_name = 'channel_folder'
     fields = DDBObjectSchema.fields + [
         ('expanded', SchemaBool()),
         ('title', SchemaString()),
-        ('section', SchemaString()),
+        ('section', SchemaString()), # not used any more
     ]
 
 class PlaylistFolderSchema(DDBObjectSchema):
@@ -636,6 +694,7 @@ class ChannelGuideSchema(DDBObjectSchema):
         ('userTitle', SchemaString(noneOk=True)),
         ('icon_cache_id', SchemaInt(noneOk=True)),
         ('firstTime', SchemaBool()),
+        ('store', SchemaInt()),
     ]
 
     @staticmethod
@@ -654,20 +713,42 @@ class ThemeHistorySchema(DDBObjectSchema):
     def handle_malformed_pastThemes(row):
         return []
 
-class WidgetsFrontendStateSchema(DDBObjectSchema):
-    klass = WidgetsFrontendState
-    table_name = 'widgets_frontend_state'
+class DisplayStateSchema(DDBObjectSchema):
+    klass = DisplayState
+    table_name = 'display_state'
     fields = DDBObjectSchema.fields + [
-        ('list_view_displays', SchemaList(SchemaBinary())),
-        ('active_filters', SchemaDict(SchemaBinary(),
-            SchemaList(SchemaBinary()))),
-        ('sort_states', SchemaDict(SchemaBinary(),
-            SchemaBinary())),
+        ('type', SchemaString()),
+        ('id_', SchemaString()),
+        ('selected_view', SchemaInt(noneOk=True)),
+        ('active_filters', SchemaInt(noneOk=True)),
+        ('list_view_columns', SchemaList(SchemaString(), noneOk=True)),
+        ('list_view_widths', SchemaDict(SchemaString(), SchemaInt(), noneOk=True)),
+        ('shuffle', SchemaBool(noneOk=True)),
+        ('repeat', SchemaInt(noneOk=True)),
+        ('selection', SchemaList(SchemaMultiValue(), noneOk=True)),
+        ('sort_state', SchemaString(noneOk=True)),
+        ('last_played_item_id', SchemaInt(noneOk=True)),
     ]
 
+    indexes = (
+        ('display_state_display', ('type', 'id_')),
+    )
+
     @staticmethod
-    def handle_malformed_list_view_displays(row):
-        return []
+    def handle_malformed_list_view_columns(value):
+        return None
+
+    @staticmethod
+    def handle_malformed_list_view_widths(value):
+        return None
+
+class GlobalStateSchema(DDBObjectSchema):
+    klass = GlobalState
+    table_name = 'global_state'
+    fields = DDBObjectSchema.fields + [
+        ('item_details_expanded', SchemaDict(SchemaInt(), SchemaBool())),
+        ('guide_sidebar_expanded', SchemaBool())
+    ]
 
 class DBLogEntrySchema(DDBObjectSchema):
     klass = DBLogEntry
@@ -678,11 +759,29 @@ class DBLogEntrySchema(DDBObjectSchema):
         ('description', SchemaString()),
     ]
 
-    @staticmethod
-    def handle_malformed_list_view_displays(row):
-        return []
+class ViewStateSchema(DDBObjectSchema):
+    klass = ViewState
+    table_name = 'view_state'
+    fields = DDBObjectSchema.fields + [
+        ('display_type', SchemaString()),
+        ('display_id', SchemaString()),
+        ('view_type', SchemaInt()),
+        ('scroll_position', SchemaTuple(SchemaInt(), SchemaInt(), noneOk=True)),
+    ]
 
-VERSION = 119
+    indexes = (
+        ('view_state_key', ('display_type', 'display_id', 'view_type')),
+    )
+
+    @staticmethod
+    def handle_malformed_scroll_position(value):
+        return None
+
+    @staticmethod
+    def handle_malformed_selection(value):
+        return None
+
+VERSION = 159
 object_schemas = [
     IconCacheSchema, ItemSchema, FeedSchema,
     FeedImplSchema, RSSFeedImplSchema, SavedSearchFeedImplSchema,
@@ -690,9 +789,8 @@ object_schemas = [
     SearchFeedImplSchema, DirectoryFeedImplSchema, DirectoryWatchFeedImplSchema,
     SearchDownloadsFeedImplSchema, RemoteDownloaderSchema,
     ChannelGuideSchema, ManualFeedImplSchema,
-    SingleFeedImplSchema,
-    PlaylistSchema, ChannelFolderSchema, PlaylistFolderSchema,
+    PlaylistSchema, HideableTabSchema, ChannelFolderSchema, PlaylistFolderSchema,
     PlaylistItemMapSchema, PlaylistFolderItemMapSchema,
-    TabOrderSchema, ThemeHistorySchema, WidgetsFrontendStateSchema,
-    DBLogEntrySchema,
+    TabOrderSchema, ThemeHistorySchema, DisplayStateSchema, GlobalStateSchema,
+    DBLogEntrySchema, ViewStateSchema,
 ]
