@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,21 +39,21 @@ FIXME
 from miro.gtcache import gettext as _
 import logging
 import os
+import config
 import traceback
 import threading
 import time
 
-from miro.clock import clock
 from miro import app
 from miro import autodler
 from miro import autoupdate
-from miro import config
 from miro import commandline
-from miro import crashreport
 from miro import controller
+from miro import extensionmanager
 from miro import database
 from miro import databaselog
 from miro import databaseupgrade
+from miro import dbupgradeprogress
 from miro import dialogs
 from miro import downloader
 from miro import eventloop
@@ -62,14 +63,18 @@ from miro import httpauth
 from miro import httpclient
 from miro import iconcache
 from miro import item
+from miro import itemsource
+from miro import iteminfocache
 from miro import feed
 from miro import folder
 from miro import messages
 from miro import messagehandler
+from miro import metadataprogress
 from miro import models
 from miro import moviedata
 from miro import playlist
 from miro import prefs
+import miro.plat.resources
 from miro.plat.utils import setup_logging
 from miro.plat import config as platformcfg
 from miro import tabs
@@ -77,7 +82,11 @@ from miro import theme
 from miro import util
 from miro import searchengines
 from miro import storedatabase
-from miro import videoconversion
+from miro import conversions
+from miro import devices
+from miro import sharing
+from miro import transcode
+from miro.plat import devicetracker
 
 DEBUG_DB_MEM_USAGE = False
 mem_usage_test_event = threading.Event()
@@ -129,13 +138,13 @@ def startup_function(func):
                       "It's possible that your database file is corrupted and "
                       "cannot be used.\n\n"
                       "You can start fresh and your damaged database will be "
-                      "removed, but you will have to re-add your feeds and "
+                      "removed, but you will have to re-add your podcasts and "
                       "media files.  If you want to do this, press the "
                       "Start Fresh button and restart %(appname)s.\n\n"
                       "To help us fix problems like this in the future, "
                       "please file a bug report at %(url)s.",
-                      {"appname": config.get(prefs.SHORT_APP_NAME),
-                       "url": config.get(prefs.BUG_REPORT_URL),
+                      {"appname": app.config.get(prefs.SHORT_APP_NAME),
+                       "url": app.config.get(prefs.BUG_REPORT_URL),
                        "error": msg}
                       ))
                 m.send_to_frontend()
@@ -147,8 +156,8 @@ def startup_function(func):
                     _("Unknown Error"),
                     _("An unknown error prevented %(appname)s from startup.  "
                       "Please file a bug report at %(url)s.",
-                      {"appname": config.get(prefs.SHORT_APP_NAME),
-                       "url": config.get(prefs.BUG_REPORT_URL)}
+                      {"appname": app.config.get(prefs.SHORT_APP_NAME),
+                       "url": app.config.get(prefs.BUG_REPORT_URL)}
                       ))
                 m.send_to_frontend()
     return wrapped
@@ -193,12 +202,21 @@ def initialize(themeName):
     """Initialize Miro.  This sets up things like logging and the config
     system and should be called as early as possible.
     """
+    # set debugmode if it hasn't already been set
+    if app.debugmode == None:
+        if app.config.get(prefs.APP_FINAL_RELEASE) == u"0":
+            # if it's not a final release, then we're in debugmode
+            app.debugmode = True
+        else:
+            # if it is a final release, then we're not in debugmode
+            app.debugmode = False
+
     # this is platform specific
     setup_logging()
     # this is portable general
     util.setup_logging()
     app.controller = controller.Controller()
-    config.load(themeName)
+    config.set_theme(themeName)
 
 def startup():
     """Startup Miro.
@@ -215,11 +233,12 @@ def startup():
 
     initialize() must be called before startup().
     """
-    logging.info("Starting up %s", config.get(prefs.LONG_APP_NAME))
-    logging.info("Version:    %s", config.get(prefs.APP_VERSION))
-    logging.info("Revision:   %s", config.get(prefs.APP_REVISION))
-    logging.info("Builder:    %s", config.get(prefs.BUILD_MACHINE))
-    logging.info("Build Time: %s", config.get(prefs.BUILD_TIME))
+    logging.info("Starting up %s", app.config.get(prefs.LONG_APP_NAME))
+    logging.info("Version:    %s", app.config.get(prefs.APP_VERSION))
+    logging.info("Revision:   %s", app.config.get(prefs.APP_REVISION))
+    logging.info("Builder:    %s", app.config.get(prefs.BUILD_MACHINE))
+    logging.info("Build Time: %s", app.config.get(prefs.BUILD_TIME))
+    logging.info("Debugmode:  %s", app.debugmode)
     eventloop.connect('thread-started', finish_startup)
     logging.info("Reading HTTP Password list")
     httpauth.init()
@@ -231,6 +250,15 @@ def startup():
     eventloop.startup()
     if DEBUG_DB_MEM_USAGE:
         mem_usage_test_event.wait()
+    load_extensions()
+
+@startup_function
+def load_extensions():
+    core_ext_dirs = miro.plat.resources.extension_core_roots()
+    user_ext_dirs = miro.plat.resources.extension_user_roots()
+    app.extension_manager = extensionmanager.ExtensionManager(
+        core_ext_dirs, user_ext_dirs)
+    app.extension_manager.load_extensions()
 
 @startup_function
 def finish_startup(obj, thread):
@@ -246,7 +274,7 @@ def finish_startup(obj, thread):
             "You have a database that was saved with a newer version of "
             "%(appname)s. You must download the latest version of "
             "%(appname)s and run that.",
-            {"appname": config.get(prefs.SHORT_APP_NAME)},
+            {"appname": app.config.get(prefs.SHORT_APP_NAME)},
         )
         raise StartupError(summary, description)
     except storedatabase.UpgradeErrorSendCrashReport, e:
@@ -266,8 +294,18 @@ def finish_startup(obj, thread):
         util.db_mem_usage_test()
         mem_usage_test_event.set()
 
+    # MetadataProgressUpdater needs to be installed before ItemInfoCache,
+    # since ItemInfoCache may create items if it uses failsafe mode
+    app.metadata_progress_updater = metadataprogress.MetadataProgressUpdater()
+    app.item_info_cache = iteminfocache.ItemInfoCache()
+    app.item_info_cache.load()
+    dbupgradeprogress.upgrade_end()
+
     logging.info("Loading video converters...")
-    videoconversion.conversion_manager.startup()
+    conversions.conversion_manager.startup()
+    app.device_manager = devices.DeviceManager()
+    app.device_tracker = devicetracker.DeviceTracker()
+
     searchengines.create_engines()
     setup_global_feeds()
     # call fix_database_inconsistencies() ASAP after the manual feed is set up
@@ -277,8 +315,14 @@ def finish_startup(obj, thread):
     logging.info("setup theme...")
     setup_theme()
     install_message_handler()
+    itemsource.setup_handlers()
     downloader.init_controller()
 
+    # Call this late, after the message handlers have been installed.
+    app.sharing_tracker = sharing.SharingTracker()
+    app.sharing_manager = sharing.SharingManager()
+    app.transcode_manager = transcode.TranscodeManager()
+
     eventloop.add_urgent_call(check_firsttime, "check first time")
 
 def fix_database_inconsistencies():
@@ -291,9 +335,10 @@ def fix_database_inconsistencies():
 def check_firsttime():
     """Run the first time wizard if need be.
     """
-    callback = lambda: eventloop.add_urgent_call(check_movies_gone, "check movies gone")
+    callback = lambda: eventloop.add_urgent_call(check_movies_gone,
+                                                 "check movies gone")
     if is_first_time():
-        logging.info("First time -- calling handler.")
+        logging.info("First time run -- calling handler.")
         _first_time_handler(callback)
         return
 
@@ -311,13 +356,15 @@ def check_movies_gone():
         _movies_directory_gone_handler(callback)
         return
 
-    movies_dir = fileutil.expand_filename(config.get(prefs.MOVIES_DIRECTORY))
+    movies_dir = fileutil.expand_filename(app.config.get(
+        prefs.MOVIES_DIRECTORY))
     # if the directory doesn't exist, create it.
     if not os.path.exists(movies_dir):
         try:
-            os.makedirs(movies_dir)
+            fileutil.makedirs(movies_dir)
         except OSError:
-            logging.info("Movies directory can't be created -- calling handler")
+            logging.info(
+                "Movies directory can't be created -- calling handler")
             # FIXME - this isn't technically correct, but it's probably
             # close enough that a user can fix the issue and Miro can
             # run happily.
@@ -325,20 +372,15 @@ def check_movies_gone():
             return
 
     # make sure the directory is writeable
-    try:
-        fn = os.path.join(movies_dir, "testfile")
-        f = open(fn, "w")
-        f.write("test")
-        f.close()
-        os.remove(fn)
-    except IOError:
+    if not os.access(movies_dir, os.W_OK):
         _movies_directory_gone_handler(callback)
         return
     eventloop.add_urgent_call(finish_backend_startup, "reconnect downloaders")
 
 @startup_function
 def fix_movies_gone():
-    config.set(prefs.MOVIES_DIRECTORY, platformcfg.get(prefs.MOVIES_DIRECTORY))
+    app.config.set(prefs.MOVIES_DIRECTORY, platformcfg.get(
+        prefs.MOVIES_DIRECTORY))
     eventloop.add_urgent_call(finish_backend_startup, "reconnect downloaders")
 
 @startup_function
@@ -361,28 +403,22 @@ def on_frontend_started():
     yield None
     feed.expire_items()
     yield None
-    logging.info("Starting movie data updates")
-    item.update_incomplete_movie_data()
-    yield None
     moviedata.movie_data_updater.start_thread()
     yield None
     commandline.startup()
     yield None
     autoupdate.check_for_updates()
     yield None
-    # Wait a bit before starting the downloader daemon.  It can cause a bunch
-    # of disk/CPU load, so try to avoid it slowing other stuff down.
+    # Delay running high CPU/IO operations for a bit
     eventloop.add_timeout(5, downloader.startup_downloader,
             "start downloader daemon")
-    # ditto for feed updates
     eventloop.add_timeout(30, feed.start_updates, "start feed updates")
-    # ditto for clearing stale icon cache files, except it's the very lowest
-    # priority
-    eventloop.add_timeout(10, clear_icon_cache_orphans, "clear orphans")
+    eventloop.add_timeout(60, item.update_incomplete_movie_data,
+            "update movie data")
+    eventloop.add_timeout(90, clear_icon_cache_orphans, "clear orphans")
 
 def setup_global_feeds():
     setup_global_feed(u'dtv:manualFeed', initiallyAutoDownloadable=False)
-    setup_global_feed(u'dtv:singleFeed', initiallyAutoDownloadable=False)
     setup_global_feed(u'dtv:search', initiallyAutoDownloadable=False)
     setup_global_feed(u'dtv:searchDownloads')
     setup_global_feed(u'dtv:directoryfeed')
@@ -391,13 +427,12 @@ def setup_tabs():
     def setup_tab_order(type):
         current_tab_orders = list(tabs.TabOrder.view_for_type(type))
         if len(current_tab_orders) == 0:
-            logging.info("Creating %s tab order" % type)
-            tab_order = tabs.TabOrder(type)
+            logging.info("Creating %s tab order", type)
+            tabs.TabOrder(type)
         else:
             current_tab_orders[0].restore_tab_list()
     setup_tab_order(u'site')
     setup_tab_order(u'channel')
-    setup_tab_order(u'audio-channel')
     setup_tab_order(u'playlist')
 
 def is_first_time():
@@ -407,17 +442,23 @@ def is_first_time():
 
     Returns True if yes, False if no.
     """
-    return not config.get(prefs.STARTUP_TASKS_DONE)
+    return not app.config.get(prefs.STARTUP_TASKS_DONE)
 
 def mark_first_time():
-    config.set(prefs.STARTUP_TASKS_DONE, True)
+    app.config.set(prefs.STARTUP_TASKS_DONE, True)
+    # we just bothered the user about importing media, don't do it again
+    app.config.set(prefs.MUSIC_TAB_CLICKED, True)
+    # make sure we save the config now, it's really annoying if a first-time
+    # startup dialog pops up again
+    app.config.save()
 
 def is_movies_directory_gone():
     """Checks to see if the MOVIES_DIRECTORY exists.
 
     Returns True if yes, False if no.
     """
-    movies_dir = fileutil.expand_filename(config.get(prefs.MOVIES_DIRECTORY))
+    movies_dir = fileutil.expand_filename(app.config.get(
+        prefs.MOVIES_DIRECTORY))
     if not movies_dir.endswith(os.path.sep):
         movies_dir += os.path.sep
     logging.info("Checking movies directory %r...", movies_dir)
@@ -479,7 +520,8 @@ def clear_icon_cache_orphans():
     # delete files in the icon cache directory that don't belong to IconCache
     # objects.
 
-    cachedir = fileutil.expand_filename(config.get(prefs.ICON_CACHE_DIRECTORY))
+    cachedir = fileutil.expand_filename(app.config.get(
+        prefs.ICON_CACHE_DIRECTORY))
     if not os.path.isdir(cachedir):
         return
 
@@ -512,9 +554,9 @@ def send_startup_crash_report(report):
         "%(appname)s will now submit a crash report to our crash "
         "database\n\n"
         "Do you want to include entire program database "
-        "including all video and feed metadata with crash report? "
+        "including all video and podcast metadata with crash report? "
         "This will help us diagnose the issue.",
-        {"appname": config.get(prefs.SHORT_APP_NAME)})
+        {"appname": app.config.get(prefs.SHORT_APP_NAME)})
     d = dialogs.ChoiceDialog(title, description,
             dialogs.BUTTON_INCLUDE_DATABASE,
             dialogs.BUTTON_DONT_INCLUDE_DATABASE)
