@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -63,7 +64,6 @@ except ImportError:
     from pysqlite2 import dbapi2 as sqlite3
 
 from miro import app
-from miro import config
 from miro import crashreport
 from miro import convert20database
 from miro import databaseupgrade
@@ -71,13 +71,13 @@ from miro import dbupgradeprogress
 from miro import dialogs
 from miro import eventloop
 from miro import fileutil
+from miro import iteminfocache
 from miro import messages
 from miro import schema
 from miro import prefs
 from miro import util
-from miro.download_utils import next_free_filename
 from miro.gtcache import gettext as _
-from miro.plat.utils import FilenameType, filename_to_unicode
+from miro.plat.utils import PlatformFilenameType, filename_to_unicode
 
 class UpgradeError(Exception):
     """While upgrading the database, we ran out of disk space."""
@@ -99,6 +99,7 @@ _sqlite_type_map = {
         schema.SchemaDateTime: 'timestamp',
         schema.SchemaTimeDelta: 'pythonrepr',
         schema.SchemaReprContainer: 'pythonrepr',
+        schema.SchemaTuple: 'pythonrepr',
         schema.SchemaDict: 'pythonrepr',
         schema.SchemaList: 'pythonrepr',
         schema.SchemaStatusContainer: 'pythonrepr',
@@ -129,7 +130,7 @@ class LiveStorage:
     """
     def __init__(self, path=None, object_schemas=None, schema_version=None):
         if path is None:
-            path = config.get(prefs.SQLITE_PATHNAME)
+            path = app.config.get(prefs.SQLITE_PATHNAME)
         if object_schemas is None:
             object_schemas = schema.object_schemas
         if schema_version is None:
@@ -152,7 +153,6 @@ class LiveStorage:
         self._dc = None
         self._query_times = {}
         self.path = path
-        self.open_connection()
         self._quitting_from_operational_error = False
         self._object_schemas = object_schemas
         self._schema_version = schema_version
@@ -173,6 +173,8 @@ class LiveStorage:
                 self._schema_column_map[oschema, name] = schema_item
         self._converter = SQLiteConverter()
 
+        self.open_connection()
+
         if not db_existed:
             self._init_database()
 
@@ -184,6 +186,14 @@ class LiveStorage:
                 isolation_level=None,
                 detect_types=sqlite3.PARSE_DECLTYPES)
         self.cursor = self.connection.cursor()
+        try:
+            self.cursor.execute("PRAGMA journal_mode=PERSIST");
+        except sqlite3.DatabaseError:
+            msg = "Error running 'PRAGMA journal_mode=PERSIST'"
+            self._show_corrupt_db_dialog()
+            self._handle_load_error(msg)
+            # rerun the command with our fresh database
+            self.cursor.execute("PRAGMA journal_mode=PERSIST");
 
     def close(self):
         logging.info("closing database")
@@ -247,7 +257,7 @@ class LiveStorage:
     def _handle_upgrade_error(self):
         self._backup_failed_upgrade_db()
         title = _("%(appname)s database upgrade failed",
-                  {"appname": config.get(prefs.SHORT_APP_NAME)})
+                  {"appname": app.config.get(prefs.SHORT_APP_NAME)})
         description = _(
             "We're sorry, %(appname)s was unable to upgrade your database "
             "due to errors.\n\n"
@@ -257,8 +267,8 @@ class LiveStorage:
             "If your disk is not full, help us understand the problem by "
             "reporting a bug to our crash database.\n\n"
             "Finally, you can start fresh and your damaged database will be "
-            "removed, but you will have to re-add your feeds and media "
-            "files.", {"appname": config.get(prefs.SHORT_APP_NAME)}
+            "removed, but you will have to re-add your podcasts and media "
+            "files.", {"appname": app.config.get(prefs.SHORT_APP_NAME)}
             )
         d = dialogs.ThreeChoiceDialog(title, description,
                 dialogs.BUTTON_QUIT, dialogs.BUTTON_SUBMIT_REPORT,
@@ -286,7 +296,7 @@ class LiveStorage:
 
         :param ver: the current version (as string)
         """
-        logging.info("path of database: %s", self.path)
+        logging.info("database path: %s", self.path)
         # close database
         self.close()
 
@@ -328,20 +338,17 @@ class LiveStorage:
             raise databaseupgrade.DatabaseTooNewError(msg)
 
         if current_version < self._schema_version:
-            dbupgradeprogress.upgrade_start()
-            try:
-                self._upgrade_20_database()
-                # need to pull the variable again here because
-                # _upgrade_20_database will have done an upgrade
-                current_version = self._get_version()
-                self._change_database_file(current_version)
-                databaseupgrade.new_style_upgrade(self.cursor,
-                                                  current_version,
-                                                  self._schema_version)
-                self._set_version()
-                self._change_database_file_back()
-            finally:
-                dbupgradeprogress.upgrade_end()
+            self._upgrade_20_database()
+            # need to pull the variable again here because
+            # _upgrade_20_database will have done an upgrade
+            dbupgradeprogress.doing_new_style_upgrade()
+            current_version = self._get_version()
+            self._change_database_file(current_version)
+            databaseupgrade.new_style_upgrade(self.cursor,
+                                              current_version,
+                                              self._schema_version)
+            self._set_version()
+            self._change_database_file_back()
         self.current_version = self._schema_version
 
     def _upgrade_20_database(self):
@@ -368,13 +375,15 @@ class LiveStorage:
                 self._set_version(80)
                 self._change_database_file_back()
 
-    def _get_variable(self, name):
+    def get_variable(self, name):
         self.cursor.execute("SELECT serialized_value FROM dtv_variables "
                 "WHERE name=?", (name,))
         row = self.cursor.fetchone()
+        if row is None:
+            raise KeyError(name)
         return cPickle.loads(str(row[0]))
 
-    def _set_variable(self, name, value):
+    def set_variable(self, name, value):
         # we only store one variable and it's easier to deal with if we store
         # it using ASCII-base protocol.
         db_value = buffer(cPickle.dumps(value, 0))
@@ -391,8 +400,14 @@ class LiveStorage:
         self._ids_loaded.add(obj.id)
 
     def forget_object(self, obj):
-        del self._object_map[obj.id]
-        self._ids_loaded.remove(obj.id)
+        try:
+            del self._object_map[obj.id]
+        except KeyError:
+            details = "key error in forget_object: %s (obj: %s)" % (obj.id,
+                    obj)
+            app.controller.failed_soft('storedatabase.forget_object', details,
+                    with_exception=True)
+        self._ids_loaded.discard(obj.id)
 
     def _insert_sql_for_schema(self, obj_schema):
         return "INSERT INTO %s (%s) VALUES(%s)" % (obj_schema.table_name,
@@ -468,8 +483,13 @@ class LiveStorage:
             self._execute(sql, values, is_update=True)
             if (self.cursor.rowcount != 1 and not
                     self._quitting_from_operational_error):
-                raise AssertionError("update_obj changed %s rows" %
-                        self.cursor.rowcount)
+                if self.cursor.rowcount == 0:
+                    raise KeyError("Updating non-existent row (id: %s)" %
+                            obj.id)
+                else:
+                    raise ValueError("Update changed multiple rows "
+                            "(id: %s, count: %s)" %
+                            (obj.id, self.cursor.rowcount))
 
     def remove_obj(self, obj):
         """Remove a DDBObject from disk."""
@@ -505,10 +525,10 @@ class LiveStorage:
     def get_last_id(self):
         try:
             return self._get_last_id()
-        except (KeyError, SystemError,
-                databaseupgrade.DatabaseTooNewError):
+        except databaseupgrade.DatabaseTooNewError:
             raise
-        except:
+        except StandardError:
+            self._show_corrupt_db_dialog()
             self._handle_load_error("Error calculating last id")
             return self._get_last_id()
 
@@ -519,13 +539,17 @@ class LiveStorage:
             max_id = max(max_id, self.cursor.fetchone()[0])
         return max_id
 
-    def get_obj_by_id(self, id):
+    def get_obj_by_id(self, id_):
         """Get a particular DDBObject.
 
         This will throw a KeyError if id is not in the database, or if the
         object for id has not been loaded yet.
         """
-        return self._object_map[id]
+        return self._object_map[id_]
+
+    def id_alive(self, id_):
+        """Check if an id_ is exists and is loaded in the databes."""
+        return id_ in self._object_map
 
     def table_name(self, klass):
         return self._schema_map[klass].table_name
@@ -549,29 +573,38 @@ class LiveStorage:
 
     def query(self, klass, where, values=None, order_by=None, joins=None,
             limit=None):
-        for id in self.query_ids(klass, where, values, order_by, joins, limit):
-            yield self._object_map[id]
-
-    def query_ids(self, klass, where, values=None, order_by=None, joins=None,
-            limit=None):
         schema = self._schema_map[klass]
-        sql = StringIO()
-        sql.write("SELECT %s.id " % schema.table_name)
-        sql.write(self._get_query_bottom(schema.table_name, where, joins,
-            order_by, limit))
-        self.cursor.execute(sql.getvalue(), values)
-        rv = [row[0] for row in self.cursor.fetchall()]
-        unrestored_ids = set(rv).difference(self._ids_loaded)
-        if unrestored_ids:
-            # restore any objects that we don't already have in memory.
-            # query() calls query_ids() and expects that for all the object
-            # ids returned, it can look them up in self._object_map
-            self._restore_objects(schema, unrestored_ids)
-
+        id_list = list(self.query_ids(schema.table_name, where, values,
+            order_by, joins,
+            limit))
+        if self.ensure_objects_loaded(klass, id_list):
             # sometimes objects will call remove() in setup_restored().
             # We need to filter those out.
-            rv = [id for id in rv if id in self._object_map]
-        return rv
+            id_list = [id_ for id_ in id_list if id_ in self._object_map]
+        for id_ in id_list:
+            yield self._object_map[id_]
+
+    def ensure_objects_loaded(self, klass, id_list):
+        """Ensure that a list of ids are loaded into memory.
+
+        :returns: True iff we needed to load objects
+        """
+        unrestored_ids = set(id_list).difference(self._ids_loaded)
+        if unrestored_ids:
+            # restore any objects that we don't already have in memory.
+            schema = self._schema_map[klass]
+            self._restore_objects(schema, unrestored_ids)
+            return True
+        return False
+
+    def query_ids(self, table_name, where, values=None, order_by=None,
+            joins=None, limit=None):
+        sql = StringIO()
+        sql.write("SELECT %s.id " % table_name)
+        sql.write(self._get_query_bottom(table_name, where, joins,
+            order_by, limit))
+        self.cursor.execute(sql.getvalue(), values)
+        return (row[0] for row in self.cursor.fetchall())
 
     def _restore_objects(self, schema, id_set):
         column_names = ['%s.%s' % (schema.table_name, f[0])
@@ -629,11 +662,11 @@ class LiveStorage:
     def persistent_object_count(self):
         return len(self._object_map)
 
-    def query_count(self, klass, where, values=None, joins=None, limit=None):
-        schema = self._schema_map[klass]
+    def query_count(self, table_name, where, values=None, joins=None,
+            limit=None):
         sql = StringIO()
         sql.write('SELECT COUNT(*) ')
-        sql.write(self._get_query_bottom(schema.table_name, where, joins,
+        sql.write(self._get_query_bottom(table_name, where, joins,
             None, limit))
         return self._execute(sql.getvalue(), values)[0][0]
 
@@ -645,12 +678,13 @@ class LiveStorage:
             sql.write('\nWHERE %s' % where)
         self._execute(sql.getvalue(), values, is_update=True)
 
-    def select(self, klass, columns, where, values, convert=True):
+    def select(self, klass, columns, where, values, joins=None, limit=None,
+            convert=True):
         schema = self._schema_map[klass]
         sql = StringIO()
         sql.write('SELECT %s ' % ', '.join(columns))
-        sql.write(self._get_query_bottom(schema.table_name, where, None, None,
-            None))
+        sql.write(self._get_query_bottom(schema.table_name, where, joins, None,
+            limit))
         results = self._execute(sql.getvalue(), values)
         if not convert:
             return results
@@ -712,7 +746,7 @@ class LiveStorage:
 
         if failed and not self._quitting_from_operational_error:
             title = _("%(appname)s database save succeeded",
-                      {"appname": config.get(prefs.SHORT_APP_NAME)})
+                      {"appname": app.config.get(prefs.SHORT_APP_NAME)})
             description = _("The database has been successfully saved. "
                     "It is now safe to quit without losing any data.")
             dialogs.MessageBoxDialog(title, description).run()
@@ -752,7 +786,7 @@ class LiveStorage:
         for (sql, values, many) in to_run:
             try:
                 self._time_execute(sql, values, many)
-            except sqlite3.OperationalError, e:
+            except sqlite3.OperationalError:
                 self._log_error(sql, values, many)
                 return False
         return True
@@ -775,7 +809,7 @@ class LiveStorage:
 
     def _show_save_error_dialog(self, error_text):
         title = _("%(appname)s database save failed",
-                  {"appname": config.get(prefs.SHORT_APP_NAME)})
+                  {"appname": app.config.get(prefs.SHORT_APP_NAME)})
         description = _(
             "%(appname)s was unable to save its database.\n\n"
             "If your disk is full, we suggest freeing up some space and "
@@ -787,7 +821,7 @@ class LiveStorage:
             "you reduce the number of simultaneous downloads in the Options "
             "dialog in the Download tab.\n\n"
             "Error: %(error_text)s\n\n",
-            {"appname": config.get(prefs.SHORT_APP_NAME),
+            {"appname": app.config.get(prefs.SHORT_APP_NAME),
              "error_text": error_text}
             )
         d = dialogs.ChoiceDialog(title, description,
@@ -833,17 +867,18 @@ class LiveStorage:
                 self.cursor.execute("CREATE INDEX %s ON %s (%s)" %
                         (name, schema.table_name, ', '.join(columns)))
         self._create_variables_table()
+        self.cursor.execute(iteminfocache.create_sql())
         self._set_version()
 
     def _get_version(self):
-        return self._get_variable(VERSION_KEY)
+        return self.get_variable(VERSION_KEY)
 
     def _set_version(self, version=None):
         """Set the database version to the current schema version."""
 
         if version is None:
             version = self._schema_version
-        self._set_variable(VERSION_KEY, version)
+        self.set_variable(VERSION_KEY, version)
 
     def _calc_sqlite_types(self, object_schema):
         """What datatype should we use for the attributes of an object schema?
@@ -851,11 +886,11 @@ class LiveStorage:
 
         types = []
         for name, schema_item in object_schema.fields:
-            type = _sqlite_type_map[schema_item.__class__]
+            typ = _sqlite_type_map[schema_item.__class__]
             if name != 'id':
-                types.append('%s %s' % (name, type))
+                types.append('%s %s' % (name, typ))
             else:
-                types.append('%s %s PRIMARY KEY' % (name, type))
+                types.append('%s %s PRIMARY KEY' % (name, typ))
         return ', '.join(types)
 
     def reset_database(self):
@@ -867,6 +902,14 @@ class LiveStorage:
         self.open_connection()
         self._init_database()
 
+    def _show_corrupt_db_dialog(self):
+        title = _("%(appname)s database corrupt.",
+                  {"appname": app.config.get(prefs.SHORT_APP_NAME)})
+        description = _("Your Miro database is corrupt.  It will be "
+                "backed up in your Miro database directory and a new "
+                "database will be created now.")
+        dialogs.MessageBoxDialog(title, description).run_blocking()
+
     def _handle_load_error(self, message):
         """Handle errors happening when we try to load the database.  Our
         basic strategy is to log the error, save the current database then
@@ -892,39 +935,6 @@ class LiveStorage:
             save_name = "%s.%d" % (org_save_name, i)
         return save_name
 
-    def dumpDatabase(self):
-        output = open(next_free_filename(os.path.join(config.get(prefs.SUPPORT_DIRECTORY), "database-dump.xml")), 'w')
-        def indent(level):
-            output.write('    ' * level)
-        def output_object(table_name, values):
-            indent(1)
-            if 'id' in values:
-                output.write('<%s id="%s">\n' % (table_name, values['id']))
-            else:
-                output.write('<%s>\n' % (table_name,))
-            for key, value in values.items():
-                if key == 'id':
-                    continue
-                indent(2)
-                output.write('<%s>' % (key,))
-                if isinstance (value, unicode):
-                    output.write (value.encode('ascii', 'xmlcharrefreplace'))
-                else:
-                    output.write (str(value))
-                output.write ('</%s>\n' % (key,))
-            indent(1)
-            output.write ('</%s>\n' % (table_name))
-        output.write ('<?xml version="1.0"?>\n')
-        output.write ('<database schema="%d">\n' % (self._schema_version,))
-        for schema in self._object_schemas:
-            self.cursor.execute("SELECT * FROM %s" % schema.table_name)
-            column_names = [d[0] for d in self.cursor.description]
-            for row in self.cursor:
-                output_object(schema.table_name.replace('_', '-'),
-                        dict(zip(column_names, row)))
-        output.write ('</database>\n')
-        output.close()
-
 class SQLiteConverter(object):
     def __init__(self):
         self._to_sql_converters = {}
@@ -932,6 +942,7 @@ class SQLiteConverter(object):
 
         repr_types = (schema.SchemaTimeDelta,
                 schema.SchemaReprContainer,
+                schema.SchemaTuple,
                 schema.SchemaDict,
                 schema.SchemaList,
                 )
@@ -945,7 +956,7 @@ class SQLiteConverter(object):
         # bools get stored as integers in sqlite
         self._from_sql_converters[schema.SchemaBool] = bool
         # filenames are always stored in sqlite as unicode
-        if FilenameType != unicode:
+        if PlatformFilenameType != unicode:
             self._to_sql_converters[schema.SchemaFilename] = filename_to_unicode
             self._from_sql_converters[schema.SchemaFilename] = \
                     self._unicode_to_filename
@@ -996,7 +1007,7 @@ class SQLiteConverter(object):
         filename_fields = schema.SchemaStatusContainer.filename_fields
         for key in filename_fields:
             value = status_dict.get(key)
-            if value is not None and FilenameType != unicode:
+            if value is not None and PlatformFilenameType != unicode:
                 status_dict[key] = self._unicode_to_filename(value)
         return status_dict
 
