@@ -1,7 +1,6 @@
-import rfc822
+import functools
 import os
 import pycurl
-import urllib
 import pickle
 from cStringIO import StringIO
 
@@ -11,10 +10,22 @@ from miro import httpauth
 from miro import httpclient
 from miro import signals
 from miro.plat import resources
+from miro.test import mock
 from miro.test.framework import EventLoopTest, uses_httpclient
 
+from miro.gtcache import gettext as _
+
 TEST_PATH = 'test.txt'
-TEST_BODY = 'Miro HTTP Test\n'
+
+def uses_mock_httpclient(fun):
+    def _uses_mock_httpclient(self):
+        self.mocked_multi = httpclient.curl_manager.multi = mock.Mock()
+        self.mocked_multi.timeout.return_value = -1
+        self.mocked_multi.fdset.return_value = ([], [], [])
+        self.mocked_multi.perform.return_value = (None, None)
+        return fun(self)
+    wrapped = functools.update_wrapper(_uses_mock_httpclient, fun)
+    return uses_httpclient(wrapped)
 
 class HTTPClientTestBase(EventLoopTest):
     def setUp(self):
@@ -23,6 +34,8 @@ class HTTPClientTestBase(EventLoopTest):
         self.grab_url_info = self.grab_url_error = None
         self.expecting_errback = False
         self.event_loop_timeout = 1.0
+        test_txt_path = resources.path("testdata/httpserver/test.txt")
+        self.test_response_data = open(test_txt_path).read()
 
     def grab_url_callback(self, info):
         self.grab_url_info = info
@@ -57,31 +70,37 @@ class HTTPClientTestBase(EventLoopTest):
         self.assertEquals(self.grab_url_error, None)
         self.assertEquals(self.grab_url_info, None)
 
+    def _write_partial_file(self, filename, bytes=30):
+        # Write the start of test.txt to a file to test HTTP resume capability
+        fp = open(filename, 'w')
+        fp.write(self.test_response_data[:bytes])
+        fp.close()
+
 class HTTPClientTest(HTTPClientTestBase):
     @uses_httpclient
     def test_simple_get(self):
         self.grab_url(self.httpserver.build_url('test.txt'))
-        self.assertEquals(self.grab_url_info['body'], TEST_BODY)
+        self.assertEquals(self.grab_url_info['body'], self.test_response_data)
 
     @uses_httpclient
     def test_file_get(self):
         path = resources.path("testdata/httpserver/test.txt")
         self.grab_url("file://" + path)
-        self.assertEquals(self.grab_url_info['body'], TEST_BODY)
+        self.assertEquals(self.grab_url_info['body'], self.test_response_data)
 
     @uses_httpclient
     def test_simple_get_url_with_spaces(self):
         self.grab_url(self.httpserver.build_url('test%20with%20spaces.txt'))
-        self.assertEquals(self.grab_url_info['body'], TEST_BODY)
+        self.assertEquals(self.grab_url_info['body'], self.test_response_data)
         # Having spaces in the URL is not legal, but could happen in the wild.
         # We should work around buggy feeds.
         self.grab_url(self.httpserver.build_url('test with spaces.txt'))
-        self.assertEquals(self.grab_url_info['body'], TEST_BODY)
+        self.assertEquals(self.grab_url_info['body'], self.test_response_data)
 
     @uses_httpclient
     def test_unicode_url(self):
         self.grab_url(unicode(self.httpserver.build_url('test.txt')))
-        self.assertEquals(self.grab_url_info['body'], TEST_BODY)
+        self.assertEquals(self.grab_url_info['body'], self.test_response_data)
 
     def check_header(self, key, value):
         self.assertEqual(self.last_http_info('headers')[key], value)
@@ -112,7 +131,8 @@ class HTTPClientTest(HTTPClientTestBase):
 
     @uses_httpclient
     def test_post_vars(self):
-        self.grab_url(self.httpserver.build_url('test.txt'), post_vars={'abc': '123', 'foo': 'bar'})
+        self.grab_url(self.httpserver.build_url('test.txt'),
+                      post_vars={'abc': '123', 'foo': 'bar'})
         self.assertEqual(self.last_http_info('method'), 'POST')
         post_data = self.last_http_info('post_data')
         self.assertSameSet(post_data.keys(), ['abc', 'foo'])
@@ -129,8 +149,9 @@ class HTTPClientTest(HTTPClientTestBase):
                     'mimetype': 'application/unknown',
                     'handle': StringIO('abc\0\0def')
         }
-        self.grab_url(self.httpserver.build_url('test.txt'), post_vars={'abc': '123'},
-                post_files={'file1': f1, 'file2': f2})
+        self.grab_url(self.httpserver.build_url('test.txt'),
+                      post_vars={'abc': '123'},
+                      post_files={'file1': f1, 'file2': f2})
         post_data = self.last_http_info('post_data')
         self.assertEquals(post_data.getvalue('abc'), '123')
         self.assertEquals(post_data.getvalue('file1'), 'contents#1')
@@ -177,7 +198,8 @@ class HTTPClientTest(HTTPClientTestBase):
 
         def on_headers(headers):
             self.header_callback_info = headers.copy()
-        self.grab_url(self.httpserver.build_url('test.txt'), header_callback=on_headers)
+        self.grab_url(self.httpserver.build_url('test.txt'),
+                      header_callback=on_headers)
         self.assertEquals(self.header_callback_info['x-foo'], 'bar')
         path = resources.path("testdata/httpserver/test.txt")
         self.assertEquals(self.header_callback_info['content-length'],
@@ -219,7 +241,8 @@ class HTTPClientTest(HTTPClientTestBase):
             self.headers = headers
         self.grab_url(self.httpserver.build_url('temp-redirect'),
                 header_callback=header_callback)
-        self.assertEquals(self.headers['content-length'], len(TEST_BODY))
+        self.assertEquals(self.headers['content-length'],
+                len(self.test_response_data))
 
     @uses_httpclient
     def test_circular_redirect_headers(self):
@@ -264,7 +287,6 @@ class HTTPClientTest(HTTPClientTestBase):
         }
         self.event_loop_timeout = 5
         TIMEOUT = 0.001
-        progress_stats = []
 
         self.last_uploaded = None
         self.last_total = None
@@ -309,7 +331,7 @@ class HTTPClientTest(HTTPClientTestBase):
         self.grab_url(self.httpserver.build_url('test.txt'),
                 content_check_callback=check_content)
         self.assertEquals(self.grab_url_error, None)
-        self.assertEquals(self.grab_url_info['body'], TEST_BODY)
+        self.assertEquals(self.grab_url_info['body'], self.test_response_data)
 
     @uses_httpclient
     def test_content_checker_cancel(self):
@@ -357,22 +379,17 @@ class HTTPClientTest(HTTPClientTestBase):
         self.grab_url(self.httpserver.build_url('test.txt'),
                 write_file=filename)
         self.assert_('body' not in self.grab_url_info)
-        self.assertEquals(open(filename).read(), TEST_BODY)
-
-    def _write_partial_file(self, filename):
-        # Write the start of test.txt to a file to test HTTP resume capability
-        fp = open(filename, 'w')
-        fp.write("Miro ")
-        fp.close()
+        self.assertEquals(open(filename).read(), self.test_response_data)
 
     @uses_httpclient
     def test_write_file_resume(self):
         filename = self.make_temp_path(".txt")
-        self._write_partial_file(filename)
+        initial_size = 5
+        self._write_partial_file(filename, initial_size)
         self.grab_url(self.httpserver.build_url('test.txt'),
                 write_file=filename, resume=True)
-        self.assertEquals(open(filename).read(), TEST_BODY)
-        self.check_header('Range', 'bytes=5-')
+        self.assertEquals(open(filename).read(), self.test_response_data)
+        self.check_header('Range', 'bytes=%d-' % initial_size)
 
     @uses_httpclient
     def test_write_file_no_resume(self):
@@ -380,7 +397,7 @@ class HTTPClientTest(HTTPClientTestBase):
         self._write_partial_file(filename)
         self.grab_url(self.httpserver.build_url('test.txt'),
                 write_file=filename, resume=False)
-        self.assertEquals(open(filename).read(), TEST_BODY)
+        self.assertEquals(open(filename).read(), self.test_response_data)
         self.check_header_not_present('Range')
 
     @uses_httpclient
@@ -396,14 +413,14 @@ class HTTPClientTest(HTTPClientTestBase):
     @uses_httpclient
     def test_resume_sizes(self):
         filename = self.make_temp_path(".txt")
-        self._write_partial_file(filename)
+        initial_size = 30
+        self._write_partial_file(filename, initial_size)
         def on_headers(headers):
             self.headers = headers
         self.grab_url(self.httpserver.build_url('test.txt'),
                 write_file=filename, resume=True, header_callback=on_headers)
         path = resources.path("testdata/httpserver/test.txt")
         file_size = len(open(path).read())
-        initial_size = len("Miro ")
         download_size = file_size - initial_size
         self.assertEquals(self.headers['total-size'], file_size)
         self.assertEquals(self.headers['content-length'], file_size -
@@ -416,7 +433,7 @@ class HTTPClientTest(HTTPClientTestBase):
         filename = self.make_temp_path(".txt")
         self.grab_url(self.httpserver.build_url('test.txt'),
                 write_file=filename, resume=True)
-        self.assertEquals(open(filename).read(), TEST_BODY)
+        self.assertEquals(open(filename).read(), self.test_response_data)
  
     @uses_httpclient
     def test_cancel(self):
@@ -506,6 +523,47 @@ class HTTPAuthTest(HTTPClientTestBase):
         self.grab_url(self.httpserver.build_url('protected/index.txt'))
         self.assertEquals(self.dialogs_seen, 1)
 
+    @uses_httpclient
+    def test_auth_with_write_file(self):
+        # check that if the auth fails, we don't write data to our file
+        filename = self.make_temp_path(".txt")
+        self.setup_answer("user", "wrongpassword")
+        self.expecting_errback = True
+        self.grab_url(self.httpserver.build_url('protected/index.txt'),
+                write_file=filename)
+        self.assertEquals(open(filename).read(), "")
+
+    @uses_httpclient
+    def test_auth_failed_with_write_file_resume(self):
+        # check the corner case where are resuming a transfer, but we are
+        # getting back an auth failed response
+        filename = self.make_temp_path(".txt")
+        # write more data than is contained in our auth failed response.  We
+        # should still get a AuthorizationFailed, rather than a ResumeFailed
+        # response.
+        self._write_partial_file(filename, 30)
+        self.setup_answer("user", "wrongpassword")
+        self.expecting_errback = True
+        self.grab_url(self.httpserver.build_url('protected/index.txt'),
+                write_file=filename, resume=True)
+        self.check_auth_errback_called()
+
+    @uses_httpclient
+    def test_auth_success_with_write_file_resume(self):
+        # check this corner case when we are resuming a transfer:
+        #  - we get an auth failed response
+        #  - we then send the correct credentials
+        # In this case, we have to make sure that the initial auth failed
+        # reponse didn't mess up the resume.
+        filename = self.make_temp_path(".txt")
+        # write more data than is contained in our auth failed response.
+        self._write_partial_file(filename, 30)
+        self.setup_answer("user", "password")
+        self.grab_url(self.httpserver.build_url('protected/index.txt'),
+                write_file=filename, resume=True)
+        self.assertEquals(self.dialogs_seen, 1)
+        self.assertEquals(open(filename).read(), self.test_response_data)
+
     @uses_httpclient
     def test_auth_memory(self):
         self.setup_answer("user", "password")
@@ -574,7 +632,8 @@ class HTTPAuthTest(HTTPClientTestBase):
         self.grab_url(self.httpserver.build_url('digest-protected/index2.txt'))
         self.assertEquals(self.dialogs_seen, 1)
         # Even for ones in a subdirectory
-        self.grab_url(self.httpserver.build_url('digest-protected/foo/index2.txt'))
+        self.grab_url(self.httpserver.build_url(
+                'digest-protected/foo/index2.txt'))
         self.assertEquals(self.dialogs_seen, 1)
         # Or even for ones outside the directory (only true for digest auth)
         self.grab_url(self.httpserver.build_url('digest-protected2/index.txt'))
@@ -643,6 +702,37 @@ class HTTPAuthTest(HTTPClientTestBase):
         self.grab_url(self.httpserver.build_url('protected/index.txt'))
         self.assertEquals(self.dialogs_seen, 2)
 
+    @uses_httpclient
+    def test_bad_auth(self):
+        self.setup_answer("user", "password")
+        self.expecting_errback = True
+
+        # test when schemes that aren't "basic" or "digest"
+        self.grab_url(self.httpserver.build_url('invalid-auth/badscheme'))
+        self.assert_(isinstance(self.grab_url_error,
+            httpclient.AuthorizationFailed))
+        # when we get invalid auth headers, we shouldn't pop up dialogs for
+        # the user
+        self.assertEquals(self.dialogs_seen, 0)
+
+        # test auth header without realm
+        self.grab_url(self.httpserver.build_url('invalid-auth/norealm'))
+        self.assert_(isinstance(self.grab_url_error,
+            httpclient.AuthorizationFailed))
+        self.assertEquals(self.dialogs_seen, 0)
+
+        # test completely garbled auth header
+        self.grab_url(self.httpserver.build_url('invalid-auth/garbled'))
+        self.assert_(isinstance(self.grab_url_error,
+            httpclient.AuthorizationFailed))
+        self.assertEquals(self.dialogs_seen, 0)
+
+        # test auth header with no data
+        self.grab_url(self.httpserver.build_url('invalid-auth/'))
+        self.assert_(isinstance(self.grab_url_error,
+            httpclient.AuthorizationFailed))
+        self.assertEquals(self.dialogs_seen, 0)
+
 class HTTPAuthBackendTest(EventLoopTest):
     """ Test using the  httpauth module directly.
 
@@ -787,7 +877,6 @@ class HTTPAuthBackendTestDLDaemonVersion(HTTPAuthBackendTest):
         self.ask_for_http_auth(url, header)
 
         from miro.dl_daemon import daemon
-        from miro.dl_daemon import command
 
         class FakeDaemon(daemon.ControllerDaemon):
             def __init__(self):
@@ -798,7 +887,9 @@ class HTTPAuthBackendTestDLDaemonVersion(HTTPAuthBackendTest):
                 # run the command back and forth through pickle to simulate
                 # sending it over the wire.  This ensures ojects things like
                 # lists are seen as copies rather than the same object (see
-                # UpdatePasswords for an example)
+                # UpdatePasswords for an example).  We could use the copy
+                # module, but I (BDK) wanted to simulate what happens between
+                # the downloader and main process as closely as possible.
                 command = pickle.loads(pickle.dumps(command))
                 command.set_daemon(self)
                 command.action()
@@ -900,12 +991,18 @@ class NetworkErrorTest(HTTPClientTestBase):
         self.grab_url(url)
         self.assert_(isinstance(self.grab_url_error,
             httpclient.UnexpectedStatusCode))
+        # FIXME: It'd be nice if we could check a HTTP code rather than a
+        # static message.
         self.assertEquals(self.grab_url_error.friendlyDescription,
-                "File not found")
+                _("File not found"))
 
-    @uses_httpclient
+    @uses_mock_httpclient
     def test_bad_domain_name(self):
-        self.grab_url('http://unknowndomainname/')
+        def mock_add_handle(handle):
+            error = (handle, pycurl.E_COULDNT_RESOLVE_HOST, 'fake message')
+            self.mocked_multi.info_read.return_value = ([], [], [error])
+        self.mocked_multi.add_handle.side_effect = mock_add_handle
+        self.grab_url('http://pculture.org/')
         self.check_errback_called()
         self.assert_(isinstance(self.grab_url_error,
             httpclient.UnknownHostError))
