@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,6 +33,8 @@ This module contains self-contained utility functions.  It shouldn't import
 any other Miro modules.
 """
 
+import sys
+import itertools
 import os
 import random
 import re
@@ -61,7 +64,7 @@ PREFERRED_TYPES = [
     'video/x-xvid', 'video/x-divx', 'video/x-wmv',
     'video/x-msmpeg', 'video/x-flv']
 
-PREFERRED_TYPES_ORDER = dict((type, i) for i, type in
+PREFERRED_TYPES_ORDER = dict((typ, i) for i, typ in
                              enumerate(PREFERRED_TYPES))
 
 MAX_TORRENT_SIZE = 500 * (2**10) # 500k
@@ -83,8 +86,22 @@ def get_nice_stack():
             stack = stack[:i+1]
             break
 
+    # remove after the call to crashreport.format_crash_report
+    for i in xrange(len(stack)):
+        if ((os.path.basename(stack[i][0]) == 'application.py'
+            and stack[i][2] == 'handle_soft_failure')):
+            stack = stack[:i]
+            break
+
+    # remove after the call to controller.failed_soft
+    for i in xrange(len(stack)):
+        if ((os.path.basename(stack[i][0]) == 'controller.py'
+            and stack[i][2] == 'failed_soft')):
+            stack = stack[:i]
+            break
+
     # remove trap_call calls
-    stack = [i for i in stack if 'trap_call' in i]
+    stack = [i for i in stack if 'trap_call' not in i]
     return stack
 
 
@@ -118,7 +135,8 @@ def read_simple_config_file(path):
         key = match.group(1)
         value = match.group(2)
         if key in ret:
-            print "WARNING: %s: ignored duplicate directive '%s'" % (path, line)
+            print "WARNING: %s: ignored duplicate directive '%s'" % (path,
+                                                                     line)
             continue
 
         ret[key] = value
@@ -204,6 +222,32 @@ class AutoLoggingStream(StringIO):
         if data:
             self.logging_callback(self.prefix + data)
 
+_use_ipv6 = None
+def use_ipv6():
+    """Should we use ipv6 for networking?"""
+
+    global _use_ipv6
+    if _use_ipv6 is None:
+        if socket.has_ipv6:
+            try:
+                socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+                _use_ipv6 = True
+            except socket.error:
+                _use_ipv6 = False
+        else:
+            _use_ipv6 = False
+    return _use_ipv6
+
+def localhost_family_and_addr():
+    """Returns the tuple (family, address) to use for the localhost.
+
+    This method is aware of ipv6 and tries to use it when possible
+    """
+    if use_ipv6():
+        return (socket.AF_INET6, '::1')
+    else:
+        return (socket.AF_INET, '127.0.0.1')
+
 def make_dummy_socket_pair():
     """Create a pair of sockets connected to each other on the local
     interface.  Used to implement SocketHandler.wakeup().
@@ -213,14 +257,17 @@ def make_dummy_socket_pair():
     firewall software.  So if we hit a socketerror with port 0, we
     try ports between 50000 and 65500.
     """
+    attempts = 0
     port = 0
+    family, addr = localhost_family_and_addr()
     while 1:
+        attempts += 1
         try:
-            dummy_server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            dummy_server.bind(("127.0.0.1", port))
+            dummy_server = socket.socket(family, socket.SOCK_STREAM)
+            dummy_server.bind((addr, port))
             dummy_server.listen(1)
             server_address = dummy_server.getsockname()
-            first = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            first = socket.socket(dummy_server.family, socket.SOCK_STREAM)
             first.connect(server_address)
             second, address = dummy_server.accept()
             dummy_server.close()
@@ -228,13 +275,34 @@ def make_dummy_socket_pair():
         except socket.error:
             # if we hit this, then it's hopeless--give up
             if port > 65500:
+
+                # if we're using ipv6 and it's sucking wind, then switch it
+                # off and try ipv4.
+                global _use_ipv6
+                if _use_ipv6:
+                    sys.stdout.write(
+                        "ipv6 is sucking wind, so we're switching to ipv4.")
+                    _use_ipv6 = False
+                    return make_dummy_socket_pair()
+
+                sys.stderr.write(
+                    ("Tried %s bind attempts and failed on "
+                     "addr %s port %d\n") % (
+                        attempts, addr, port))
                 raise
             # bump us into ephemeral ports if we need to try a bunch
             if port == 0:
                 port = 50000
-            port += 10
+            else:
+                port += 10
+
 
 def get_torrent_info_hash(path):
+    """get_torrent_info_hash(path)
+
+    NOTE: Important.  These OS functions can throw IOError or OSError.  Make
+    sure you catch these in the caller.
+    """
     if os.path.getsize(path) > MAX_TORRENT_SIZE:
         # file is too large, bailout.  (see #12301)
         raise ValueError("%s is not a valid torrent" % path)
@@ -242,8 +310,8 @@ def get_torrent_info_hash(path):
     import libtorrent as lt
     f = open(path, 'rb')
     try:
-        data = f.read()
-        if data[0] != 'd':
+        data = f.read(MAX_TORRENT_SIZE)
+        if not data or data[0] != 'd':
             # File doesn't start with 'd', bailout  (see #12301)
             raise ValueError("%s is not a valid torrent" % path)
         metainfo = lt.bdecode(data)
@@ -263,19 +331,30 @@ def gather_media_files(path):
     path -- absolute file path to search
     """
     from miro import prefs
-    from miro import config
+    from miro import app
+    from miro.plat.utils import dirfilt
+
     parsed = 0
     found = []
-    short_app_name = config.get(prefs.SHORT_APP_NAME)
+    short_app_name = app.config.get(prefs.SHORT_APP_NAME)
     for root, dirs, files in os.walk(path):
         for f in files:
             parsed = parsed + 1
-            if filetypes.is_video_filename(f):
+            if (filetypes.is_video_filename(f) or
+                filetypes.is_audio_filename(f)):
                 found.append(os.path.join(root, f))
 
         if short_app_name in dirs:
             dirs.remove(short_app_name)
 
+        # Filter out naughty directories on a platform-specific basis
+        # that we never want to be parsing.  This is mainly useful on 
+        # Mac OS X where we do not want to descend into file packages.  A bit
+        # of a wart, long term solution is to write our own os.walk() shim 
+        # that is platform aware cleanly but right now it's not worth the 
+        # effort.
+        dirfilt(root, dirs)
+
         if parsed > 1000:
             adjusted_parsed = int(parsed / 100.0) * 100
         elif parsed > 100:
@@ -398,6 +477,15 @@ def format_size_for_user(nbytes, zero_string="", with_decimals=True,
         return zero_string
 
 def clamp_text(text, max_length=20):
+    """Clamps text to a maximum length and ...
+
+    :param text: the string to clamp
+    :param max_length: the maximum length for the text--minimum is 4.
+
+    :returns: clamped text
+    """
+    if max_length < 4:
+        max_length = 3
     if len(text) > max_length:
         return text[:max_length-3] + '...'
     else:
@@ -407,7 +495,7 @@ def db_mem_usage_test():
     from miro import models
     from miro.database import DDBObject
     last_usage = get_mem_usage()
-    logging.info("baseline memory usage: %s", last_usage)
+    logging.debug("baseline memory usage: %s", last_usage)
     for name in dir(models):
         ddb_object_class = getattr(models, name)
         try:
@@ -426,13 +514,13 @@ def db_mem_usage_test():
         class_usage = current_usage-last_usage
         if count == 0:
             count = 1 # prevent zero division errors
-        logging.info("memory usage for %s: %s (%d bytes per object)",
+        logging.debug("memory usage for %s: %s (%d bytes per object)",
                 ddb_object_class.__name__, class_usage,
                 class_usage * 1024 / count)
         last_usage = current_usage
-    logging.info("total memory usage: %s", last_usage)
-    logging.info("feed count: %s", models.Feed.make_view().count())
-    logging.info("item count: %s", models.Item.make_view().count())
+    logging.debug("total memory usage: %s", last_usage)
+    logging.debug("feed count: %s", models.Feed.make_view().count())
+    logging.debug("item count: %s", models.Item.make_view().count())
 
 def get_mem_usage():
     return int(call_command('ps', '-o', 'rss', 'hp', str(os.getpid())))
@@ -441,15 +529,20 @@ def setup_logging():
     """Adds TIMING and JSALERT logging levels.
     """
     logging.addLevelName(15, "STACK TRACE")
-    logging.stacktrace = lambda msg, *args, **kargs: logging.log(15, "%s\n%s" % ("".join(traceback.format_stack()), msg) , *args, **kargs)
+    logging.stacktrace = lambda msg, *args, **kargs: logging.log(
+        15, "%s\n%s" % ("".join(traceback.format_stack()), msg),
+        *args, **kargs)
 
     logging.addLevelName(25, "TIMING")
-    logging.timing = lambda msg, *args, **kargs: logging.log(25, msg, *args, **kargs)
+    logging.timing = lambda msg, *args, **kargs: logging.log(25, msg,
+                                                             *args, **kargs)
     logging.addLevelName(26, "JSALERT")
-    logging.jsalert = lambda msg, *args, **kargs: logging.log(26, msg, *args, **kargs)
+    logging.jsalert = lambda msg, *args, **kargs: logging.log(26, msg,
+                                                              *args, **kargs)
 
     logging.addLevelName(21, "DBLOG")
-    logging.dblog = lambda msg, *args, **kargs: logging.log(21, msg, *args, **kargs)
+    logging.dblog = lambda msg, *args, **kargs: logging.log(21, msg,
+                                                            *args, **kargs)
 
 class MiroUnicodeError(StandardError):
     """Returned when input to a template function isn't unicode
@@ -457,7 +550,7 @@ class MiroUnicodeError(StandardError):
     pass
 
 def check_u(text):
-    """Raise an exception if input isn't unicode
+    """Raises an exception if input isn't unicode
     """
     if text is not None and not isinstance(text, unicode):
         raise MiroUnicodeError(u"text %r is not a unicode string (type:%s)" %
@@ -472,16 +565,19 @@ def returns_unicode(func):
         if result is not None:
             check_u(result)
         return result
+    check_func.__name__ = func.__name__
+    check_func.__doc__ = func.__doc__
     return check_func
 
 def check_b(text):
     """Raise an exception if input isn't a binary string
     """
     if text is not None and not isinstance(text, str):
-        raise MiroUnicodeError, (u"text \"%s\" is not a binary string" % text)
+        raise MiroUnicodeError(
+            u"text \"%s\" is not a binary string (%r)" % (text, text))
 
 def returns_binary(func):
-    """Decorator that raised an exception if the function doesn't
+    """Decorator that raises an exception if the function doesn't
     return unicode
     """
     def check_func(*args, **kwargs):
@@ -492,15 +588,31 @@ def returns_binary(func):
     return check_func
 
 def check_f(text):
-    """Returns exception if input isn't a filename type
+    """Raises exception if input isn't a filename type
     """
-    from miro.plat.utils import FilenameType
-    if text is not None and not isinstance(text, FilenameType):
+    from miro.plat.utils import PlatformFilenameType
+    if text is not None and not isinstance(text, PlatformFilenameType):
         raise MiroUnicodeError, (u"text %r is not a valid filename type" %
                                  text)
 
+def returns_file(func):
+    """Decorator that raises an exception if the function doesn't
+    return a filename, file object
+    """
+    def check_func(*args, **kwargs):
+        try:
+            result = func(*args, **kwargs)
+            filename, fileobj = result
+            if result is not None and type(fileobj) != file:
+                raise ValueError('returns_file: not a valid file object')
+            check_f(filename)
+        except ValueError:
+            raise ValueError('returns_file: not a name, fileobj tuple')
+        return result
+    return check_func
+
 def returns_filename(func):
-    """Decorator that raised an exception if the function doesn't
+    """Decorator that raises an exception if the function doesn't
     return a filename
     """
     def check_func(*args, **kwargs):
@@ -566,7 +678,12 @@ def no_console_startupinfo():
     """
     if subprocess.mswindows:
         startupinfo = subprocess.STARTUPINFO()
-        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+        # XXX temporary: STARTF_USESHOWWINDOW is in a different location
+        # as of Python 2.6.6
+        try:
+            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+        except AttributeError:
+            startupinfo.dwFlags |= subprocess._subprocess.STARTF_USESHOWWINDOW
         return startupinfo
     else:
         return None
@@ -575,8 +692,18 @@ def call_command(*args, **kwargs):
     """Call an external command.  If the command doesn't exit with
     status 0, or if it outputs to stderr, an exception will be raised.
     Returns stdout.
+
+    :param ignore_stderr: boolean.  defaults to False.  If True and
+        the command returns stderr, it's ignored rather than raising
+        an exception.
+    :param return_everything: boolean.  defaults to False.  If True,
+        then this returns (retcode, stdout, stderr).  This implies
+        ignore_stderr is True, so you don't need to explicitly state
+        that, too.
     """
     ignore_stderr = kwargs.pop('ignore_stderr', False)
+    return_everything = kwargs.pop('return_everything', False)
+
     if kwargs:
         raise TypeError('extra keyword arguments: %s' % kwargs)
 
@@ -584,7 +711,9 @@ def call_command(*args, **kwargs):
             stdin=subprocess.PIPE, stderr=subprocess.PIPE,
             startupinfo=no_console_startupinfo())
     stdout, stderr = pipe.communicate()
-    if pipe.returncode != 0:
+    if return_everything:
+        return (pipe.returncode, stdout, stderr)
+    elif pipe.returncode != 0:
         raise OSError("call_command with %s has return code %s\n"
                       "stdout:%s\nstderr:%s" %
                       (args, pipe.returncode, stdout, stderr))
@@ -679,11 +808,11 @@ def _to_utf8_bytes(s, encoding=None):
         result = s.encode('utf-8')
     elif not isinstance(s, str):
         s = str(s)
-    if result is None and encoding is not None:
+    if result is None and encoding is not None and not encoding == '':
         # If we knew the encoding of the s, try that.
         try:
             decoded = s.decode(encoding, 'replace')
-        except UnicodeDecodeError:
+        except (UnicodeDecodeError, LookupError):
             pass
         else:
             result = decoded.encode('utf-8')
@@ -939,6 +1068,25 @@ def is_url(url):
         return True
     return False
 
+MAGNET_MATCH_RE = re.compile(r"^magnet:")
+
+def is_magnet_uri(uri):
+    """ Returns true if this is a magnet link
+        which can be handled by Miro.
+    """
+    return MAGNET_MATCH_RE.match(uri) and info_hash_from_magnet(uri)
+
+MAGNET_INFO_HASH_MATCH = re.compile(r'(?<=btih:)[a-zA-Z0-9]+')
+
+def info_hash_from_magnet(uri):
+    """ Returns the magnet URIs bittorrent info hash if it has one.
+    """
+    m = MAGNET_INFO_HASH_MATCH.search(uri)
+    if m:
+        return m.group(0)
+    else:
+       return None
+
 def _strip_accents(text):
     nfkd_form = unicodedata.normalize('NFKD', unicode(text))
     return u"".join([c for c in nfkd_form if not unicodedata.combining(c)])
@@ -957,14 +1105,21 @@ def name_sort_key(text):
 
     * names with accented characters.
     * names with numbers in them.  ex. episode 1
+    * names with a leading "the" or "a"
 
     Use it like this:
 
     >>> listofnames.sort(key=lambda x: name_sort_key(x.name))
     """
     if text is None:
-        return text
-    return [_trynum(c) for c in NUM_RE.split(text.lower())]
+        # put blank entries on the bottom (#17357)
+        return "ZZZZZZZZZZZZZ"
+    text = text.lower()
+    if text.startswith("a "):
+        text = text[2:]
+    elif text.startswith("the "):
+        text = text[4:]
+    return [_trynum(c) for c in NUM_RE.split(text)]
 
 LOWER_TRANSLATE = string.maketrans(string.ascii_uppercase,
                                    string.ascii_lowercase)
@@ -991,3 +1146,41 @@ class DebuggingTimer:
 
     def log_total_time(self):
         logging.timing("total time: %0.3f", clock() - self.start_time)
+
+class Cache(object):
+    def __init__(self, size):
+        self.size = size
+        self.dict = {}
+        self.counter = itertools.count()
+        self.access_times = {}
+
+    def get(self, key):
+        if key in self.dict:
+            self.access_times[key] = self.counter.next()
+            return self.dict[key]
+        else:
+            value = self.create_new_value(key)
+            self.set(key, value)
+            return value
+
+    def set(self, key, value):
+        if len(self.dict) == self.size:
+            self.shrink_size()
+        self.access_times[key] = self.counter.next()
+        self.dict[key] = value
+
+    def shrink_size(self):
+        # shrink by LRU
+        to_sort = self.access_times.items()
+        to_sort.sort(key=lambda m: m[1])
+        new_dict = {}
+        new_access_times = {}
+        latest_times = to_sort[len(self.dict) // 2:]
+        for (key, time) in latest_times:
+            new_dict[key] = self.dict[key]
+            new_access_times[key] = time
+        self.dict = new_dict
+        self.access_times = new_access_times
+
+    def create_new_value(self, val):
+        raise NotImplementedError()
