@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,13 +45,10 @@ import itertools
 from AppKit import *
 from Foundation import *
 from objc import YES, NO, nil, signature, loadBundle
-import WebKit
 
 from miro.plat.frontends.widgets import tableview
 from miro.plat.frontends.widgets import wrappermap
-from miro.plat.frontends.widgets import viewport
 from miro.plat.frontends.widgets.base import Container, Bin, FlippedView
-from miro.plat.frontends.widgets.helpers import NotificationForwarder
 from miro.util import Matrix
 
 rbSplitViewBundlePath = '%s/RBSplitView.framework' % NSBundle.mainBundle().privateFrameworksPath()
@@ -267,7 +265,7 @@ class Alignment(Bin):
         self.bottom_pad = bottom_pad
         self.left_pad = left_pad
         self.right_pad = right_pad
-        if self.child is not None:
+        if self.child is not None and self.viewport is not None:
             self.place_children()
 
     def vertical_pad(self):
@@ -368,7 +366,7 @@ class SplitterDelegate(NSObject):
 class MiroSplitSubview(RBSplitSubview):
     def isFlipped(self):
         return YES
-    
+
 class Splitter(Container):
     """See https://develop.participatoryculture.org/index.php/WidgetAPI for a description of the API for this class."""
     def __init__(self):
@@ -419,10 +417,23 @@ class Splitter(Container):
         if self.left is not None:
             self.left.place(self.left_view.bounds(), self.left_view)
 
+        self._set_subview_min_width(self.left, self.left_view)
+
     def place_right_children(self):
         if self.right is not None:
             self.right.place(self.right_view.bounds(), self.right_view)
 
+        self._set_subview_min_width(self.right, self.right_view)
+
+    def _set_subview_min_width(self, widget, subview):
+        if widget is not None:
+            # constrain the splitter so we honor the size request
+            width, height = widget.get_size_request()
+            min_width = max(width, 1) # handle get_size_request returning -1
+        else:
+            min_width = 1.0
+        subview.setMinDimension_andMaxDimension_(min_width, 0.0)
+
     def place_children(self):
         self.place_left_children()
         self.place_right_children()
@@ -633,11 +644,21 @@ class Scroller(Bin):
         self.view.setHasVerticalScroller_(vertical)
         self.document_view = FlippedView.alloc().init()
         self.view.setDocumentView_(self.document_view)
-        self.view.setAutohidesScrollers_(True)
 
     def set_has_borders(self, has_border):
         self.view.setBorderType_(NSBezelBorder)
 
+    def viewport_repositioned(self):
+        # If the window is resized, this translates to a viewport_repositoned()
+        # event.  So, do whatever super requires of us, then place the 
+        # chilren to work out if we need a scrollbar, then get the new size,
+        # then replace the children (which now takes into account of scrollbar
+        # size.)
+        super(Scroller, self).viewport_repositioned()
+        self.place_children()
+        self.cached_size_request = self.calc_size_request()
+        self.place_children()
+
     def set_background_color(self, color):
         self.view.setBackgroundColor_(self.make_color(color))
 
@@ -649,6 +670,21 @@ class Scroller(Bin):
         child.parent_is_scroller = False
         Bin.remove(self)
 
+    def children_changed(self):
+        # since our size isn't dependent on our children, don't call
+        # invalidate_size_request() here.  Just call place_children() so that
+        # they get positioned correctly in the document view.
+        #
+        # XXX dodgy - why are we laying out the children twice?  When the
+        # children change, the scroller could appear/disappear.  But you have
+        # no idea if that's going to happen without knowing how big your
+        # children are.  So we lay it out, get the size, then, place the 
+        # children again.  This makes sure that the right side of the children
+        # are redrawn.  There's got to be a better way??
+        self.place_children()
+        self.cached_size_request = self.calc_size_request()
+        self.place_children()
+
     def calc_size_request(self):
         if self.child:
             width = height = 0
@@ -658,28 +694,32 @@ class Scroller(Bin):
                 height = self.child.get_size_request()[1]
             # Add a little room for the scrollbars
             if self.view.hasHorizontalScroller():
-                height += 10
+                height += NSScroller.scrollerWidth()
             if self.view.hasVerticalScroller():
-                width += 10
+                width += NSScroller.scrollerWidth()
             return width, height
         else:
             return 0, 0
 
     def place_children(self):
         if self.child is not None:
+            scroll_view_size = self.view.contentView().frame().size
             child_width, child_height = self.child.get_size_request()
-            child_width = max(child_width, self.view.contentView().frame().size.width)
+            child_width = max(child_width, scroll_view_size.width)
+            child_height = max(child_height, scroll_view_size.height)
             frame = NSRect(NSPoint(0,0), NSSize(child_width, child_height))
             if isinstance(self.child, tableview.TableView) and self.child.is_showing_headers():
                 # Hack to allow the content of a table view to scroll, but not
                 # the headers
                 self.child.place(frame, self.document_view)
-                self.view.setDocumentView_(self.child.tableview)
+                if self.view.documentView() is not self.child.tableview:
+                    self.view.setDocumentView_(self.child.tableview)
             else:
                 self.child.place(frame, self.document_view)
             self.document_view.setFrame_(frame)
             self.document_view.setNeedsDisplay_(YES)
         self.view.setNeedsDisplay_(YES)
+        self.child.emit('place-in-scroller')
 
 class ExpanderView(FlippedView):
     def init(self):
