@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,7 +32,6 @@ import time
 from objc import nil, YES, NO, IBOutlet
 from AppKit import *
 from Foundation import *
-from PyObjCTools import AppHelper
 
 from miro import app
 from miro import messages
@@ -75,8 +75,11 @@ class OverlayPalette (NSWindowController):
     keepButton          = IBOutlet('keepButton')
     deleteButton        = IBOutlet('deleteButton')
     fsButton            = IBOutlet('fsButton')
+    fsLabel             = IBOutlet('fsLabel')
     popInOutButton      = IBOutlet('popInOutButton')
     popInOutLabel       = IBOutlet('popInOutLabel')
+    subtitlesButton     = IBOutlet('subtitlesButton')
+    subtitlesLabel      = IBOutlet('subtitlesLabel')
     
     playbackControls    = IBOutlet('playbackControls')
     playPauseButton     = IBOutlet('playPauseButton')
@@ -116,6 +119,10 @@ class OverlayPalette (NSWindowController):
         return self
 
     def awakeFromNib(self):
+        image_path = resources.path('images/subtitles_down.png')
+        self.subtitlesButton.setImage_(NSImage.alloc().initWithContentsOfFile_(image_path))
+        self.subtitlesLabel.setTitleWithMnemonic_(_("Subtitles"))
+        self.fsLabel.setTitleWithMnemonic_(_("Fullscreen"))
         self.shareButton.setImage_(getOverlayButtonImage(self.shareButton.bounds().size))
         self.shareButton.setAlternateImage_(getOverlayButtonAlternateImage(self.shareButton.bounds().size))
         self.shareButton.setTitle_(_("Share"))
@@ -148,6 +155,8 @@ class OverlayPalette (NSWindowController):
     def setup(self, item_info, renderer, video_window):
         from miro.frontends.widgets import widgetutil
         self.item_info = item_info
+        if item_info.remote:
+            self.deleteButton.setEnabled_(False)
         self.renderer = renderer
         self.titleLabel.setStringValue_(item_info.name)
         try:
@@ -155,7 +164,7 @@ class OverlayPalette (NSWindowController):
         except:
             self.feedLabel.setStringValue_("")
         self.keepButton.setEnabled_(item_info.can_be_saved)
-        self.shareButton.setEnabled_(item_info.has_sharable_url)
+        self.shareButton.setEnabled_(item_info.has_shareable_url)
         self.adjustContent(video_window, False)
         self.update_(nil)
         if NSApplication.sharedApplication().isActive() and video_window.isVisible():
@@ -166,7 +175,7 @@ class OverlayPalette (NSWindowController):
         for item_info in changed:
             if item_info.id == self.item_info.id:
                 self.keepButton.setEnabled_(item_info.can_be_saved)
-                self.shareButton.setEnabled_(item_info.has_sharable_url)
+                self.shareButton.setEnabled_(item_info.has_shareable_url)
                 self.update_(nil)
                 
     def enter_fullscreen(self, videoWindow):
@@ -202,8 +211,8 @@ class OverlayPalette (NSWindowController):
         if videoWindow.is_fullscreen:
             self.popInOutButton.setHidden_(YES)
             self.popInOutLabel.setHidden_(YES)
-            self.fsButton.setImage_(NSImage.imageNamed_('fs-button-exitfullscreen'))
-            self.fsButton.setAlternateImage_(NSImage.imageNamed_('fs-button-exitfullscreen-alt'))
+            image_path = resources.path('images/fullscreen_exit.png')
+            self.fsButton.setImage_(NSImage.alloc().initWithContentsOfFile_(image_path))
         else:
             if app.playback_manager.detached_window is None:
                 image_path = resources.path('images/popout.png')
@@ -215,8 +224,8 @@ class OverlayPalette (NSWindowController):
             self.popInOutButton.setHidden_(NO)
             self.popInOutLabel.setHidden_(NO)
             self.popInOutLabel.setStringValue_(label)
-            self.fsButton.setImage_(NSImage.imageNamed_('fs-button-enterfullscreen'))
-            self.fsButton.setAlternateImage_(NSImage.imageNamed_('fs-button-enterfullscreen-alt'))
+            image_path = resources.path('images/fullscreen_enter.png')
+            self.fsButton.setImage_(NSImage.alloc().initWithContentsOfFile_(image_path))
 
         newFrame = self.window().frame() 
         if videoWindow.is_fullscreen or app.playback_manager.detached_window is not None: 
@@ -239,7 +248,7 @@ class OverlayPalette (NSWindowController):
         self.resetAutoHiding()
         if (not self.window().isVisible() and not self.revealing) or (self.window().isVisible() and self.hiding):
             self.update_(nil)
-            if self.renderer.movie is not None:
+            if self.renderer and self.renderer.movie is not None:
                 self.set_volume(self.renderer.movie.volume())
 
             self.adjustPosition(videoWindow)
@@ -326,7 +335,6 @@ class OverlayPalette (NSWindowController):
     
     def expireNow_(self, sender):
         item_info = self.item_info
-        app.playback_manager.on_movie_finished()
         app.widgetapp.remove_items([item_info])
         
     def share_(self, sender):
@@ -368,10 +376,10 @@ class OverlayPalette (NSWindowController):
         app.playback_manager.stop()
 
     def playPause_(self, sender):
-        app.playback_manager.play_pause()
+        app.playback_manager.toggle_paused()
 
     def update_(self, timer):
-        if self.renderer.movie is None:
+        if self.renderer is None or self.renderer.movie is None:
             return
         elapsed = self.renderer.get_elapsed_playback_time()
         total = self.renderer.get_total_playback_time()
