@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +27,6 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-import threading
 import os
 import urllib
 import statvfs
@@ -36,18 +36,21 @@ import sys
 import time
 import errno
 import signal
-import locale
 import subprocess
+import plistlib
 
-from objc import NO, YES, nil
+from objc import nil
 from Foundation import *
 from AppKit import *
 
+from miro import app
 from miro import prefs
-from miro import config
+from miro.gtcache import gettext as _
+from miro.importmedia import import_itunes_path
 from miro.util import returns_unicode, returns_binary, check_u, check_b
-from miro.plat.filenames import (os_filename_to_filename_type,
-                                 filename_type_to_os_filename, FilenameType)
+from miro.plat.filenames import (PlatformFilenameType,
+                                 os_filename_to_filename_type,
+                                 filename_type_to_os_filename)
 from miro.plat.frontends.widgets.threads import on_ui_thread
 
 # We need to define samefile for the portable code.  Lucky for us, this is
@@ -58,6 +61,26 @@ _locale_initialized = False
 
 dlTask = None
 
+def dirfilt(root, dirs):
+    """
+    Platform hook to filter out any directories that should not be
+    descended into, root and dirs corresponds as per os.dirwalk() and
+    same semantics for these objects apply.
+    """
+    removelist = []
+    ws = NSWorkspace.sharedWorkspace()
+    for d in dirs:
+        if ws.isFilePackageAtPath_(os.path.join(root, d)):
+            removelist.append(d)
+    for x in removelist:
+        dirs.remove(x)
+    return dirs
+
+# A file bundle is a collection of files inside a directory.
+def is_file_bundle(path):
+    ws = NSWorkspace.sharedWorkspace()
+    return ws.isFilePackageAtPath_(path)
+
 ###############################################################################
 #### Helper method used to get the free space on the disk where downloaded ####
 #### movies are stored                                                     ####
@@ -66,7 +89,7 @@ dlTask = None
 def get_available_bytes_for_movies():
     pool = NSAutoreleasePool.alloc().init()
     fm = NSFileManager.defaultManager()
-    movies_dir = config.get(prefs.MOVIES_DIRECTORY)
+    movies_dir = app.config.get(prefs.MOVIES_DIRECTORY)
     if not os.path.exists(movies_dir):
         try:
             os.makedirs(movies_dir)
@@ -147,8 +170,8 @@ def setup_logging (in_downloader=False):
         log_name = os.environ.get("DEMOCRACY_DOWNLOADER_LOG")
         level = logging.INFO
     else:
-        log_name = config.get(prefs.LOG_PATHNAME)
-        if config.get(prefs.APP_VERSION).endswith("git"):
+        log_name = app.config.get(prefs.LOG_PATHNAME)
+        if app.config.get(prefs.APP_VERSION).endswith("git"):
             level = logging.DEBUG
         else:
             level = logging.WARN
@@ -172,7 +195,7 @@ def utf8_to_filename(filename):
 # valid byte representation of it attempting to preserve extensions
 #
 # This is not guaranteed to give the same results every time it is run,
-# not is it garanteed to reverse the results of filename_to_unicode
+# not is it guaranteed to reverse the results of filename_to_unicode
 @returns_binary
 def unicode_to_filename(filename, path = None):
     check_u(filename)
@@ -214,7 +237,7 @@ def shortenFilename(filename):
 # Given a filename in raw bytes, return the unicode representation
 #
 # Since this is not guaranteed to give the same results every time it is run,
-# not is it garanteed to reverse the results of unicode_to_filename
+# not is it guaranteed to reverse the results of unicode_to_filename.
 @returns_unicode
 def filename_to_unicode(filename, path = None):
     if path:
@@ -324,9 +347,9 @@ def kill_process(pid):
 def launch_download_daemon(oldpid, env):
     kill_process(oldpid)
 
-    env['DEMOCRACY_DOWNLOADER_LOG'] = config.get(prefs.DOWNLOADER_LOG_PATHNAME)
+    env['DEMOCRACY_DOWNLOADER_LOG'] = app.config.get(prefs.DOWNLOADER_LOG_PATHNAME)
     env['VERSIONER_PYTHON_PREFER_32_BIT'] = "yes"
-    env["MIRO_APP_VERSION"] = config.get(prefs.APP_VERSION)
+    env["MIRO_APP_VERSION"] = app.config.get(prefs.APP_VERSION)
     env.update(os.environ)
 
     exe = NSBundle.mainBundle().executablePath()
@@ -335,11 +358,11 @@ def launch_download_daemon(oldpid, env):
     os_version = int(os_info[2].split('.')[0])
     if os_version < 9:
         launch_path = exe
-        launch_arguments = [u'download_daemon']
+        launch_arguments = [u'--download-daemon']
     else:
         arch = subprocess.Popen("arch", stdout=subprocess.PIPE).communicate()[0].strip()
         launch_path = '/usr/bin/arch'
-        launch_arguments = ['-%s' % arch, exe, u'download_daemon']
+        launch_arguments = ['-%s' % arch, exe, u'--download-daemon']
 
     global dlTask
     dlTask = NSTask.alloc().init()
@@ -351,7 +374,7 @@ def launch_download_daemon(oldpid, env):
     nc = NSNotificationCenter.defaultCenter()
     nc.addObserver_selector_name_object_(controller, 'downloaderDaemonDidTerminate:', NSTaskDidTerminateNotification, dlTask)
 
-    logging.info('Launching Download Daemon')
+    logging.debug('Launching Download Daemon')
     dlTask.launch()
 
 def ensureDownloadDaemonIsTerminated():
@@ -362,7 +385,7 @@ def ensureDownloadDaemonIsTerminated():
     global dlTask
     if dlTask is not None:
         if dlTask.isRunning():
-            logging.info('Waiting for the downloader daemon to terminate...')
+            logging.debug('Waiting for the downloader daemon to terminate...')
             timeout = 5.0
             sleepTime = 0.2
             loopCount = int(timeout / sleepTime)
@@ -374,7 +397,7 @@ def ensureDownloadDaemonIsTerminated():
             else:
                 # If the daemon is still alive at this point, it's likely to be
                 # in a bad state, so nuke it.
-                logging.info("Timeout expired - Killing downloader daemon!")
+                logging.warn("Timeout expired - Killing downloader daemon!")
                 dlTask.terminate()
         dlTask.waitUntilExit()
     dlTask = None
@@ -414,6 +437,26 @@ def movie_data_program_info(movie_path, thumbnail_path):
 
 ###############################################################################
 
+def get_segmenter_executable_path():
+    bundle_path = NSBundle.mainBundle().bundlePath()
+    # XXX Unicode kludge.  This wouldn't be a problem once we switch to 
+    # Python 3.
+    path = os.path.join(bundle_path, "Contents", "Helpers", "miro-segmenter")
+    return path.encode('utf-8')
+
+def get_transcode_video_options():
+    has_video_args = ['-vcodec', 'libx264', '-vpre', 'ipod320',
+                      '-vpre', 'superfast', '-threads', '0', '-s', '480x360',
+                      '-vbsf', 'h264_mp4toannexb']
+    return has_video_args
+
+def get_transcode_audio_options():
+    # XXX: should we use libmp3lame?
+    has_audio_args = ['-acodec', 'aac', '-strict', 'experimental', 
+                      '-threads', '0', '-ab', '96k']
+    #has_audio_args = ['-acodec', 'libmp3lame', '-ab', '160k']
+    return has_audio_args
+
 def setup_ffmpeg_presets():
     os.environ['FFMPEG_DATADIR'] = os.path.join(
             NSBundle.mainBundle().resourcePath().encode('utf-8'),
@@ -472,7 +515,7 @@ def get_logical_cpu_count():
     try:
         import multiprocessing
         return multiprocessing.cpu_count()
-    except ImportError, e:
+    except ImportError:
         try:
             ncpus = os.sysconf("SC_NPROCESSORS_ONLN")
             if isinstance(ncpus, int) and ncpus > 0:
@@ -490,3 +533,67 @@ def begin_thread_loop(context_object):
 
 def finish_thread_loop(context_object):
     del context_object.autorelease_pool
+
+# monkeypatch _dateFromString to support >4 char years because apple.com sets a
+# cookie for Y10K :(
+_origDateFromString = plistlib._dateFromString
+def _dateFromString(s):
+    if s.find('-') > 4: # 10000+ year
+        year, rest = s.split('-', 1)
+        s = '9999-%s' % rest
+    return _origDateFromString(s)
+plistlib._dateFromString = _dateFromString
+
+def _timestamp(dt):
+    try:
+        return str(int(time.mktime(dt.timetuple())))
+    except (ValueError, OverflowError):
+        return '2082787201' # far in the future
+
+def _generate_netscape_cookies(final_path, plist_path):
+    cookies_plist = plistlib.readPlist(plist_path)
+    with file(final_path, 'w') as cookie_file:
+        for cookie in cookies_plist:
+            cookie_file.write('\t'.join([
+                        cookie['Domain'],
+                        'TRUE',
+                        cookie['Path'],
+                        'FALSE',
+                        _timestamp(cookie['Expires']),
+                        cookie['Name'],
+                        cookie['Value']]))
+            cookie_file.write('\n')
+
+
+def get_cookie_path():
+    """
+    Returns the path to a Netscape-style cookie file for Curl to use.
+
+    Nothing is written to this file, but we use the cookies for downloading
+    from Amazon.
+
+    Since OS X doesn't have a real cookies file, we generate one!
+    """
+    final_path = os.path.join(
+        app.config.get(prefs.SUPPORT_DIRECTORY),
+        'cookies.txt')
+    plist_path = os.path.expanduser('~/Library/Cookies/Cookies.plist')
+    if not os.path.exists(plist_path):
+        # nothing to convert
+        return ''
+    if not os.path.exists(final_path) or \
+            os.stat(final_path).st_mtime < os.stat(plist_path).st_mtime:
+        _generate_netscape_cookies(final_path, plist_path)
+    return final_path
+
+def get_plat_media_player_name_path():
+    itunespath = os.path.join(os.path.expanduser("~"), "Music", "iTunes")
+    return (_("iTunes"), import_itunes_path(itunespath))
+
+# Band-aid: you need to create autorelease pool for each thread, we did it
+# for the downloader loop, but not for everything else.  Need an API to
+# properly create threads, and stuff.
+def thread_body(func, *args, **kwargs):
+    pool = NSAutoreleasePool.alloc().init()
+    func(*args, **kwargs)
+    del pool
