@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -62,6 +63,8 @@ PORTABLE_DIR = os.path.join(ROOT_DIR, 'lib')
 PLATFORM_DIR = os.path.join(ROOT_DIR, 'osx')
 PLATFORM_PACKAGE_DIR = os.path.join(PLATFORM_DIR, 'plat')
 
+INFOLIST_DIR = os.path.join(PORTABLE_DIR, 'frontends', 'widgets', 'infolist')
+
 PYTHON_VERSION = sys.version[0:3]
 
 SANDBOX_ROOT_DIR = os.path.normpath(os.path.normpath(os.path.join(ROOT_DIR, '..')))
@@ -268,12 +271,18 @@ class MiroBuild (py2app):
         self.distribution.ext_modules.append(self.get_growl_ext())
         self.distribution.ext_modules.append(self.get_growl_image_ext())
         self.distribution.ext_modules.append(self.get_fasttypes_ext())
+        self.distribution.ext_modules.append(self.get_ngrams_ext())
+        self.distribution.ext_modules.append(self.get_infolist_ext())
 
         self.distribution.packages = [
             'miro',
+            'miro.libdaap',
             'miro.dl_daemon',
             'miro.dl_daemon.private',
             'miro.frontends',
+            'miro.frontends.cli',
+            'miro.frontends.profilewidgets',
+            'miro.frontends.shell',
             'miro.frontends.widgets',
             'miro.plat',
             'miro.plat.frontends',
@@ -297,9 +306,11 @@ class MiroBuild (py2app):
             os.path.join('Resources-Widgets', 'Credits.html'),
             'qt_extractor.py'
         ]
+
         self.resources.extend(glob(os.path.join('Resources-Widgets', '*.png')))
+        self.resources.extend(glob(os.path.join(ROOT_DIR, 'resources', 'images', 'pref_tab*.png')))
         self.resources.extend(glob(os.path.join('Resources-Widgets', 'warmup.mov')))
-    
+
     def setup_options(self):
         self.bundleRoot = os.path.join(self.dist_dir, '%s.app/Contents' % self.config.get('shortAppName'))
         self.rsrcRoot = os.path.join(self.bundleRoot, 'Resources')
@@ -307,7 +318,8 @@ class MiroBuild (py2app):
         self.cmpntRoot = os.path.join(self.bundleRoot, 'Components')
         self.helpersRoot = os.path.join(self.bundleRoot, 'Helpers')
         self.prsrcRoot = os.path.join(self.rsrcRoot, 'resources')
-        
+        self.extRoot = os.path.join(self.rsrcRoot, 'extensions')
+
     def get_idletime_ext(self):
         idletime_src = glob(os.path.join(ROOT_DIR, 'osx', 'modules', 'idletime.c'))
         idletime_link_args = ['-framework', 'CoreFoundation', '-framework', 'IOKit']
@@ -334,8 +346,25 @@ class MiroBuild (py2app):
         return Extension("miro.plat._growlImage", sources=growl_image_src, extra_link_args=growl_image_link_args)
     
     def get_fasttypes_ext(self):
-        return Extension("miro.fasttypes", 
+        return Extension("miro.fasttypes",
                 [os.path.join(PORTABLE_DIR, 'fasttypes.c')])
+
+    def get_ngrams_ext(self):
+        return Extension("miro.ngrams",
+                [os.path.join(PORTABLE_DIR, 'ngrams.c')])
+
+    def get_infolist_ext(self):
+        return Extension("miro.infolist",
+                [
+                    os.path.join(INFOLIST_DIR, 'infolist-nodelist.c'),
+                    os.path.join(INFOLIST_DIR, 'infolist.pyx'),
+                    os.path.join(INFOLIST_DIR, 'cocoa', 'infolist-cocoa.m'),
+                    ],
+                include_dirs = [
+                    INFOLIST_DIR,
+                    ],
+                )
+
     
     def fillTemplate(self, templatepath, outpath, **vars):
         s = open(templatepath, 'rt').read()
@@ -357,6 +386,7 @@ class MiroBuild (py2app):
         self.copy_conversion_helpers()
         self.copy_ffmpeg_presets()
         self.copy_portable_resources()
+        self.copy_extensions()
         self.copy_config_file()
         self.copy_localization_files()
         if self.theme is not None:
@@ -454,6 +484,7 @@ class MiroBuild (py2app):
 
     def copy_conversion_helpers(self):
         print 'Copying video conversion helpers to application bundle'
+        copy_binaries('segmenter/bin/', self.helpersRoot, ["miro-segmenter"])
         copy_binaries('ffmpeg/bin/', self.helpersRoot, ["ffmpeg"])
         copy_binaries('ffmpeg2theora/bin/', self.helpersRoot, ["ffmpeg2theora"])
 
@@ -472,7 +503,7 @@ class MiroBuild (py2app):
         if not os.path.exists(self.prsrcRoot):
             os.mkdir(self.prsrcRoot)
 
-        resources = ['searchengines', 'images', 'conversions']
+        resources = ['searchengines', 'images', 'conversions', 'devices']
         if self.keep_tests:
             resources.append('testdata')
 
@@ -491,6 +522,21 @@ class MiroBuild (py2app):
                 print "    %s" % dest
                 copy(src, dest)
 
+    def copy_extensions(self):
+        print "Copying extensions to application bundle"
+
+        if self.force_update and os.path.exists(self.extRoot):
+            shutil.rmtree(self.extRoot, True)
+
+        src = os.path.join(ROOT_DIR, 'extensions')
+        dest = self.extRoot
+
+        if os.path.exists(dest):
+            print "    (extensions skipped, already bundled)"
+        else:
+            print "    %s" % dest
+            shutil.copytree(src, dest)
+
     def copy_config_file(self):
         print "Copying config file to application bundle"
         shutil.move(self.appConfigPath, os.path.join(self.prsrcRoot, 'app.config'))
