@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,15 +32,16 @@ import logging
 import os
 import traceback
 import urllib
-import logging
+
+vlc_logger = logging.getLogger('vlc')
 
 import gtk
 import gobject
 
 from miro.gtcache import gettext as _
 from miro.plat import resources
+from miro.plat.renderers import vlchack
 from miro import app
-from miro import config
 from miro import prefs
 from miro.frontends.widgets import menus
 from miro.frontends.widgets.widgetconst import MAX_VOLUME
@@ -135,7 +137,7 @@ STOPPED, PAUSED, PLAYING = range(3)
 
 class VLCSniffer(object):
     def __init__(self):
-        plugin_dir = os.path.join(resources.appRoot(), 'vlc-plugins')
+        plugin_dir = os.path.join(resources.app_root(), 'vlc-plugins')
         self.exc = VLCException()
 
         # Note: if you need vlc output to stdout, remove the --quiet
@@ -165,7 +167,7 @@ class VLCSniffer(object):
         self.exc.check()
 
     def shutdown(self):
-        logging.info("shutting down VLC Sniffer")
+        vlc_logger.info("shutting down VLC Sniffer")
         libvlc.libvlc_media_player_release(self.media_player)
         libvlc.libvlc_release(self.vlc)
 
@@ -181,7 +183,7 @@ class VLCSniffer(object):
 
     def handle_event(self, obj, type_, state, arg2):
         if type_ != libvlc_MediaStateChanged:
-            return 
+            return
         if obj != self.media_playing:
             return
         if self.callback_info is None:
@@ -201,8 +203,10 @@ class VLCSniffer(object):
                 self._open_success()
 
     def _open_success(self):
-        # FIXME - sometimes _open_success is called, but callback_info
-        # is None.  not sure why this happens.
+        # bz:17471 - sometimes _open_success is called, but callback_info
+        # is None.  The reason is due to multiple gobject.timeout_add()
+        # or gobject.idle_add() leading to callback_info being executed
+        # after which it is set to None.
         item_type = "failure"
         if self.callback_info:
             video_tracks = libvlc.libvlc_video_get_track_count(
@@ -228,8 +232,10 @@ class VLCSniffer(object):
             libvlc.libvlc_media_player_stop(self.media_player, self.exc.ref())
             self.exc.check()
         except VLCError, vlce:
-            logging.warning("sniffer reset failed: %s", vlce)
-        self.callback_info[0](item_type)
+            vlc_logger.warning("sniffer reset failed: %s", vlce)
+        # bz:17471
+        if self.callback_info:
+            self.callback_info[0](item_type)
         self.callback_info = None
         self.media_playing = None
 
@@ -238,8 +244,10 @@ class VLCSniffer(object):
             libvlc.libvlc_media_player_stop(self.media_player, self.exc.ref())
             self.exc.check()
         except VLCError, vlce:
-            logging.warning("sniffer reset failed: %s", vlce)
-        self.callback_info[1]()
+            vlc_logger.warning("sniffer reset failed: %s", vlce)
+        # bz:17471
+        if self.callback_info:
+            self.callback_info[1]()
         self.callback_info = None
         self.media_playing = None
 
@@ -247,6 +255,12 @@ class VLCSniffer(object):
         """starts playing the specified file"""
         filename = iteminfo.video_path
 
+        # Convert to URL if necessary
+        try:
+            filename = iteminfo.video_path.urlize()
+        except AttributeError:
+            filename = iteminfo.video_path
+
         # filenames coming in are unicode objects, VLC expects utf-8
         # strings.
         filename = filename.encode('utf-8')
@@ -254,23 +268,24 @@ class VLCSniffer(object):
         self.callback_info = (success_callback, error_callback)
         self.play_state = STOPPED
 
-        media = libvlc.libvlc_media_new(self.vlc, ctypes.c_char_p(filename),
+        media = libvlc.libvlc_media_new(self.vlc,
+                ctypes.c_char_p(filename),
                 self.exc.ref())
         self.exc.check()
         if media is None:
             raise AssertionError(
                 "libvlc_media_new returned NULL for %s" % filename)
-        event_manager = libvlc.libvlc_media_event_manager(media, 
+        event_manager = libvlc.libvlc_media_event_manager(media,
                                                           self.exc.ref())
         self.exc.check()
-        libvlc.libvlc_event_attach(event_manager, 
+        libvlc.libvlc_event_attach(event_manager,
                                    libvlc_MediaStateChanged,
-                                   self._callback_ref, 
-                                   None, 
+                                   self._callback_ref,
+                                   None,
                                    self.exc.ref())
         self.exc.check()
         try:
-            libvlc.libvlc_media_player_set_media(self.media_player, 
+            libvlc.libvlc_media_player_set_media(self.media_player,
                                                  media,
                                                  self.exc.ref())
             self.exc.check()
@@ -288,8 +303,8 @@ class VLCSniffer(object):
 
 class VLCRenderer(object):
     def __init__(self):
-        logging.info("Initializing VLC")
-        plugin_dir = os.path.join(resources.appRoot(), 'vlc-plugins')
+        vlc_logger.info("Initializing VLC")
+        plugin_dir = os.path.join(resources.app_root(), 'vlc-plugins')
         self.exc = VLCException()
 
         # Note: if you need vlc output to stdout, remove the --quiet
@@ -308,6 +323,7 @@ class VLCRenderer(object):
         self._callback_ref = VLC_EVENT_CALLBACK(self.event_callback)
         self.play_from_time = None
         self.play_state = STOPPED
+        self.hwnd = None
         self._duration = None
         self._filename = None
         self._rate = 1.0
@@ -322,7 +338,7 @@ class VLCRenderer(object):
         self.unset_widget()
 
     def shutdown(self):
-        logging.info("shutting down VLC")
+        vlc_logger.info("shutting down VLC")
         self.reset()
         libvlc.libvlc_media_player_release(self.media_player)
         vlc_object_release(self.vlc_instance)
@@ -343,7 +359,7 @@ class VLCRenderer(object):
         if type_ == libvlc_MediaStateChanged:
             self._handle_state_change(obj, arg1)
         else:
-            logging.warn("Unknown VLC event type: %s", type_)
+            vlc_logger.warning("Unknown VLC event type: %s", type_)
 
     def _handle_state_change(self, obj, state):
         if obj != self.media_playing:
@@ -387,18 +403,27 @@ class VLCRenderer(object):
             gobject.timeout_add(500, self._length_check, attempt+1)
 
     def _open_success(self):
-        # FIXME - sometimes _open_success is called, but callback_info
-        # is None.  not sure why this happens.
+        # bz:17471 - sometimes _open_success is called, but callback_info
+        # is None.  The reason is due to multiple gobject.timeout_add()
+        # or gobject.idle_add() leading to callback_info being executed
+        # after which it is set to None.
+        if not self.callback_info:
+            vlc_logger.warning("_open_success: callback_info is None")
+            return
         self.setup_subtitles()
-        if self.callback_info:
-            self.callback_info[0]()
+        callback = self.callback_info[0]
         self.callback_info = None
+        callback()
 
     def _open_failure(self):
-        logging.info("_open_failure\n%s", "".join(traceback.format_stack()))
-        self.callback_info[1]()
-        self.callback_info = None
+        vlc_logger.warning(
+            "_open_failure\n%s", "".join(traceback.format_stack()))
         self.media_playing = None
+        # bz:17471
+        if self.callback_info:
+            errback = self.callback_info[1]
+            self.callback_info = None
+            errback()
 
     def _disable_video(self):
         desc = libvlc.libvlc_video_get_track_description(
@@ -413,16 +438,17 @@ class VLCRenderer(object):
             self.exc.check()
 
     def set_widget(self, widget):
-        hwnd = widget.persistent_window.handle
-        libvlc.libvlc_media_player_set_hwnd(self.media_player, hwnd,
+        self.hwnd = widget.persistent_window.handle
+        libvlc.libvlc_media_player_set_hwnd(self.media_player, self.hwnd,
                                                 self.exc.ref())
         self.exc.check()
 
         widget.add_events(gtk.gdk.EXPOSURE_MASK)
         widget.connect('expose-event', self._on_expose)
-        EnableWindow(hwnd, 0)
+        EnableWindow(self.hwnd, 0)
 
     def unset_widget(self):
+        self.hwnd = None
         libvlc.libvlc_media_player_set_hwnd(
             self.media_player, self._hidden_window.handle, self.exc.ref())
         self.exc.check()
@@ -430,12 +456,17 @@ class VLCRenderer(object):
     def _on_expose(self, widget, event):
         gc = widget.style.black_gc
         widget.persistent_window.draw_rectangle(
-            gc, True, event.area.x, event.area.y, 
+            gc, True, event.area.x, event.area.y,
             event.area.width, event.area.height)
 
     def select_file(self, iteminfo, callback, errback):
         """starts playing the specified file"""
-        filename = iteminfo.video_path
+
+        # Convert to URL if necessary
+        try:
+            filename = iteminfo.video_path.urlize()
+        except AttributeError:
+            filename = iteminfo.video_path
 
         # filenames coming in are unicode objects, VLC expects utf-8
         # strings.
@@ -447,13 +478,14 @@ class VLCRenderer(object):
         self.play_from_time = None
         self.play_state = STOPPED
 
-        media = libvlc.libvlc_media_new(self.vlc, ctypes.c_char_p(filename),
+        media = libvlc.libvlc_media_new(self.vlc,
+                                        ctypes.c_char_p(filename),
                                         self.exc.ref())
         self.exc.check()
         if media is None:
             raise AssertionError(
                 "libvlc_media_new returned NULL for %s" % filename)
-        event_manager = libvlc.libvlc_media_event_manager(media, 
+        event_manager = libvlc.libvlc_media_event_manager(media,
                                                           self.exc.ref())
         self.exc.check()
         libvlc.libvlc_event_attach(event_manager, libvlc_MediaStateChanged,
@@ -474,15 +506,18 @@ class VLCRenderer(object):
         self.exc.check()
         # For unknown reasons, sometimes we don't see the state
         # changed event if they happen quickly enough.  To work around
-        # that, check the initial state of the media player.
+        # that, check the initial state of the media player.  However, do that
+        # in an idle callback, so that we don't call the errback before
+        # returning from setup()
         state = libvlc.libvlc_media_player_get_state(self.media_player,
                 self.exc.ref())
         self.exc.check()
-        self._handle_state_change(self.media_playing, state)
+        gobject.idle_add(self._handle_state_change, self.media_playing, state)
 
     def play(self):
         if self.play_state == PLAYING:
             return
+        gobject.timeout_add(500, self._vlc_hack)
         libvlc.libvlc_media_player_play(self.media_player, self.exc.ref())
         self.exc.check()
         self.play_state = PLAYING
@@ -490,6 +525,11 @@ class VLCRenderer(object):
             self.set_current_time(self.play_from_time)
             self.play_from_time = None
 
+    def _vlc_hack(self):
+        if self.play_state == PLAYING:
+            vlchack.hack_window(self.hwnd)
+        return False
+
     def pause(self):
         if self.play_state == PAUSED:
             return
@@ -512,13 +552,18 @@ class VLCRenderer(object):
         self.play_from_time = None
         self.play_state = STOPPED
 
+
     def get_current_time(self):
+        if self.play_state != PLAYING and self.play_from_time is not None:
+            # We're paused and waiting for the next play() call before we
+            # seek.  Return the time we will seek to
+            return self.play_from_time
         t = libvlc.libvlc_media_player_get_time(
             self.media_player, self.exc.ref())
         try:
             self.exc.check()
         except VLCError, e:
-            logging.warn("exception getting time: %s" % e)
+            vlc_logger.warning("exception getting time: %s" % e)
             return None
 
         return t / 1000.0
@@ -528,16 +573,12 @@ class VLCRenderer(object):
             self.play_from_time = seconds
             return
         t = int(seconds * 1000)
-        if t == 0:
-            # I have no clue why this this is, but setting time=1
-            # (1/1000th of a second) fixes #15079
-            t = 1
         libvlc.libvlc_media_player_set_time(
             self.media_player, ctypes.c_longlong(t), self.exc.ref())
         try:
             self.exc.check()
         except VLCError, e:
-            logging.warn("exception setting current time %s" % e)
+            vlc_logger.warning("exception setting current time %s" % e)
 
     def get_duration(self):
         # self._duration = (filename, duration)
@@ -549,7 +590,7 @@ class VLCRenderer(object):
         try:
             self.exc.check()
         except VLCError, e:
-            logging.warn("exception getting duration: %s" % e)
+            vlc_logger.warning("exception getting duration: %s" % e)
             return None
 
         self._duration = (self._filename, length / 1000.0)
@@ -566,16 +607,15 @@ class VLCRenderer(object):
         return rv / 100.0
 
     def set_rate(self, rate):
-        logging.info("set_rate: rate %s", rate)
         if self._rate == rate:
             return
         self._rate = rate
-        libvlc.libvlc_media_player_set_rate(self.media_player, 
+        libvlc.libvlc_media_player_set_rate(self.media_player,
                 ctypes.c_float(rate), self.exc.ref())
         try:
             self.exc.check()
         except VLCError, e:
-            logging.warn("exception setting rate: %s" % e)
+            vlc_logger.warning("exception setting rate: %s" % e)
             return None
 
     def get_rate(self):
@@ -583,7 +623,7 @@ class VLCRenderer(object):
 
     def setup_subtitles(self):
         self.setup_subtitle_info()
-        if config.get(prefs.ENABLE_SUBTITLES):
+        if app.config.get(prefs.ENABLE_SUBTITLES):
             track_index = self.get_enabled_subtitle_track()
             if track_index == 0:
                 count = libvlc.libvlc_video_get_spu_count(
@@ -594,12 +634,13 @@ class VLCRenderer(object):
             self.disable_subtitles()
 
     def setup_subtitle_font(self):
-        font_path = config.get(prefs.SUBTITLE_FONT)
-        config_PutPsz(self.vlc_instance,
-                ctypes.c_char_p('freetype-font'),
-                ctypes.c_char_p(font_path))
-        logging.info("Setting VLC subtitle font: %s", font_path)
-        
+        font_path = app.config.get(prefs.SUBTITLE_FONT)
+        if font_path is not None:
+            config_PutPsz(self.vlc_instance,
+                          ctypes.c_char_p('freetype-font'),
+                          ctypes.c_char_p(font_path))
+            vlc_logger.info("Setting VLC subtitle font: %s", font_path)
+
     def get_subtitle_tracks(self):
         return self.subtitle_info
 
@@ -614,12 +655,15 @@ class VLCRenderer(object):
             self.exc.check()
             first_desc = desc
             for i in range(0, count):
-                if i > 0: # track 0 is "disabled", don't include it
+                # track 0 is "disabled", don't include it
+                if i > 0:
                     self.subtitle_info.append((i, desc.contents.name))
                 desc = desc.contents.next
             libvlc.libvlc_track_description_release(first_desc)
-        except VLCError, e:
-            logging.warn("exception when getting list of subtitle tracks")
+        except VLCError:
+            vlc_logger.warning(
+                "setup_subtitle_info: exception when getting list "
+                "of subtitle tracks")
 
     def get_enabled_subtitle_track(self):
         track_index = libvlc.libvlc_video_get_spu(
@@ -627,7 +671,7 @@ class VLCRenderer(object):
         try:
             self.exc.check()
         except VLCError, e:
-            logging.warn("exception when getting enabled subtitle track")
+            vlc_logger.warning("exception when getting enabled subtitle track")
             return None
         return track_index
 
@@ -639,7 +683,7 @@ class VLCRenderer(object):
 
     def disable_subtitles(self):
         self._set_active_subtitle_track(0)
-        
+
     def _set_active_subtitle_track(self, track_index):
         count = libvlc.libvlc_video_get_spu_count(
             self.media_player, self.exc.ref())
@@ -651,15 +695,16 @@ class VLCRenderer(object):
             return
 
         if track_index >= count:
-            logging.warn("Subtitle track too high: %s (count: %s)",
+            vlc_logger.warning("Subtitle track too high: %s (count: %s)",
                     track_index, count)
 
-        libvlc.libvlc_video_set_spu(self.media_player, track_index, 
+        libvlc.libvlc_video_set_spu(self.media_player, track_index,
                                     self.exc.ref())
         try:
             self.exc.check()
         except VLCError, e:
-            logging.warn("exception when setting subtitle track: %s", e)
+            vlc_logger.warning(
+                "exception when setting subtitle track: %s", e)
 
     def select_subtitle_file(self, item, sub_path, handle_successful_select):
         try:
@@ -670,7 +715,7 @@ class VLCRenderer(object):
             # for VLC to pick it up on the next playback forcing the
             # user to select it again.
             # This is bug 12813.
-            logging.exception("exception thrown when copying subtitle file")
+            vlc_logger.exception("exception thrown when copying subtitle file")
 
         sub_path = sub_path.encode('utf-8')
         res = libvlc.libvlc_video_set_subtitle_file(
@@ -678,7 +723,8 @@ class VLCRenderer(object):
         try:
             self.exc.check()
         except VLCError, e:
-            logging.warn("exception when setting subtitle track to file: %s", e)
+            vlc_logger.warn(
+                "exception when setting subtitle track to file: %s", e)
         else:
             handle_successful_select()
             # 1 is the track of the external file, don't select it quite yet
@@ -694,12 +740,11 @@ class VLCRenderer(object):
     def select_subtitle_encoding(self, encoding):
         if self.media_playing is None:
             return
-        if encoding is None:
-            encoding = ''
-        config_PutPsz(self.vlc_instance,
-                ctypes.c_char_p('subsdec-encoding'),
-                ctypes.c_char_p(encoding))
-        logging.info("Setting VLC subtitle encoding: %s", encoding)
+        if encoding:
+            config_PutPsz(self.vlc_instance,
+                          ctypes.c_char_p('subsdec-encoding'),
+                          ctypes.c_char_p(encoding))
+            vlc_logger.info("Setting VLC subtitle encoding: %s", encoding)
 
     def setup_subtitle_encoding_menu(self, menubar):
         menus.add_subtitle_encoding_menu(menubar, _('Eastern European'),
