@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,7 +39,7 @@ import subprocess
 import zipfile as zip
 from glob import glob, iglob
 from xml.sax.saxutils import escape
-from distutils import sysconfig 
+from distutils import sysconfig
 from distutils.core import Command
 import distutils.command.install_data
 from distutils.ccompiler import new_compiler
@@ -67,7 +68,7 @@ BINARY_KIT_ROOT = "miro-binary-kit-win-%s" % BINARY_KIT_VERSION
 
 if not os.path.exists or not os.path.isdir(BINARY_KIT_ROOT):
     print "Binary kit %s is missing.  Run 'setup_binarykit.sh'." % BINARY_KIT_ROOT
-    sys.exit()
+    sys.exit(1)
 
 ZLIB_INCLUDE_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'include')
 ZLIB_LIB_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'lib')
@@ -93,6 +94,7 @@ GTK_INCLUDE_DIRS = [
 ]
 
 PYGOBJECT_INCLUDE_DIR = os.path.join(BINARY_KIT_ROOT, 'pygobject')
+PYGTK_INCLUDE_DIR = os.path.join(BINARY_KIT_ROOT, 'pygtk')
 
 # path to the Mozilla "xulrunner-sdk" distribution.
 XULRUNNER_SDK_PATH = os.path.join(BINARY_KIT_ROOT, 'xulrunner-sdk')
@@ -100,11 +102,14 @@ XULRUNNER_SDK_BIN_PATH = os.path.join(XULRUNNER_SDK_PATH, 'bin')
 
 VLC_PATH = os.path.join(BINARY_KIT_ROOT, 'libvlc')
 LIBTORRENT_PATH = os.path.join(BINARY_KIT_ROOT, 'libtorrent')
+MUTAGEN_PATH = os.path.join(BINARY_KIT_ROOT, 'mutagen-1.20')
 
+SEGMENTER_PATH = os.path.join(BINARY_KIT_ROOT, 'segmenter')
 FFMPEG_PATH = os.path.join(
-    BINARY_KIT_ROOT, 'ffmpeg', 'ffmpeg-r25766-swscale-r32562-mingw32-static')
+    BINARY_KIT_ROOT, 'ffmpeg', 'ffmpeg-r25766-swscale-r32562-mingw32-shared')
 FFMPEG2THEORA_PATH = os.path.join(BINARY_KIT_ROOT, 'ffmpeg2theora')
 
+VCREDIST71_PATH = os.path.join(BINARY_KIT_ROOT, 'vc71redist')
 VCREDIST90_PATH = os.path.join(BINARY_KIT_ROOT, 'vc90redist')
 
 def find_data_files(dest_path_base, source_path):
@@ -149,9 +154,11 @@ platform_package_dir = os.path.join(platform_dir, 'plat')
 widgets_dir = os.path.join(platform_package_dir, 'frontends', 'widgets')
 portable_dir = os.path.join(root_dir, 'lib')
 portable_widgets_dir = os.path.join(portable_dir, 'frontends', 'widgets')
+infolist_dir = os.path.join(portable_widgets_dir, 'infolist')
 portable_xpcom_dir = os.path.join(portable_widgets_dir, 'gtk', 'xpcom')
 test_dir = os.path.join(root_dir, 'resources')
 resources_dir = os.path.join(root_dir, 'resources')
+extensions_dir = os.path.join(root_dir, 'extensions')
 
 sys.path.insert(0, root_dir)
 # when we install the portable modules, they will be in the miro
@@ -165,8 +172,13 @@ from miro import util
 # put the libtorrent extension on the path, so py2exe can find the
 # .pyd file
 sys.path.insert(0, LIBTORRENT_PATH)
+sys.path.insert(0, MUTAGEN_PATH)
 
 #### Extensions ####
+ngrams_ext = \
+    Extension("miro.ngrams",
+        [os.path.join(portable_dir, 'ngrams.c')],
+    )
 
 pygtkhacks_ext = Extension(
     "miro.frontends.widgets.gtk.pygtkhacks",
@@ -177,6 +189,7 @@ pygtkhacks_ext = Extension(
     library_dirs=[GTK_LIB_PATH],
     libraries=[
         'gtk-win32-2.0',
+        'pango-1.0',
         ]
     )
 
@@ -190,7 +203,7 @@ xulrunnerbrowser_ext = Extension(
         portable_xpcom_dir,
         ] + GTK_INCLUDE_DIRS,
     define_macros=[
-        ("XP_WIN", 1), 
+        ("XP_WIN", 1),
         ("XPCOM_GLUE", 1),
         ("PCF_USING_XULRUNNER19", 1),
         ],
@@ -214,20 +227,45 @@ xulrunnerbrowser_ext = Extension(
         os.path.join(portable_xpcom_dir, 'HttpObserver.cc'),
         os.path.join(xulrunnerbrowser_ext_dir, 'MiroBrowserEmbed.cpp'),
         os.path.join(xulrunnerbrowser_ext_dir, 'MiroWindowCreator.cpp'),
+        os.path.join(xulrunnerbrowser_ext_dir, 'MiroDirectoryProvider.cpp'),
         os.path.join(xulrunnerbrowser_ext_dir, 'FixFocus.cpp'),
         os.path.join(xulrunnerbrowser_ext_dir, 'Init.cpp'),
         ]
     )
 
+infolist_ext = \
+    Extension("miro.infolist",
+        [
+            os.path.join(infolist_dir, 'infolist-nodelist.c'),
+            os.path.join(infolist_dir, 'infolist.pyx'),
+            os.path.join(infolist_dir, 'gtk', 'infolist-gtk.c'),
+        ],
+        include_dirs=GTK_INCLUDE_DIRS + [
+            PYGOBJECT_INCLUDE_DIR,
+            PYGTK_INCLUDE_DIR,
+            infolist_dir,
+            os.path.join(infolist_dir, 'gtk'),
+        ],
+        library_dirs=[GTK_LIB_PATH],
+        libraries=[
+            'gobject-2.0',
+            'glib-2.0',
+            'gtk-win32-2.0',
+            'pango-1.0',
+            ],
+    )
+
 # Setting the path here allows py2exe to find the DLLS
 os.environ['PATH'] = ';'.join([
-        OPENSSL_LIB_PATH, ZLIB_RUNTIME_LIBRARY_PATH, 
+        OPENSSL_LIB_PATH, ZLIB_RUNTIME_LIBRARY_PATH,
         LIBTORRENT_PATH, GTK_BIN_PATH, os.environ['PATH']])
 
 # Private extension modules to build.
 ext_modules = [
+    ngrams_ext,
     pygtkhacks_ext,
     xulrunnerbrowser_ext,
+    infolist_ext,
 ]
 
 def fill_template(templatepath, outpath, **vars):
@@ -255,21 +293,33 @@ data_files.append(('', [os.path.join(VLC_PATH, 'libvlc.dll')]))
 data_files.append(('', [os.path.join(VLC_PATH, 'libvlccore.dll')]))
 data_files.append(('', [os.path.join(LIBTORRENT_PATH, 'libtorrent.pyd')]))
 data_files.append(('', [
+            os.path.join(SEGMENTER_PATH, 'miro-segmenter.exe'),
             os.path.join(FFMPEG_PATH, 'bin', 'ffmpeg.exe'),
+            os.path.join(FFMPEG_PATH, 'bin', 'avcodec-52.dll'),
+            os.path.join(FFMPEG_PATH, 'bin', 'avcore-0.dll'),
+            os.path.join(FFMPEG_PATH, 'bin', 'avdevice-52.dll'),
+            os.path.join(FFMPEG_PATH, 'bin', 'avfilter-1.dll'),
+            os.path.join(FFMPEG_PATH, 'bin', 'avformat-52.dll'),
+            os.path.join(FFMPEG_PATH, 'bin', 'avutil-50.dll'),
+            os.path.join(FFMPEG_PATH, 'bin', 'swscale-0.dll'),
             os.path.join(FFMPEG2THEORA_PATH, 'bin', 'ffmpeg2theora.exe')]))
 data_files.append(('', glob(os.path.join(FFMPEG_PATH, 'presets', '*.ffpreset'))))
+data_files.append(('', glob(os.path.join(VCREDIST71_PATH, '*.dll'))))
 data_files.extend(find_data_files('Microsoft.VC90.CRT',
     os.path.join(VCREDIST90_PATH, 'Microsoft.VC90.CRT')))
+data_files.append((os.path.join('etc', 'gtk-2.0'), [
+            os.path.join(os.path.dirname(__file__), 'gtkrc')]))
 
 # handle the resources subdirectories.
-for dir in ('searchengines', 'images', 'conversions'):
+for dir in ('searchengines', 'images', 'conversions', 'devices'):
     dest_dir = os.path.join('resources', dir)
     source_dir = os.path.join(resources_dir, dir)
     data_files.extend(find_data_files(dest_dir, source_dir))
 
-data_files.append(('resources', [os.path.join(root_dir, 'ADOPTERS')]))
 data_files.append(('resources', [os.path.join(root_dir, 'CREDITS')]))
 
+data_files.extend(find_data_files("extensions", extensions_dir))
+
 locale_temp_dir = os.path.join(os.path.dirname(__file__), "build", "locale")
 
 def copy_locale_files():
@@ -297,7 +347,7 @@ template_vars = util.read_simple_config_file(app_config)
 
 # pixmap for the about dialog
 icon_path = os.path.join("icons", "hicolor", "128x128", "apps")
-data_files.append((os.path.join("resources", icon_path), 
+data_files.append((os.path.join("resources", icon_path),
                    [os.path.join(platform_dir, icon_path, "miro.png")]))
 
 ###########################################################################
@@ -307,7 +357,6 @@ class install_data(distutils.command.install_data.install_data):
     """install_data extends to default implementation so that it
     automatically installs app.config from app.config.template.
     """
-
     def install_app_config(self):
         template = os.path.join(resources_dir, 'app.config.template')
         dest = os.path.join(self.install_dir, 'resources', 'app.config')
@@ -371,10 +420,12 @@ py2exe.build_exe.isSystemDLL = isSystemDLL
 class bdist_miro(Command):
     description = "Build Miro"
 
-    user_options = []
+    user_options = [
+        ('test', None, 'Install test bits.')
+        ]
 
     def initialize_options(self):
-        pass
+        self.test = False
 
     def finalize_options(self):
         pass
@@ -382,27 +433,8 @@ class bdist_miro(Command):
     def run(self):
         self.run_command('py2exe')
         self.copy_ico()
-
-    def copy_ico(self):
-        dist_dir = self.get_finalized_command('py2exe').dist_dir
-        shortappname = template_vars["shortAppName"]
-        self.copy_file("Miro.ico", 
-                       os.path.join(dist_dir, "%s.ico" % shortappname))
-
-class bdist_test(Command):
-    description = "Builds Miro with unit tests"
-
-    user_options = []
-
-    def initialize_options(self):
-        pass
-
-    def finalize_options(self):
-        pass
-
-    def run(self):
-        self.run_command('bdist_miro')
-        self.copy_test_data()
+        if self.test:
+            self.copy_test_data()
 
     def copy_test_data(self):
         # copy test data over
@@ -411,6 +443,12 @@ class bdist_test(Command):
         self.copy_tree(os.path.join(resources_dir, 'testdata'),
                        os.path.join(dist_dir, 'resources', 'testdata'))
 
+    def copy_ico(self):
+        dist_dir = self.get_finalized_command('py2exe').dist_dir
+        shortappname = template_vars["shortAppName"]
+        self.copy_file("Miro.ico",
+                       os.path.join(dist_dir, "%s.ico" % shortappname))
+
 class runmiro(Command):
     description = "build Miro and start it up"
 
@@ -504,8 +542,8 @@ class bdist_nsis(Command):
             output_file = "%s-nozugo" % output_file
         if self.mozilla:
             output_file = '%s-mozilla' % output_file
-            
-        output_file = (output_file % 
+
+        output_file = (output_file %
                        (template_vars['shortAppName'], template_vars['appVersion']))
         nsis_vars['CONFIG_OUTPUT_FILE'] = '%s.exe' % output_file
         nsis_vars['CONFIG_TWOSTAGE'] = "No"
@@ -524,7 +562,7 @@ class bdist_nsis(Command):
             output_file = '%s-%s-generic-twostage.exe'
         else:
             output_file = "%s-%s-twostage.exe"
-        output_file = (output_file % 
+        output_file = (output_file %
                        (template_vars['shortAppName'], template_vars['appVersion']))
         nsis_vars['CONFIG_OUTPUT_FILE'] = output_file
         nsis_vars['CONFIG_TWOSTAGE'] = "Yes"
@@ -557,7 +595,7 @@ class bdist_nsis(Command):
         for filename in iglob(wildcard):
             if filename[:length] == self.dist_dir:
                 filename = filename[length:]
-                while (len(filename) > 0 
+                while (len(filename) > 0
                        and (filename[0] == '/' or filename[0] == '\\')):
                     filename = filename[1:]
             print "Compressing %s" % filename
@@ -567,7 +605,7 @@ class bdist_nsis(Command):
         length = len(self.dist_dir)
         if filename[:length] == self.dist_dir:
             filename = filename[length:]
-            while (len(filename) > 0 
+            while (len(filename) > 0
                    and (filename[0] == '/' or filename[0] == '\\')):
                 filename = filename[1:]
         print "Compressing %s" % filename
@@ -593,20 +631,22 @@ if __name__ == "__main__":
                 },
             ],
         console=[
+            {
+                # runs miro from the console--for debugging
+                'script': 'Miro.py',
+                'dest_base': 'mirodebug',
+                'icon_resources': [(0, "Miro.ico")],
+                },
             {
                 'script': 'moviedata_util.py',
                 'dest_base': '%s_MovieData' % template_vars['shortAppName'],
                 'icon_resources': [(0, "Miro.ico")],
                 },
-            {
-                'script': 'mirotest.py',
-                'dest_base': 'mirotest',
-                'icon_resources': [(0, "Miro.ico")],
-                }
             ],
         ext_modules=ext_modules,
         packages=[
             'miro',
+            'miro.libdaap',
             'miro.dl_daemon',
             'miro.dl_daemon.private',
             'miro.frontends',
@@ -628,15 +668,15 @@ if __name__ == "__main__":
             'install_data': install_data,
             'bdist_miro': bdist_miro,
             'bdist_nsis': bdist_nsis,
-            'bdist_test': bdist_test,
             'runmiro': runmiro,
             },
         options={
             'py2exe': {
                 'packages': [
-                    'encodings',
+                    'encodings'
                     ],
-                'includes': 'cairo, pango, pangocairo, atk, gobject, gio, libtorrent',
+                'includes': ('cairo, pango, pangocairo, atk, gobject, '
+                             'gio, libtorrent, mutagen'),
                 },
             },
         )
